
/*
 * engine-3d v0.13.0
 * (c) 2018 @cocos
 * Released under the MIT License.
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

// NOTE: you must have `_prev` and `_next` field in the object returns by `fn`

var LinkedArray = function LinkedArray() {
  this._count = 0;
  this._head = null;
  this._tail = null;
};

var prototypeAccessors = { head: { configurable: true },tail: { configurable: true },length: { configurable: true } };

prototypeAccessors.head.get = function () {
  return this._head;
};

prototypeAccessors.tail.get = function () {
  return this._tail;
};

prototypeAccessors.length.get = function () {
  var cursor = this._head;
  var next = cursor;
  var count = 0;
  while (cursor) {
    next = cursor._next;
    count++;
    cursor = next;
  }
  return count;
};

LinkedArray.prototype.add = function add (node) {
  if (!this._tail) {
    this._head = node;
  } else {
    this._tail._next = node;
    node._prev = this._tail;
  }
  this._tail = node;
  this._count += 1;
};

LinkedArray.prototype.remove = function remove (node) {
    var this$1 = this;

  var cursor = this._head;
  var next = cursor;

  while (cursor) {
    next = cursor._next;

    if (cursor === node) {
      if (node._prev) {
        node._prev._next = node._next;
      } else {
        this$1._head = node._next;
      }

      if (node._next) {
        node._next._prev = node._prev;
      } else {
        this$1._tail = node._prev;
      }

      node._next = null;
      node._prev = null;
      break;
    }
    cursor = next;
  }
  this._count -= 1;
};

LinkedArray.prototype.forEach = function forEach (fn, binder) {
    var this$1 = this;

  var cursor = this._head;
  if (!cursor) {
    return;
  }

  if (binder) {
    fn = fn.bind(binder);
  }

  var idx = 0;
  var next = cursor;

  while (cursor) {
    next = cursor._next;
    fn(cursor, idx, this$1);

    cursor = next;
    ++idx;
  }
};

Object.defineProperties( LinkedArray.prototype, prototypeAccessors );

/**
 * @ignore
 */
var _d2r = Math.PI / 180.0;
/**
 * @ignore
 */
var _r2d = 180.0 / Math.PI;

/**
 * @property {number} EPSILON
 */
var EPSILON = 0.000001;

/**
 * Tests whether or not the arguments have approximately the same value, within an absolute
 * or relative tolerance of glMatrix.EPSILON (an absolute tolerance is used for values less
 * than or equal to 1.0, and a relative tolerance is used for larger values)
 *
 * @param {Number} a The first number to test.
 * @param {Number} b The second number to test.
 * @returns {Boolean} True if the numbers are approximately equal, false otherwise.
 */


/**
 * Tests whether or not the arguments have approximately the same value by given maxDiff
 *
 * @param {Number} a The first number to test.
 * @param {Number} b The second number to test.
 * @param {Number} maxDiff Maximum difference.
 * @returns {Boolean} True if the numbers are approximately equal, false otherwise.
 */


/**
 * Clamps a value between a minimum float and maximum float value.
 *
 * @method clamp
 * @param {number} val
 * @param {number} min
 * @param {number} max
 * @return {number}
 */
function clamp(val, min, max) {
  return val < min ? min : val > max ? max : val;
}

/**
 * Clamps a value between 0 and 1.
 *
 * @method clamp01
 * @param {number} val
 * @return {number}
 */


/**
 * @method lerp
 * @param {number} from
 * @param {number} to
 * @param {number} ratio - the interpolation coefficient
 * @return {number}
 */


/**
* Convert Degree To Radian
*
* @param {Number} a Angle in Degrees
*/


/**
* Convert Radian To Degree
*
* @param {Number} a Angle in Radian
*/


/**
* @method random
*/
var random = Math.random;

/**
 * Returns a floating-point random number between min (inclusive) and max (exclusive).
 *
 * @method randomRange
 * @param {number} min
 * @param {number} max
 * @return {number} the random number
 */


/**
 * Returns a random integer between min (inclusive) and max (exclusive).
 *
 * @method randomRangeInt
 * @param {number} min
 * @param {number} max
 * @return {number} the random integer
 */


/**
 * Linear congruential generator using Hull-Dobell Theorem.
 *
 * @method pseudoRandom
 * @param {number} seed the random seed
 * @return {number} the pseudo random
 */


/**
 * Returns a floating-point pseudo-random number between min (inclusive) and max (exclusive).
 *
 * @method pseudoRandomRange
 * @param {number} seed
 * @param {number} min
 * @param {number} max
 * @return {number} the random number
 */


/**
 * Returns a pseudo-random integer between min (inclusive) and max (exclusive).
 *
 * @method pseudoRandomRangeInt
 * @param {number} seed
 * @param {number} min
 * @param {number} max
 * @return {number} the random integer
 */


/**
 * Returns the next power of two for the value
 *
 * @method nextPow2
 * @param {number} val
 * @return {number} the the next power of two
 */


/**
 * Returns float remainder for t / length
 *
 * @method repeat
 * @param {number} t time start at 0
 * @param {number} length time of one cycle
 * @return {number} the time wrapped in the first cycle
 */


/**
 * Returns time wrapped in ping-pong mode
 *
 * @method repeat
 * @param {number} t time start at 0
 * @param {number} length time of one cycle
 * @return {number} the time wrapped in the first cycle
 */


/**
 * Returns ratio of a value within a given range
 *
 * @method repeat
 * @param {number} from start value
 * @param {number} to end value
 * @param {number} value given value
 * @return {number} the ratio between [from,to]
 */

/**
 * Bit twiddling hacks for JavaScript.
 *
 * Author: Mikola Lysenko
 *
 * Ported from Stanford bit twiddling hack library:
 *    http://graphics.stanford.edu/~seander/bithacks.html
 */

// Number of bits in an integer
/**
 * @ignore
 */

/**
 * @ignore
 */

/**
 * @ignore
 */


/**
 * Returns -1, 0, +1 depending on sign of x
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes absolute value of integer
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes minimum of integers x and y
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Computes maximum of integers x and y
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Checks if a number is a power of two
 *
 * @param {number} v
 * @returns {boolean}
 */


/**
 * Computes log base 2 of v
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes log base 10 of v
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Counts number of bits
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Counts number of trailing zeros
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Rounds to next power of 2
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Rounds down to previous power of 2
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes parity of word
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * @ignore
 */
var REVERSE_TABLE = new Array(256);

(function(tab) {
  for(var i=0; i<256; ++i) {
    var v = i, r = i, s = 7;
    for (v >>>= 1; v; v >>>= 1) {
      r <<= 1;
      r |= v & 1;
      --s;
    }
    tab[i] = (r << s) & 0xff;
  }
})(REVERSE_TABLE);

/**
 * Reverse bits in a 32 bit word
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Interleave bits of 2 coordinates with 16 bits. Useful for fast quadtree codes
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Extracts the nth interleaved component
 *
 * @param {number} v
 * @param {number} n
 * @returns {number}
 */


/**
 * Interleave bits of 3 coordinates, each with 10 bits.  Useful for fast octree codes
 *
 * @param {number} x
 * @param {number} y
 * @param {number} z
 * @returns {number}
 */


/**
 * Extracts nth interleaved component of a 3-tuple
 *
 * @param {number} v
 * @param {number} n
 * @returns {number}
 */


/**
 * Computes next combination in colexicographic order (this is mistakenly called nextPermutation on the bit twiddling hacks page)
 *
 * @param {number} v
 * @returns {number}
 */

/**
 * Mathematical 2-dimensional vector.
 *
 * x, y is alias of the first, second component of vector, respectively.
 */
var vec2 = function vec2(x, y)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
};

/**
 * Creates a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @return {vec2} The newly created vector.
 */
vec2.new = function new$1 (x, y)
{
  return new vec2(x, y);
};

/**
 * Creates a zero vector.
 *
 * @return {vec2} The newly created vector.
 */
vec2.zero = function zero ()
{
  return new vec2(0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec2} a - Vector to clone.
 * @returns {vec2} The newly created vector.
 */
vec2.clone = function clone (a)
{
  return new vec2(a.x, a.y);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec2} out - The vector to modified.
 * @param {vec2} a - The specified vector.
 * @returns {vec2} out.
 */
vec2.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec2} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @returns {vec2} out.
 */
vec2.set = function set (out, x, y)
{
  out.x = x;
  out.y = y;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  return out;
};

/**
 *Alias of {@link vec2.subtract}.
 */
vec2.sub = function sub (out, a, b)
{
  return vec2.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  return out;
};

/**
 *Alias of {@link vec2.multiply}.
 */
vec2.mul = function mul (out, a, b)
{
  return vec2.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  return out;
};

/**
 *Alias of {@link vec2.divide}.
 */
vec2.div = function div (out, a, b)
{
  return vec2.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec2} out.
 * */
vec2.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec2} out.
 */
vec2.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec2.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y;
  return Math.sqrt(x * x + y * y);
};

/**
 *Alias of {@link vec2.distance}.
 */
vec2.dist = function dist (a, b)
{
  return vec2.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec2.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y;
  return x * x + y * y;
};

/**
 *Alias of {@link vec2.squaredDistance}.
 */
vec2.sqrDist = function sqrDist (a, b)
{
  return vec2.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec2.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y;
  return Math.sqrt(x * x + y * y);
};

/**
 *Alias of {@link vec2.magnitude}.
 */
vec2.mag = function mag (a)
{
  return vec2.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec2.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y;
  return x * x + y * y;
};

/**
 *Alias of {@link vec2.squaredMagnitude}
 */
vec2.sqrMag = function sqrMag (a)
{
  return vec2.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to negate.
 * @returns {vec2} out.
 */
vec2.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  return out;
};

/**
 * Invert the components of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to invert.
 * @returns {vec2} out.
 */
vec2.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  return out;
};

/**
 * Safely invert the components of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to invert.
 * @returns {vec2} out.
 */
vec2.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / a.y;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to normalize.
 * @returns {vec2} out.
 */
vec2.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y;
  var len = x * x + y * y;
  if (len > 0)
  {
    //TODO: evaluate use of glm_invsqrt here?
    len = 1 / Math.sqrt(len);
    out.x = a.x * len;
    out.y = a.y * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec2.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y;
};

/**
 * Calculate the cross product of two vectors.
 * Note that the cross product must by definition produce a 3D vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec3} out.
 */
vec2.cross = function cross (out, a, b)
{
  var z = a.x * b.y - a.y * b.x;
  out.x = out.y = 0;
  out.z = z;
  return out;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec2} out.
 */
vec2.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec2} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec2} out.
 */
vec2.random = function random$1 (out, scale)
{
  scale = scale || 1.0;
  var r = random() * 2.0 * Math.PI;
  out.x = Math.cos(r) * scale;
  out.y = Math.sin(r) * scale;
  return out;
};

/**
 * Transforms a vector with a 2x2 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat2} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat2 = function transformMat2 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m02 * y;
  out.y = m.m01 * x + m.m03 * y;
  return out;
};

/**
 * Transforms a vector with a 2x3 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat23} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat23 = function transformMat23 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m02 * y + m.m04;
  out.y = m.m01 * x + m.m03 * y + m.m05;
  return out;
};

/**
 * Transforms a vector with a 3x3 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat3} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat3 = function transformMat3 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m03 * y + m.m06;
  out.y = m.m01 * x + m.m04 * y + m.m07;
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix.
 * 3rd vector component is implicitly '0'.
 * 4th vector component is implicitly '1'.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m04 * y + m.m12;
  out.y = m.m01 * x + m.m05 * y + m.m13;
  return out;
};

/**
 * Perform some operation over an array of vec2s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec2.forEach = function forEach (a, stride, offset, count, fn, arg)
{
  return vec2._forEach(a, stride, offset, count, fn, arg);
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec2.str = function str (a)
{
  return ("vec2(" + (a.x) + ", " + (a.y) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec2} v - The vector.
 * @returns {Array} out.
 */
vec2.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec2} a - The first vector.
 * @param {vec2} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec2.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec2} a The first vector.
 * @param {vec2} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec2.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y;
  var b0 = b.x, b1 = b.y;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)));
};

/**
 * Perform some operation over an array of vec2s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 * @ignore
 */
vec2._forEach = (function ()
{
  var vec = vec2.zero();

  return function (a, stride, offset, count, fn, arg)
  {
    var i, l;
    if (!stride)
    {
      stride = 2;
    }

    if (!offset)
    {
      offset = 0;
    }

    if (count)
    {
      l = Math.min((count * stride) + offset, a.length);
    } else
    {
      l = a.length;
    }

    for (i = offset; i < l; i += stride)
    {
      vec.x = a[i];
      vec.y = a[i + 1];
      fn(vec, vec, arg);
      a[i] = vec.x;
      a[i + 1] = vec.y;
    }

    return a;
  };
})();

/**
 * Mathematical 3-dimensional vector.
 *
 * x, y, z is alias of the first, second, third component of vector, respectively.
 */
var vec3 = function vec3(x, y, z)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
};

/**
 * Creates a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @return {vec3} The newly created vector.
 */
vec3.new = function new$1 (x, y, z)
{
  return new vec3(x, y, z);
};

/**
 * Creates a zero vector.
 *
 * @return {vec3} The newly created vector.
 */
vec3.zero = function zero ()
{
  return vec3.new(0, 0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec3} a - Vector to clone.
 * @returns {vec3} The newly created vector.
 */
vec3.clone = function clone (a)
{
  return new vec3(a.x, a.y, a.z);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec3} out - The vector to modified.
 * @param {vec3} a - The specified vector.
 * @returns {vec3} out.
 */
vec3.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  out.z = a.z;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec3} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @returns {vec3} out.
 */
vec3.set = function set (out, x, y, z)
{
  out.x = x;
  out.y = y;
  out.z = z;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  out.z = a.z + b.z;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  out.z = a.z - b.z;
  return out;
};

/**
 * Alias of {@link vec3.subtract}.
 */
vec3.sub = function sub (out, a, b)
{
	return vec3.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  out.z = a.z * b.z;
  return out;
};

/**
 * Alias of {@link vec3.multiply}.
 */
vec3.mul = function mul (out, a, b)
{
	return vec3.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  out.z = a.z / b.z;
  return out;
};

/**
 * Alias of {@link vec3.divide}.
 */
vec3.div = function div (out, a, b)
{
	return vec3.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  out.z = Math.ceil(a.z);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  out.z = Math.floor(a.z);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  out.z = Math.min(a.z, b.z);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  out.z = Math.max(a.z, b.z);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  out.z = Math.round(a.z);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec3} out.
 * */
vec3.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec3} out.
 */
vec3.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  out.z = a.z + (b.z * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec3.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z;
  return Math.sqrt(x * x + y * y + z * z);
};

/**
 * Alias of {@link vec3.distance}.
 */
vec3.dist = function dist (a, b)
{
	return vec3.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec3.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z;
  return x * x + y * y + z * z;
};

/**
 * Alias of {@link vec3.squaredDistance}.
 */
vec3.sqrDist = function sqrDist (a, b)
{
	return vec3.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec3.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z;
  return Math.sqrt(x * x + y * y + z * z);
};

/**
 *Alias of {@link vec3.magnitude}.
 */
vec3.mag = function mag (a)
{
  return vec3.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec3.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z;
  return x * x + y * y + z * z;
};

/**
 *Alias of {@link vec3.squaredMagnitude}
 */
vec3.sqrMag = function sqrMag (a)
{
  return vec3.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to negate.
 * @returns {vec3} out.
 */
vec3.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  return out;
};

/**
 * Inverts the components of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to invert.
 * @returns {vec3} out.
 */
vec3.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  out.z = 1.0 / a.z;
  return out;
};

/**
 * Safely inverts the components of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to invert.
 * @returns {vec3} out.
 */
vec3.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / y;
  }

  if (Math.abs(z) < EPSILON)
  {
    out.z = 0;
  } else
  {
    out.z = 1.0 / z;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to normalize.
 * @returns {vec3} out.
 */
vec3.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z;

  var len = x * x + y * y + z * z;
  if (len > 0)
  {
    //TODO: evaluate use of glm_invsqrt here?
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec3.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z;
};

/**
 * Calculates the cross product of two vectors.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.cross = function cross (out, a, b)
{
  var ax = a.x, ay = a.y, az = a.z,
    bx = b.x, by = b.y, bz = b.z;

  out.x = ay * bz - az * by;
  out.y = az * bx - ax * bz;
  out.z = ax * by - ay * bx;
  return out;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  return out;
};

/**
 * Performs a hermite interpolation with two control points.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {vec3} c - The third operand.
 * @param {vec3} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.hermite = function hermite (out, a, b, c, d, t)
{
  var factorTimes2 = t * t,
    factor1 = factorTimes2 * (2 * t - 3) + 1,
    factor2 = factorTimes2 * (t - 2) + t,
    factor3 = factorTimes2 * (t - 1),
    factor4 = factorTimes2 * (3 - 2 * t);

  out.x = a.x * factor1 + b.x * factor2 + c.x * factor3 + d.x * factor4;
  out.y = a.y * factor1 + b.y * factor2 + c.y * factor3 + d.y * factor4;
  out.z = a.z * factor1 + b.z * factor2 + c.z * factor3 + d.z * factor4;

  return out;
};

/**
 * Performs a bezier interpolation with two control points.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {vec3} c - The third operand.
 * @param {vec3} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.bezier = function bezier (out, a, b, c, d, t)
{
  var inverseFactor = 1 - t,
    inverseFactorTimesTwo = inverseFactor * inverseFactor,
    factorTimes2 = t * t,
    factor1 = inverseFactorTimesTwo * inverseFactor,
    factor2 = 3 * t * inverseFactorTimesTwo,
    factor3 = 3 * factorTimes2 * inverseFactor,
    factor4 = factorTimes2 * t;

  out.x = a.x * factor1 + b.x * factor2 + c.x * factor3 + d.x * factor4;
  out.y = a.y * factor1 + b.y * factor2 + c.y * factor3 + d.y * factor4;
  out.z = a.z * factor1 + b.z * factor2 + c.z * factor3 + d.z * factor4;

  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec3} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec3} out.
 */
vec3.random = function random$1 (out, scale)
{
  scale = scale || 1.0;

  var r = random() * 2.0 * Math.PI;
  var z = (random() * 2.0) - 1.0;
  var zScale = Math.sqrt(1.0 - z * z) * scale;

  out.x = Math.cos(r) * zScale;
  out.y = Math.sin(r) * zScale;
  out.z = z * scale;
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix. 4th vector component is implicitly '1'.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec3} out.
 */
vec3.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z,
    w = m.m03 * x + m.m07 * y + m.m11 * z + m.m15;
  w = w || 1.0;
  out.x = (m.m00 * x + m.m04 * y + m.m08 * z + m.m12) / w;
  out.y = (m.m01 * x + m.m05 * y + m.m09 * z + m.m13) / w;
  out.z = (m.m02 * x + m.m06 * y + m.m10 * z + m.m14) / w;
  return out;
};

/**
 * Transforms a vector with a 3x3 matrix.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {mat3} m - The matrix.
 * @returns {vec3} out.
 */
vec3.transformMat3 = function transformMat3 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z;
  out.x = x * m.m00 + y * m.m03 + z * m.m06;
  out.y = x * m.m01 + y * m.m04 + z * m.m07;
  out.z = x * m.m02 + y * m.m05 + z * m.m08;
  return out;
};

/**
 * Transforms a vector with a quaternion.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {quat} q - The quaternion.
 * @returns {vec3} out.
 */
vec3.transformQuat = function transformQuat (out, a, q)
{
  // benchmarks: http://jsperf.com/quaternion-transform-vec3-implementations

  var x = a.x, y = a.y, z = a.z;
  var qx = q.x, qy = q.y, qz = q.z, qw = q.w;

  // calculate quat * vec
  var ix = qw * x + qy * z - qz * y;
  var iy = qw * y + qz * x - qx * z;
  var iz = qw * z + qx * y - qy * x;
  var iw = -qx * x - qy * y - qz * z;

  // calculate result * inverse quat
  out.x = ix * qw + iw * -qx + iy * -qz - iz * -qy;
  out.y = iy * qw + iw * -qy + iz * -qx - ix * -qz;
  out.z = iz * qw + iw * -qz + ix * -qy - iy * -qx;
  return out;
};

/**
 * Rotates a 3D vector around the x-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateX = function rotateX (out, a, b, c)
{
  var p = [], r = [];
  // Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.x;
  r.y = p.y * Math.cos(c) - p.z * Math.sin(c);
  r.z = p.y * Math.sin(c) + p.z * Math.cos(c);

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Rotates a 3D vector around the y-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateY = function rotateY (out, a, b, c)
{
  var p = [], r = [];
  //Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.z * Math.sin(c) + p.x * Math.cos(c);
  r.y = p.y;
  r.z = p.z * Math.cos(c) - p.x * Math.sin(c);

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Rotates a 3D vector around the z-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateZ = function rotateZ (out, a, b, c)
{
  var p = [], r = [];
  //Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.x * Math.cos(c) - p.y * Math.sin(c);
  r.y = p.x * Math.sin(c) + p.y * Math.cos(c);
  r.z = p.z;

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec3.str = function str (a)
{
  return ("vec3(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec3} v - The vector.
 * @returns {Array} out.
 */
vec3.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;
  out[2] = v.z;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec3} a - The first vector.
 * @param {vec3} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec3.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y && a.z === b.z;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec3} a The first vector.
 * @param {vec3} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec3.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y, a2 = a.z;
  var b0 = b.x, b1 = b.y, b2 = b.z;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
    Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)));
};
  
/**
 * Perform some operation over an array of vec3s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec3.forEach = function forEach (a, stride, offset, count, fn, arg)
{
	return vec3._forEach(a, stride, offset, count, fn, arg);
};
  
/**
 * Gets the angle between two 3D vectors.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {Number} - The angle in radians.
 */
vec3.angle = function angle (a, b)
{
	return vec3._angle(a, b);
};

/**
* Perform some operation over an array of vec3s.
*
* @param {Array} a the array of vectors to iterate over.
* @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed.
* @param {Number} offset Number of elements to skip at the beginning of the array.
* @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array.
* @param {Function} fn Function to call for each vector in the array.
* @param {Object} [arg] additional argument to pass to fn.
* @returns {Array} a.
* @ignore.
*/
vec3._forEach = (function ()
{
var vec = vec3.zero();

return function (a, stride, offset, count, fn, arg)
{
  var i, l;
  if (!stride)
  {
	stride = 3;
  }

  if (!offset)
  {
	offset = 0;
  }

  if (count)
  {
	l = Math.min((count * stride) + offset, a.length);
  } else
  {
	l = a.length;
  }

  for (i = offset; i < l; i += stride)
  {
	vec.x = a[i];
	vec.y = a[i + 1];
	vec.z = a[i + 2];
	fn(vec, vec, arg);
	a[i] = vec.x;
	a[i + 1] = vec.y;
	a[i + 2] = vec.z;
  }

  return a;
};
})();

  /**
   * Get the angle between two 3D vectors.
   * @param {vec3} a The first operand.
   * @param {vec3} b The second operand.
   * @returns {Number} The angle in radians.
   * @ignore
   */
vec3._angle = (function ()
{
var tempA = vec3.zero();
var tempB = vec3.zero();

return function (a, b)
{
  vec3.copy(tempA, a);
  vec3.copy(tempB, b);

  vec3.normalize(tempA, tempA);
  vec3.normalize(tempB, tempB);

  var cosine = vec3.dot(tempA, tempB);

  if (cosine > 1.0)
  {
	return 0;
  }

  if (cosine < -1.0)
  {
	return Math.PI;
  }

  return Math.acos(cosine);
};
})();

/**
 * Mathematical 4-dimensional vector.
 *
 * x, y, z, w is alias of the first, second, third, fourth component of vector, respectively.
 */
var vec4 = function vec4(x, y, z, w)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
  /**
   * The w component.
   * @type {number}
   * */
  this.w = w;
};

/**
 * Create a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @param {number} w - Value assigned to w component.
 * @return {vec4} The newly created vector.
 */
vec4.new = function new$1 (x, y, z, w)
{
  return new vec4(x, y, z, w);
};

/**
 * Creates a zero vector.
 *
 * @return {vec4} The newly created vector.
 */
vec4.zero = function zero ()
{
  return vec4.new(0, 0, 0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec4} a - Vector to clone.
 * @returns {vec4} The newly created vector.
 */
vec4.clone = function clone (a)
{
  return new vec4(a.x, a.y, a.z, a.w);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec4} out - The vector to modified.
 * @param {vec4} a - The specified vector.
 * @returns {vec4} out.
 */
vec4.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  out.z = a.z;
  out.w = a.w;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec4} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @param {Number} w - Value set to w component.
 * @returns {vec4} out.
 */
vec4.set = function set (out, x, y, z, w)
{
  out.x = x;
  out.y = y;
  out.z = z;
  out.w = w;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  out.z = a.z + b.z;
  out.w = a.w + b.w;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  out.z = a.z - b.z;
  out.w = a.w - b.w;
  return out;
};

/**
 * Alias of {@link vec4.subtract}.
 */
vec4.sub = function sub (out, a, b)
{
  return vec4.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  out.z = a.z * b.z;
  out.w = a.w * b.w;
  return out;
};

/**
 * Alias of {@link vec4.multiply}.
 */
vec4.mul = function mul (out, a, b)
{
  return vec4.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  out.z = a.z / b.z;
  out.w = a.w / b.w;
  return out;
};

/**
 * Alias of {@link vec4.divide}.
 */
vec4.div = function div (out, a, b)
{
  return vec4.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  out.z = Math.ceil(a.z);
  out.w = Math.ceil(a.w);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  out.z = Math.floor(a.z);
  out.w = Math.floor(a.w);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  out.z = Math.min(a.z, b.z);
  out.w = Math.min(a.w, b.w);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  out.z = Math.max(a.z, b.z);
  out.w = Math.max(a.w, b.w);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  out.z = Math.round(a.z);
  out.w = Math.round(a.w);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec4} out.
 * */
vec4.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  out.w = a.w * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec4} out.
 */
vec4.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  out.z = a.z + (b.z * scale);
  out.w = a.w + (b.w * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec4.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z,
    w = b.w - a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 * Alias of {@link vec4.distance}.
 */
vec4.dist = function dist (a, b)
{
  return vec4.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec4.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z,
    w = b.w - a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 * Alias of {@link vec4.squaredDistance}.
 */
vec4.sqrDist = function sqrDist (a, b)
{
  return vec4.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec4.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 *Alias of {@link vec4.magnitude}.
 */
vec4.mag = function mag (a)
{
  return vec4.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec4.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 *Alias of {@link vec4.squaredMagnitude}
 */
vec4.sqrMag = function sqrMag (a)
{
  return vec4.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to negate.
 * @returns {vec4} out.
 */
vec4.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  out.w = -a.w;
  return out;
};

/**
 * Inverts the components of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to invert.
 * @returns {vec4} out.
 */
vec4.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  out.z = 1.0 / a.z;
  out.w = 1.0 / a.w;
  return out;
};

/**
 * Safely inverts the components of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to invert.
 * @returns {vec4} out.
 */
vec4.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / y;
  }

  if (Math.abs(z) < EPSILON)
  {
    out.z = 0;
  } else
  {
    out.z = 1.0 / z;
  }

  if (Math.abs(w) < EPSILON)
  {
    out.w = 0;
  } else
  {
    out.w = 1.0 / w;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to normalize.
 * @returns {vec4} out.
 */
vec4.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  var len = x * x + y * y + z * z + w * w;
  if (len > 0)
  {
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
    out.w = w * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec4.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec4} out.
 */
vec4.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z,
    aw = a.w;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  out.w = aw + t * (b.w - aw);
  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec4} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec4} out.
 */
vec4.random = function random$1 (out, scale)
{
  scale = scale || 1.0;

  //TODO: This is a pretty awful way of doing this. Find something better.
  out.x = random();
  out.y = random();
  out.z = random();
  out.w = random();
  vec4.normalize(out, out);
  vec4.scale(out, out, scale);
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec4} out.
 */
vec4.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z, w = a.w;
  out.x = m.m00 * x + m.m04 * y + m.m08 * z + m.m12 * w;
  out.y = m.m01 * x + m.m05 * y + m.m09 * z + m.m13 * w;
  out.z = m.m02 * x + m.m06 * y + m.m10 * z + m.m14 * w;
  out.w = m.m03 * x + m.m07 * y + m.m11 * z + m.m15 * w;
  return out;
};

/**
 * Transforms a vector with a quaternion.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to transform.
 * @param {quat} q - The quaternion.
 * @returns {vec4} out.
 */
vec4.transformQuat = function transformQuat (out, a, q)
{
  var x = a.x, y = a.y, z = a.z;
  var qx = q.x, qy = q.y, qz = q.z, qw = q.w;

  // calculate quat * vec
  var ix = qw * x + qy * z - qz * y;
  var iy = qw * y + qz * x - qx * z;
  var iz = qw * z + qx * y - qy * x;
  var iw = -qx * x - qy * y - qz * z;

  // calculate result * inverse quat
  out.x = ix * qw + iw * -qx + iy * -qz - iz * -qy;
  out.y = iy * qw + iw * -qy + iz * -qx - ix * -qz;
  out.z = iz * qw + iw * -qz + ix * -qy - iy * -qx;
  out.w = a.w;
  return out;
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec4.str = function str (a)
{
  return ("vec4(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ", " + (a.w) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec4} v - The vector.
 * @returns {Array} out.
 */
vec4.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;
  out[2] = v.z;
  out[3] = v.w;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec4} a - The first vector.
 * @param {vec4} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec4.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y && a.z === b.z && a.w === b.w;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec4} a The first vector.
 * @param {vec4} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec4.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y, a2 = a.z, a3 = a.w;
  var b0 = b.x, b1 = b.y, b2 = b.z, b3 = b.w;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
    Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
    Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)));
};

/**
 * Performs some operation over an array of vec4s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec4.forEach = function forEach (a, stride, offset, count, fn, arg)
{
  return vec4._forEach(a, stride, offset, count, fn, arg);
};

/**
 * Perform some operation over an array of vec4s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 * @ignore
 */
vec4._forEach = (function()
{
  var vec = vec4.zero();

  return function (a, stride, offset, count, fn, arg)
  {
    var i, l;
    if (!stride)
    {
      stride = 4;
    }

    if (!offset)
    {
      offset = 0;
    }

    if (count)
    {
      l = Math.min((count * stride) + offset, a.length);
    } else
    {
      l = a.length;
    }

    for (i = offset; i < l; i += stride)
    {
      vec.x = a[i];
      vec.y = a[i + 1];
      vec.z = a[i + 2];
      vec.w = a[i + 3];
      fn(vec, vec, arg);
      a[i] = vec.x;
      a[i + 1] = vec.y;
      a[i + 2] = vec.z;
      a[i + 3] = vec.w;
    }

    return a;
  };
})();

/**
 * Mathematical 3x3 matrix.
 *
 * NOTE: we use column-major matrix for all matrix calculation.
 *
 * This may lead to some confusion when referencing OpenGL documentation,
 * however, which represents out all matricies in column-major format.
 * This means that while in code a matrix may be typed out as:
 *
 * [1, 0, 0, 0,
 *  0, 1, 0, 0,
 *  0, 0, 1, 0,
 *  x, y, z, 0]
 *
 * The same matrix in the [OpenGL documentation](https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glTranslate.xml)
 * is written as:
 *
 *  1 0 0 x
 *  0 1 0 y
 *  0 0 1 z
 *  0 0 0 0
 *
 * Please rest assured, however, that they are the same thing!
 * This is not unique to glMatrix, either, as OpenGL developers have long been confused by the
 * apparent lack of consistency between the memory layout and the documentation.
 */
var mat3 = function mat3(
m00, m01, m02, m03,
m04, m05, m06, m07,
m08
)
{
/**
 * The element at column 0 row 0.
 * @type {number}
 * */
this.m00 = m00;
/**
 * The element at column 0 row 1.
 * @type {number}
 * */
this.m01 = m01;
/**
 * The element at column 0 row 2.
 * @type {number}
 * */
this.m02 = m02;

/**
 * The element at column 1 row 0.
 * @type {number}
 * */
this.m03 = m03;
/**
 * The element at column 1 row 1.
 * @type {number}
 * */
this.m04 = m04;
/**
 * The element at column 1 row 2.
 * @type {number}
 * */
this.m05 = m05;

/**
 * The element at column 2 row 0.
 * @type {number}
 * */
this.m06 = m06;
/**
 * The element at column 2 row 1.
 * @type {number}
 * */
this.m07 = m07;
/**
 * The element at column 2 row 2.
 * @type {number}
 * */
this.m08 = m08;
};

/**
 * Creates a new identity matrix.
 *
 * @returns {mat3} The newly created matrix.
 */
mat3.create = function create ()
{
return new mat3(
  1, 0, 0,
  0, 1, 0,
  0, 0, 1
);
};

/**
 * Creates a matrix, with elements specified separately.
 *
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m03 - Value assigned to element at column 1 row 0.
 * @param {Number} m04 - Value assigned to element at column 1 row 1.
 * @param {Number} m05 - Value assigned to element at column 1 row 2.
 * @param {Number} m06 - Value assigned to element at column 2 row 0.
 * @param {Number} m07 - Value assigned to element at column 2 row 1.
 * @param {Number} m08 - Value assigned to element at column 2 row 2.
 * @returns {mat3} The newly created matrix.
 */
mat3.new = function new$1 (m00, m01, m02, m03, m04, m05, m06, m07, m08)
{
return new mat3(m00, m01, m02, m03, m04, m05, m06, m07, m08);
};

/**
 * Clone a matrix.
 *
 * @param {mat3} a - Matrix to clone.
 * @returns {mat3} The newly created matrix.
 */
mat3.clone = function clone (a)
{
  return new mat3(
    a.m00, a.m01, a.m02,
    a.m03, a.m04, a.m05,
    a.m06, a.m07, a.m08
  );
};

/**
 * Copy content of a matrix into another.
 *
 * @param {mat3} out - Matrix to modified.
 * @param {mat3} a - The specified matrix.
 * @returns {mat3} out.
 */
mat3.copy = function copy (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = a.m02;
out.m03 = a.m03;
out.m04 = a.m04;
out.m05 = a.m05;
out.m06 = a.m06;
out.m07 = a.m07;
out.m08 = a.m08;
return out;
};

/**
 * Sets the elements of a matrix to the given values.
 *
 * @param {mat3} out - The matrix to modified.
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m10 - Value assigned to element at column 1 row 0.
 * @param {Number} m11 - Value assigned to element at column 1 row 1.
 * @param {Number} m12 - Value assigned to element at column 1 row 2.
 * @param {Number} m20 - Value assigned to element at column 2 row 0.
 * @param {Number} m21 - Value assigned to element at column 2 row 1.
 * @param {Number} m22 - Value assigned to element at column 2 row 2.
 * @returns {mat3} out.
 */
mat3.set = function set (out, m00, m01, m02, m10, m11, m12, m20, m21, m22)
{
  out.m00 = m00;
  out.m01 = m01;
  out.m02 = m02;
  out.m03 = m10;
  out.m04 = m11;
  out.m05 = m12;
  out.m06 = m20;
  out.m07 = m21;
  out.m08 = m22;
  return out;
};

/**
 * Sets a matrix as identity matrix.
 *
 * @param {mat3} out - Matrix to modified.
 * @returns {mat3} out.
 */
mat3.identity = function identity (out)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 1;
out.m05 = 0;
out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Transposes a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to transpose.
 * @returns {mat3} out.
 */
mat3.transpose = function transpose (out, a)
{
// If we are transposing ourselves we can skip a few steps but have to cache some values
if (out === a) {
  var a01 = a.m01, a02 = a.m02, a12 = a.m05;
  out.m01 = a.m03;
  out.m02 = a.m06;
  out.m03 = a01;
  out.m05 = a.m07;
  out.m06 = a02;
  out.m07 = a12;
} else {
  out.m00 = a.m00;
  out.m01 = a.m03;
  out.m02 = a.m06;
  out.m03 = a.m01;
  out.m04 = a.m04;
  out.m05 = a.m07;
  out.m06 = a.m02;
  out.m07 = a.m05;
  out.m08 = a.m08;
}

return out;
};

/**
 * Inverts a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to invert.
 * @returns {mat3} out.
 */
mat3.invert = function invert (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var b01 = a22 * a11 - a12 * a21;
var b11 = -a22 * a10 + a12 * a20;
var b21 = a21 * a10 - a11 * a20;

// Calculate the determinant
var det = a00 * b01 + a01 * b11 + a02 * b21;

if (!det) {
  return null;
}
det = 1.0 / det;

out.m00 = b01 * det;
out.m01 = (-a22 * a01 + a02 * a21) * det;
out.m02 = (a12 * a01 - a02 * a11) * det;
out.m03 = b11 * det;
out.m04 = (a22 * a00 - a02 * a20) * det;
out.m05 = (-a12 * a00 + a02 * a10) * det;
out.m06 = b21 * det;
out.m07 = (-a21 * a00 + a01 * a20) * det;
out.m08 = (a11 * a00 - a01 * a10) * det;
return out;
};

/**
 * Calculates the adjugate of a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to calculate.
 * @returns {mat3} out.
 */
mat3.adjoint = function adjoint (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

out.m00 = (a11 * a22 - a12 * a21);
out.m01 = (a02 * a21 - a01 * a22);
out.m02 = (a01 * a12 - a02 * a11);
out.m03 = (a12 * a20 - a10 * a22);
out.m04 = (a00 * a22 - a02 * a20);
out.m05 = (a02 * a10 - a00 * a12);
out.m06 = (a10 * a21 - a11 * a20);
out.m07 = (a01 * a20 - a00 * a21);
out.m08 = (a00 * a11 - a01 * a10);
return out;
};

/**
 * Calculates the determinant of a matrix.
 *
 * @param {mat3} a - Matrix to calculate.
 * @returns {Number} Determinant of a.
 */
mat3.determinant = function determinant (a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

return a00 * (a22 * a11 - a12 * a21) + a01 * (-a22 * a10 + a12 * a20) + a02 * (a21 * a10 - a11 * a20);
};

/**
 * Multiply two matrices explicitly.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.multiply = function multiply (out, a, b)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var b00 = b.m00, b01 = b.m01, b02 = b.m02;
var b10 = b.m03, b11 = b.m04, b12 = b.m05;
var b20 = b.m06, b21 = b.m07, b22 = b.m08;

out.m00 = b00 * a00 + b01 * a10 + b02 * a20;
out.m01 = b00 * a01 + b01 * a11 + b02 * a21;
out.m02 = b00 * a02 + b01 * a12 + b02 * a22;

out.m03 = b10 * a00 + b11 * a10 + b12 * a20;
out.m04 = b10 * a01 + b11 * a11 + b12 * a21;
out.m05 = b10 * a02 + b11 * a12 + b12 * a22;

out.m06 = b20 * a00 + b21 * a10 + b22 * a20;
out.m07 = b20 * a01 + b21 * a11 + b22 * a21;
out.m08 = b20 * a02 + b21 * a12 + b22 * a22;
return out;
};

/**
 * Alias of {@link mat3.multiply}.
 */
mat3.mul = function mul (out, a, b)
{
return mat3.multiply(out, a, b);
};

/**
 * Multiply a matrix with a translation matrix given by a translation offset.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to multiply.
 * @param {vec2} v - The translation offset.
 * @returns {mat3} out.
 */
mat3.translate = function translate (out, a, v)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;
var x = v.x, y = v.y;

out.m00 = a00;
out.m01 = a01;
out.m02 = a02;

out.m03 = a10;
out.m04 = a11;
out.m05 = a12;

out.m06 = x * a00 + y * a10 + a20;
out.m07 = x * a01 + y * a11 + a21;
out.m08 = x * a02 + y * a12 + a22;
return out;
};

/**
 * Rotates a matrix by the given angle.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to rotate.
 * @param {Number} rad - The rotation angle.
 * @returns {mat3} out
 */
mat3.rotate = function rotate (out, a, rad)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var s = Math.sin(rad);
var c = Math.cos(rad);

out.m00 = c * a00 + s * a10;
out.m01 = c * a01 + s * a11;
out.m02 = c * a02 + s * a12;

out.m03 = c * a10 - s * a00;
out.m04 = c * a11 - s * a01;
out.m05 = c * a12 - s * a02;

out.m06 = a20;
out.m07 = a21;
out.m08 = a22;
return out;
};

/**
 * Multiply a matrix with a scale matrix given by a scale vector.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to multiply.
 * @param {vec2} v - The scale vector.
 * @returns {mat3} out
 **/
mat3.scale = function scale (out, a, v)
{
var x = v.x, y = v.y;

out.m00 = x * a.m00;
out.m01 = x * a.m01;
out.m02 = x * a.m02;

out.m03 = y * a.m03;
out.m04 = y * a.m04;
out.m05 = y * a.m05;

out.m06 = a.m06;
out.m07 = a.m07;
out.m08 = a.m08;
return out;
};

/**
 * Copies the upper-left 3x3 values of a 4x4 matrix into a 3x3 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat4} a - The 4x4 matrix.
 * @returns {mat3} out.
 */
mat3.fromMat4 = function fromMat4 (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = a.m02;
out.m03 = a.m04;
out.m04 = a.m05;
out.m05 = a.m06;
out.m06 = a.m08;
out.m07 = a.m09;
out.m08 = a.m10;
return out;
};

/**
 * Creates a matrix from a translation offset.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.translate(dest, dest, vec);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec2} v - The translation offset.
 * @returns {mat3} out.
 */
mat3.fromTranslation = function fromTranslation (out, v)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 1;
out.m05 = 0;
out.m06 = v.x;
out.m07 = v.y;
out.m08 = 1;
return out;
};

/**
 * Creates a matrix from a given angle.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.rotate(dest, dest, rad);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @returns {mat3} out.
 */
mat3.fromRotation = function fromRotation (out, rad)
{
var s = Math.sin(rad), c = Math.cos(rad);

out.m00 = c;
out.m01 = s;
out.m02 = 0;

out.m03 = -s;
out.m04 = c;
out.m05 = 0;

out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Creates a matrix from a scale vector.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.scale(dest, dest, vec);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec2} v - Scale vector.
 * @returns {mat3} out.
 */
mat3.fromScaling = function fromScaling (out, v)
{
out.m00 = v.x;
out.m01 = 0;
out.m02 = 0;

out.m03 = 0;
out.m04 = v.y;
out.m05 = 0;

out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Copies the values from a 2x3 matrix into a 3x3 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat23} a - The 2x3 matrix.
 * @returns {mat3} out.
 **/
mat3.fromMat2d = function fromMat2d (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = 0;

out.m03 = a.m02;
out.m04 = a.m03;
out.m05 = 0;

out.m06 = a.m04;
out.m07 = a.m05;
out.m08 = 1;
return out;
};

/**
 * Calculates a 3x3 matrix from the given quaternion.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {quat} q - The quaternion.
 *
 * @returns {mat3} out.
 */
mat3.fromQuat = function fromQuat (out, q)
{
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var yx = y * x2;
var yy = y * y2;
var zx = z * x2;
var zy = z * y2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;

out.m00 = 1 - yy - zz;
out.m03 = yx - wz;
out.m06 = zx + wy;

out.m01 = yx + wz;
out.m04 = 1 - xx - zz;
out.m07 = zy - wx;

out.m02 = zx - wy;
out.m05 = zy + wx;
out.m08 = 1 - xx - yy;

return out;
};

/**
 * Calculates a 3x3 matrix from view direction and up direction.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec3} view - View direction (must be normalized).
 * @param {vec3} [up] - Up direction, default is (0,1,0) (must be normalized).
 *
 * @returns {mat3} out
 */
mat3.fromViewUp = function fromViewUp (out, view, up)
{
return mat3._fromViewUp(out, view, up);
};

/**
 * Calculates a 3x3 normal matrix (transpose inverse) from the 4x4 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat4} a - A 4x4 matrix to derive the normal matrix from.
 *
 * @returns {mat3} out.
 */
mat3.normalFromMat4 = function normalFromMat4 (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

var b00 = a00 * a11 - a01 * a10;
var b01 = a00 * a12 - a02 * a10;
var b02 = a00 * a13 - a03 * a10;
var b03 = a01 * a12 - a02 * a11;
var b04 = a01 * a13 - a03 * a11;
var b05 = a02 * a13 - a03 * a12;
var b06 = a20 * a31 - a21 * a30;
var b07 = a20 * a32 - a22 * a30;
var b08 = a20 * a33 - a23 * a30;
var b09 = a21 * a32 - a22 * a31;
var b10 = a21 * a33 - a23 * a31;
var b11 = a22 * a33 - a23 * a32;

// Calculate the determinant
var det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

if (!det) {
  return null;
}
det = 1.0 / det;

out.m00 = (a11 * b11 - a12 * b10 + a13 * b09) * det;
out.m01 = (a12 * b08 - a10 * b11 - a13 * b07) * det;
out.m02 = (a10 * b10 - a11 * b08 + a13 * b06) * det;

out.m03 = (a02 * b10 - a01 * b11 - a03 * b09) * det;
out.m04 = (a00 * b11 - a02 * b08 + a03 * b07) * det;
out.m05 = (a01 * b08 - a00 * b10 - a03 * b06) * det;

out.m06 = (a31 * b05 - a32 * b04 + a33 * b03) * det;
out.m07 = (a32 * b02 - a30 * b05 - a33 * b01) * det;
out.m08 = (a30 * b04 - a31 * b02 + a33 * b00) * det;

return out;
};

/**
 * Returns a string representation of a matrix.
 *
 * @param {mat3} a - The matrix.
 * @returns {String} String representation of this matrix.
 */
mat3.str = function str (a)
{
  ("mat3(" + (a.m00) + ", " + (a.m01) + ", " + (a.m02) + ", " + (a.m03) + ", " + (a.m04) + ", " + (a.m05) + ", " + (a.m06) + ", " + (a.m07) + ", " + (a.m08) + ")");
};

/**
 * Store elements of a matrix into array.
 *
 * @param {array} out - Array to store result.
 * @param {mat3} m - The matrix.
 * @returns {Array} out.
 */
mat3.array = function array (out, m)
{
out[0] = m.m00;
out[1] = m.m01;
out[2] = m.m02;
out[3] = m.m03;
out[4] = m.m04;
out[5] = m.m05;
out[6] = m.m06;
out[7] = m.m07;
out[8] = m.m08;

return out;
};

/**
 * Returns Frobenius norm of a matrix.
 *
 * @param {mat3} a - Matrix to calculate Frobenius norm of.
 * @returns {Number} - The frobenius norm.
 */
mat3.frob = function frob (a)
{
return (Math.sqrt(Math.pow(a.m00, 2) + Math.pow(a.m01, 2) + Math.pow(a.m02, 2) + Math.pow(a.m03, 2) + Math.pow(a.m04, 2) + Math.pow(a.m05, 2) + Math.pow(a.m06, 2) + Math.pow(a.m07, 2) + Math.pow(a.m08, 2)));
};

/**
 * Adds two matrices.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.add = function add (out, a, b)
{
out.m00 = a.m00 + b.m00;
out.m01 = a.m01 + b.m01;
out.m02 = a.m02 + b.m02;
out.m03 = a.m03 + b.m03;
out.m04 = a.m04 + b.m04;
out.m05 = a.m05 + b.m05;
out.m06 = a.m06 + b.m06;
out.m07 = a.m07 + b.m07;
out.m08 = a.m08 + b.m08;
return out;
};

/**
 * Subtracts matrix b from matrix a.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.subtract = function subtract (out, a, b)
{
out.m00 = a.m00 - b.m00;
out.m01 = a.m01 - b.m01;
out.m02 = a.m02 - b.m02;
out.m03 = a.m03 - b.m03;
out.m04 = a.m04 - b.m04;
out.m05 = a.m05 - b.m05;
out.m06 = a.m06 - b.m06;
out.m07 = a.m07 - b.m07;
out.m08 = a.m08 - b.m08;
return out;
};

/**
 * Alias of {@link mat3.subtract}.
 */
mat3.sub = function sub (out, a, b)
{
return mat3.subtract(out, a, b);
};

/**
 * Multiply each element of a matrix by a scalar number.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to scale
 * @param {Number} b - The scale number.
 * @returns {mat3} out.
 */
mat3.multiplyScalar = function multiplyScalar (out, a, b)
{
out.m00 = a.m00 * b;
out.m01 = a.m01 * b;
out.m02 = a.m02 * b;
out.m03 = a.m03 * b;
out.m04 = a.m04 * b;
out.m05 = a.m05 * b;
out.m06 = a.m06 * b;
out.m07 = a.m07 * b;
out.m08 = a.m08 * b;
return out;
};

/**
 * Adds two matrices after multiplying each element of the second operand by a scalar number.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @param {Number} scale - The scale number.
 * @returns {mat3} out.
 */
mat3.multiplyScalarAndAdd = function multiplyScalarAndAdd (out, a, b, scale)
{
out.m00 = a.m00 + (b.m00 * scale);
out.m01 = a.m01 + (b.m01 * scale);
out.m02 = a.m02 + (b.m02 * scale);
out.m03 = a.m03 + (b.m03 * scale);
out.m04 = a.m04 + (b.m04 * scale);
out.m05 = a.m05 + (b.m05 * scale);
out.m06 = a.m06 + (b.m06 * scale);
out.m07 = a.m07 + (b.m07 * scale);
out.m08 = a.m08 + (b.m08 * scale);
return out;
};

/**
 * Returns whether the specified matrices are equal. (Compared using ===)
 *
 * @param {mat3} a - The first matrix.
 * @param {mat3} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat3.exactEquals = function exactEquals (a, b)
{
return a.m00 === b.m00 && a.m01 === b.m01 && a.m02 === b.m02 &&
  a.m03 === b.m03 && a.m04 === b.m04 && a.m05 === b.m05 &&
  a.m06 === b.m06 && a.m07 === b.m07 && a.m08 === b.m08;
};

/**
 * Returns whether the specified matrices are approximately equal.
 *
 * @param {mat3} a - The first matrix.
 * @param {mat3} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat3.equals = function equals$$1 (a, b)
{
var a0 = a.m00, a1 = a.m01, a2 = a.m02, a3 = a.m03, a4 = a.m04, a5 = a.m05, a6 = a.m06, a7 = a.m07, a8 = a.m08;
var b0 = b.m00, b1 = b.m01, b2 = b.m02, b3 = b.m03, b4 = b.m04, b5 = b.m05, b6 = b.m06, b7 = b.m07, b8 = b.m08;
return (
  Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
  Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
  Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
  Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)) &&
  Math.abs(a4 - b4) <= EPSILON * Math.max(1.0, Math.abs(a4), Math.abs(b4)) &&
  Math.abs(a5 - b5) <= EPSILON * Math.max(1.0, Math.abs(a5), Math.abs(b5)) &&
  Math.abs(a6 - b6) <= EPSILON * Math.max(1.0, Math.abs(a6), Math.abs(b6)) &&
  Math.abs(a7 - b7) <= EPSILON * Math.max(1.0, Math.abs(a7), Math.abs(b7)) &&
  Math.abs(a8 - b8) <= EPSILON * Math.max(1.0, Math.abs(a8), Math.abs(b8))
);
};

mat3._fromViewUp = (function ()
{
  var default_up = vec3.new(0, 1, 0);
  var x = vec3.zero();
  var y = vec3.zero();

  return function (out, view, up)
  {
    if (vec3.sqrMag(view) < EPSILON * EPSILON)
    {
      mat3.identity(out);
      return out;
    }

    up = up || default_up;
    vec3.cross(x, up, view);

    if (vec3.sqrMag(x) < EPSILON * EPSILON)
    {
      mat3.identity(out);
      return out;
    }

    vec3.cross(y, view, x);
    mat3.set(
	  out,
      x.x, x.y, x.z,
      y.x, y.y, y.z,
      view.x, view.y, view.z
    );

    return out;
  };
})();

/**
 * Mathematical quaternion.
 *
 * A quaternion is a hypercomplex number represented by w + xi + yj + zk, where
 * x, y, z and w are real numbers(called here its components), and i, j, and k are the fundamental quaternion units.
 */
var quat = function quat(x, y, z, w)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
  /**
   * The w component.
   * @type {number}
   * */
  this.w = w;
};

/**
 * Creates a quaternion, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @param {number} w - Value assigned to w component.
 * @return {quat} The newly created quaternion.
 */
quat.new = function new$1 (x, y, z, w)
{
  return new quat(x, y, z, w);
};

/**
 * Creates an identity quaternion.
 *
 * @returns {quat} The newly created quaternion.
 */
quat.create = function create ()
{
  return new quat(0, 0, 0, 1);
};

/**
 * Clone a quaternion.
 *
 * @param {quat} a - Quaternion to clone.
 * @returns {quat} The newly created quaternion.
 */
quat.clone = function clone (a)
{
  return new quat(a.x, a.y, a.z, a.w);
};

/**
 * Copy content of a quaternion into another.
 *
 * @param {quat} out - Quaternion to modified.
 * @param {quat} a - The specified quaternion.
 * @returns {quat} out.
 */
quat.copy = function copy (out, a)
{
	return vec4.copy(out, a);
};

/**
 * Sets the components of a quaternion to the given values.
 *
 * @param {quat} out - The quaternion to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @param {Number} w - Value set to w component.
 * @returns {quat} out.
 */
quat.set = function set (out, x, y, z, w)
{
  out.x = x;
  out.y = y;
  out.z = z;
  out.w = w;
  return out;
};
  
/**
 * Sets a quaternion as identity quaternion.
 *
 * @param {quat} out - Quaternion to set.
 * @returns {quat} out.
 */
quat.identity = function identity (out)
{
  out.x = 0;
  out.y = 0;
  out.z = 0;
  out.w = 1;
  return out;
};

/**
 * Sets a quaternion to represent the shortest rotation from one
 * vector to another.
 *
 * Both vectors are assumed to be unit length.
 *
 * @param {quat} out - Quaternion to set.
 * @param {vec3} a - The initial vector.
 * @param {vec3} b - The destination vector.
 * @returns {quat} out.
 */
quat.rotationTo = function rotationTo (out, a, b)
{
  return quat._rotationTo(out, a, b);
};

/**
 * Gets the rotation axis and angle for a given
 *quaternion. If a quaternion is created with
 *fromAxisAngle, this method will return the same
 *values as provided in the original parameter list
 *OR functionally equivalent values.
 * Example: The quaternion formed by axis [0, 0, 1] and
 *angle -90 is the same as the quaternion formed by
 *[0, 0, 1] and 270. This method favors the latter.
 * @param{vec3} out_axis - Vector to store the rotation axis.
 * @param{quat} q - Quaternion to be decomposed.
 * @return {Number} - Angle, in radians, of the rotation.
 */
quat.getAxisAngle = function getAxisAngle (out_axis, q)
{
  var rad = Math.acos(q.w) * 2.0;
  var s = Math.sin(rad / 2.0);
  if (s != 0.0)
  {
    out_axis.x = q.x / s;
    out_axis.y = q.y / s;
    out_axis.z = q.z / s;
  } else
  {
    // If s is zero, return any axis (no rotation - axis does not matter)
    out_axis.x = 1;
    out_axis.y = 0;
    out_axis.z = 0;
  }
  return rad;
};

/**
 * Multiply two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @returns {quat} out.
 */
quat.multiply = function multiply (out, a, b)
{
  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = b.x, by = b.y, bz = b.z, bw = b.w;

  out.x = ax * bw + aw * bx + ay * bz - az * by;
  out.y = ay * bw + aw * by + az * bx - ax * bz;
  out.z = az * bw + aw * bz + ax * by - ay * bx;
  out.w = aw * bw - ax * bx - ay * by - az * bz;
  return out;
};

/**
 * Alias of {@link quat.multiply}.
 */
quat.mul = function mul (out, a, b)
{
	return quat.multiply(out, a, b);
};

/**
 * Scales a quaternion with a number.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to scale.
 * @param {quat} b - The scale number.
 * @returns {quat} out.
 * */
quat.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  out.w = a.w * b;
  return out;
};
  
/**
 * Rotates a quaternion by the given angle about the X axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateX = function rotateX (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw + aw * bx;
  out.y = ay * bw + az * bx;
  out.z = az * bw - ay * bx;
  out.w = aw * bw - ax * bx;
  return out;
};

/**
 * Rotates a quaternion by the given angle about the Y axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateY = function rotateY (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    by = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw - az * by;
  out.y = ay * bw + aw * by;
  out.z = az * bw + ax * by;
  out.w = aw * bw - ay * by;
  return out;
};

/**
 * Rotates a quaternion by the given angle about the Z axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateZ = function rotateZ (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bz = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw + ay * bz;
  out.y = ay * bw - ax * bz;
  out.z = az * bw + aw * bz;
  out.w = aw * bw - az * bz;
  return out;
};

/**
 * Rotates a quaternion by the given angle about a world space axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} rot - Quaternion to rotate.
 * @param {vec3} axis - The axis around which to rotate in world space.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateAround = function rotateAround (out, rot, axis, rad)
{
  return quat._rotateAround(out, rot, axis, rad);
};

/**
 * Rotates a quaternion by the given angle about a local space axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} rot - Quaternion to rotate.
 * @param {vec3} axis - The axis around which to rotate in local space.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateAroundLocal = function rotateAroundLocal (out, rot, axis, rad)
{
  return quat._rotateAroundLocal(out, rot, axis, rad);
};

/**
 * Calculates the W component of a quaternion from the X, Y, and Z components.
 * Assumes that quaternion is 1 unit in length.
 * Any existing W component will be ignored.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate W.
 * @returns {quat} out.
 */
quat.calculateW = function calculateW (out, a)
{
  var x = a.x, y = a.y, z = a.z;

  out.x = x;
  out.y = y;
  out.z = z;
  out.w = Math.sqrt(Math.abs(1.0 - x * x - y * y - z * z));
  return out;
};

/**
 * Calculates the dot product of two quaternions.
 *
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @returns {Number} - The dot product of a and b.
 */
quat.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
};
  
/**
 * Performs a linear interpolation between two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out.
 */
quat.lerp = function lerp (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z,
    aw = a.w;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  out.w = aw + t * (b.w - aw);
  return out;
};
  
/**
 * Performs a spherical linear interpolation between two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out.
 */
quat.slerp = function slerp (out, a, b, t)
{
  // benchmarks:
  //  http://jsperf.com/quaternion-slerp-implementations

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = b.x, by = b.y, bz = b.z, bw = b.w;

  var omega, cosom, sinom, scale0, scale1;

  // calc cosine
  cosom = ax * bx + ay * by + az * bz + aw * bw;
  // adjust signs (if necessary)
  if (cosom < 0.0)
  {
    cosom = -cosom;
    bx = -bx;
    by = -by;
    bz = -bz;
    bw = -bw;
  }
  // calculate coefficients
  if ((1.0 - cosom) > 0.000001)
  {
    // standard case (slerp)
    omega = Math.acos(cosom);
    sinom = Math.sin(omega);
    scale0 = Math.sin((1.0 - t) * omega) / sinom;
    scale1 = Math.sin(t * omega) / sinom;
  } else
  {
    // "from" and "to" quaternions are very close
    //... so we can do a linear interpolation
    scale0 = 1.0 - t;
    scale1 = t;
  }
  // calculate final values
  out.x = scale0 * ax + scale1 * bx;
  out.y = scale0 * ay + scale1 * by;
  out.z = scale0 * az + scale1 * bz;
  out.w = scale0 * aw + scale1 * bw;

  return out;
};

/**
 * Performs a spherical linear interpolation with two control points.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {quat} c - The third operand.
 * @param {quat} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out
 */
quat.sqlerp = function sqlerp (out, a, b, c, d, t)
{
  return quat._sqlerp(out, a, b, c, d, t);
};

/**
 * Calculates the inverse of a quaternion.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate inverse of.
 * @returns {quat} out.
 */
quat.invert = function invert (out, a)
{
  var a0 = a.x, a1 = a.y, a2 = a.z, a3 = a.w;
  var dot = a0 * a0 + a1 * a1 + a2 * a2 + a3 * a3;
  var invDot = dot ? 1.0 / dot : 0;

  // TODO: Would be faster to return [0,0,0,0] immediately if dot == 0

  out.x = -a0 * invDot;
  out.y = -a1 * invDot;
  out.z = -a2 * invDot;
  out.w = a3 * invDot;
  return out;
};

/**
 * Calculates the conjugate of a quaternion.
 * If the quaternion is normalized, this function is faster than quat.inverse and produces the same result.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate conjugate of.
 * @returns {quat} out.
 */
quat.conjugate = function conjugate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  out.w = a.w;
  return out;
};

/**
 * Calculates the length of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {Number} Length of the quaternion.
 */
quat.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 *Alias of {@link quat.magnitude}.
 */
quat.mag = function mag (a)
{
  return quat.magnitude(a);
};

/**
 * Calculates the squared length of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {Number} Squared length of the quaternion.
 */
quat.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 *Alias of {@link quat.squaredMagnitude}
 */
quat.sqrMag = function sqrMag (a)
{
  return quat.squaredMagnitude(a);
};
  
/**
 * Normalizes a quaternion.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to normalize.
 * @returns {quat} out.
 * @function
 */
quat.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  var len = x * x + y * y + z * z + w * w;
  if (len > 0)
  {
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
    out.w = w * len;
  }
  return out;
};
  
/**
 * Sets the specified quaternion with values corresponding to the given
 * axes. Each axis is a vec3 and is expected to be unit length and
 * perpendicular to all other specified axes.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} xAxis - Vector representing the local "right" direction.
 * @param {vec3} yAxis - Vector representing the local "up" direction.
 * @param {vec3} zAxis - Vector representing the viewing direction.
 * @returns {quat} out.
 */
quat.fromAxes = function fromAxes (out, xAxis, yAxis, zAxis)
{
  return quat._fromAxes(out, xAxis, yAxis, zAxis);
};

/**
 * Calculates a quaternion from view direction and up direction
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} view - View direction (must be normalized).
 * @param {vec3} [up] - Up direction, default is (0,1,0) (must be normalized).
 *
 * @returns {quat} out.
 */
quat.fromViewUp = function fromViewUp (out, view, up)
{
  return quat._fromViewUp(out, view, up);
};

/**
 * Sets a quaternion from the given angle and rotation axis,
 * then returns it.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} axis - The axis around which to rotate.
 * @param {Number} rad - The angle in radians.
 * @returns {quat} out.
 **/
quat.fromAxisAngle = function fromAxisAngle (out, axis, rad)
{
  rad = rad * 0.5;
  var s = Math.sin(rad);
  out.x = s * axis.x;
  out.y = s * axis.y;
  out.z = s * axis.z;
  out.w = Math.cos(rad);
  return out;
};

/**
 * Creates a quaternion from the given 3x3 rotation matrix.
 *
 * NOTE: The resultant quaternion is not normalized, so you should be sure
 * to re-normalize the quaternion yourself where necessary.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {mat3} m - The rotation matrix.
 * @returns {quat} out.
 * @function
 */
quat.fromMat3 = function fromMat3 (out, m)
{
  // http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm

  var m00 = m.m00, m01 = m.m03, m02 = m.m06,
    m10 = m.m01, m11 = m.m04, m12 = m.m07,
    m20 = m.m02, m21 = m.m05, m22 = m.m08;

  var trace = m00 + m11 + m22;

  if (trace > 0)
  {
    var s = 0.5 / Math.sqrt(trace + 1.0);

    out.w = 0.25 / s;
    out.x = (m21 - m12) * s;
    out.y = (m02 - m20) * s;
    out.z = (m10 - m01) * s;

  } else if ((m00 > m11) && (m00 > m22))
  {
    var s$1 = 2.0 * Math.sqrt(1.0 + m00 - m11 - m22);

    out.w = (m21 - m12) / s$1;
    out.x = 0.25 * s$1;
    out.y = (m01 + m10) / s$1;
    out.z = (m02 + m20) / s$1;

  } else if (m11 > m22)
  {
    var s$2 = 2.0 * Math.sqrt(1.0 + m11 - m00 - m22);

    out.w = (m02 - m20) / s$2;
    out.x = (m01 + m10) / s$2;
    out.y = 0.25 * s$2;
    out.z = (m12 + m21) / s$2;

  } else
  {
    var s$3 = 2.0 * Math.sqrt(1.0 + m22 - m00 - m11);

    out.w = (m10 - m01) / s$3;
    out.x = (m02 + m20) / s$3;
    out.y = (m12 + m21) / s$3;
    out.z = 0.25 * s$3;
  }

  return out;
};

/**
 * Creates a quaternion from the given euler angle x, y, z.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {number} x - Angle to rotate around X axis in degrees.
 * @param {number} y - Angle to rotate around Y axis in degrees.
 * @param {number} z - Angle to rotate around Z axis in degrees.
 * @returns {quat} out.
 * @function
 */
quat.fromEuler = function fromEuler (out, x, y, z)
{
  var halfToRad = 0.5 * Math.PI / 180.0;
  x *= halfToRad;
  y *= halfToRad;
  z *= halfToRad;

  var sx = Math.sin(x);
  var cx = Math.cos(x);
  var sy = Math.sin(y);
  var cy = Math.cos(y);
  var sz = Math.sin(z);
  var cz = Math.cos(z);

  out.x = sx * cy * cz - cx * sy * sz;
  out.y = cx * sy * cz + sx * cy * sz;
  out.z = cx * cy * sz - sx * sy * cz;
  out.w = cx * cy * cz + sx * sy * sz;

  return out;
};

/**
 * Returns string representation of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {String} - String representation of this quaternion.
 */
quat.str = function str (a)
{
  return ("quat(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ", " + (a.w) + ")");
};

/**
 * Store components of a quaternion into array.
 *
 * @param {Array} out - Array to store result.
 * @param {quat} q - The quaternion.
 * @returns {Array} out.
 */
quat.array = function array (out, q)
{
  out[0] = q.x;
  out[1] = q.y;
  out[2] = q.z;
  out[3] = q.w;

  return out;
};

/**
 * Returns whether the specified quaternions are equal. (Compared using ===)
 *
 * @param {quat} a - The first quaternion.
 * @param {quat} b - The second quaternion.
 * @returns {Boolean} True if the quaternions are equal, false otherwise.
 */
quat.exactEquals = function exactEquals (a, b)
{
	return vec4.exactEquals(a, b);
};

/**
 * Returns whether the specified quaternions are approximately equal.
 *
 * @param {quat} a The first quaternion.
 * @param {quat} b The second quaternion.
 * @returns {Boolean} True if the quaternions are approximately equal, false otherwise.
 */
quat.equals = function equals (a, b)
{
	return vec4.equals(a, b);
};

/**
 * Sets a quaternion to represent the shortest rotation from one
 * vector to another.
 *
 * Both vectors are assumed to be unit length.
 *
 * @param {quat} out the receiving quaternion.
 * @param {vec3} a the initial vector
 * @param {vec3} b the destination vector
 * @returns {quat} out
 * @ignore
 */
quat._rotationTo = (function ()
{
  var tmpvec3 = vec3.zero();
  var xUnitVec3 = vec3.new(1, 0, 0);
  var yUnitVec3 = vec3.new(0, 1, 0);

  return function (out, a, b)
  {
    var dot = vec3.dot(a, b);
    if (dot < -0.999999)
    {
      vec3.cross(tmpvec3, xUnitVec3, a);
      if (vec3.magnitude(tmpvec3) < 0.000001)
      {
        vec3.cross(tmpvec3, yUnitVec3, a);
      }
      vec3.normalize(tmpvec3, tmpvec3);
      quat.fromAxisAngle(out, tmpvec3, Math.PI);
      return out;
    } else if (dot > 0.999999)
    {
      out.x = 0;
      out.y = 0;
      out.z = 0;
      out.w = 1;
      return out;
    } else
    {
      vec3.cross(tmpvec3, a, b);
      out.x = tmpvec3.x;
      out.y = tmpvec3.y;
      out.z = tmpvec3.z;
      out.w = 1 + dot;
      return quat.normalize(out, out);
    }
  };
})();

/**
 * Rotates a quaternion by the given angle about the axis in world space
 *
 * @param {quat} out quat receiving operation result
 * @param {quat} rot quat to rotate
 * @param {vec3} axis the axis around which to rotate in world space
 * @param {number} rad angle (in radians) to rotate
 * @returns {quat} out
 * @ignore
 */
quat._rotateAround = (function ()
{
  var v3_tmp = vec3.zero();
  var q_tmp = quat.create();

  return function (out, rot, axis, rad)
  {
    // get inv-axis (local to rot)
    quat.invert(q_tmp, rot);
    vec3.transformQuat(v3_tmp, axis, q_tmp);
    // rotate by inv-axis
    quat.fromAxisAngle(q_tmp, v3_tmp, rad);
    quat.mul(out, rot, q_tmp);

    return out;
  };
})();

/**
 * Rotates a quaternion by the given angle about the axis in local space
 *
 * @param {quat} out quat receiving operation result
 * @param {quat} rot quat to rotate
 * @param {vec3} axis the axis around which to rotate in local space
 * @param {number} rad angle (in radians) to rotate
 * @returns {quat} out
 * @ignore
 */
quat._rotateAroundLocal = (function ()
{
  var q_tmp = quat.create();

  return function (out, rot, axis, rad)
  {
    quat.fromAxisAngle(q_tmp, axis, rad);
    quat.mul(out, rot, q_tmp);

    return out;
  };
})();

/**
 * Performs a spherical linear interpolation with two control points
 *
 * @param {quat} out the receiving quaternion
 * @param {quat} a the first operand
 * @param {quat} b the second operand
 * @param {quat} c the third operand
 * @param {quat} d the fourth operand
 * @param {Number} t interpolation amount
 * @returns {quat} out
 * @ignore
 */
quat._sqlerp = (function ()
{
  var temp1 = quat.create();
  var temp2 = quat.create();

  return function (out, a, b, c, d, t)
  {
    quat.slerp(temp1, a, d, t);
    quat.slerp(temp2, b, c, t);
    quat.slerp(out, temp1, temp2, 2 * t * (1 - t));

    return out;
  };
}());

/**
 * Sets the specified quaternion with values corresponding to the given
 * axes. Each axis is a vec3 and is expected to be unit length and
 * perpendicular to all other specified axes.
 *
 * @param {vec3} xAxis the vector representing the local "right" direction
 * @param {vec3} yAxis the vector representing the local "up" direction
 * @param {vec3} zAxis the vector representing the viewing direction
 * @returns {quat} out
 * @ignore
 */
quat._fromAxes = (function ()
{
  var matr = mat3.create();

  return function (out, xAxis, yAxis, zAxis)
  {
    mat3.set(matr,
      xAxis.x, xAxis.y, xAxis.z,
      yAxis.x, yAxis.y, yAxis.z,
      zAxis.x, zAxis.y, zAxis.z
    );
    return quat.normalize(out, quat.fromMat3(out, matr));
  };
})();

/**
 * Calculates a quaternion from view direction and up direction
 *
 * @param {quat} out mat3 receiving operation result
 * @param {vec3} view view direction (must be normalized)
 * @param {vec3} [up] up direction, default is (0,1,0) (must be normalized)
 *
 * @returns {quat} out
 * @ignore
 */
quat._fromViewUp = (function ()
{
  var matr = mat3.create();

  return function (out, view, up)
  {
    mat3.fromViewUp(matr, view, up);
    if (!matr)
    {
      return null;
    }

    return quat.normalize(out, quat.fromMat3(out, matr));
  };
})();

/**
 * Represents a color with red(r), green(g), blue(b) component of that color.
 */
var color3 = function color3(r, g, b)
{
/**
 * The r component.
 * @type {number}
 * */
this.r = r;
/**
 * The g component.
 * @type {number}
 * */
this.g = g;
/**
 * The b component.
 * @type {number}
 * */
this.b = b;
};

/**
 * Creates a color, with components specified separately.
 *
 * @param {number} r - Value assigned to r component.
 * @param {number} g - Value assigned to g component.
 * @param {number} b - Value assigned to b component.
 * @return {color3} The newly created color.
 */
color3.new = function new$1 (r, g, b)
{
return new color3(r, g, b);
};

/**
 * Creates a color representing white color.
 *
 * @return {color3} The newly created color.
 */
color3.create = function create ()
{
return new color3(1, 1, 1);
};

/**
 * Clone a color.
 *
 * @param {color3} a - Color to clone.
 * @returns {color3} The newly created color.
 */
color3.clone = function clone (a)
{
  return new color3(a.r, a.g, a.b);
};

/**
 * Copy content of a color into another.
 *
 * @param {color3} out - The color to modified.
 * @param {color3} a - The specified color.
 * @returns {color3} out.
 */
color3.copy = function copy (out, a)
{
out.r = a.r;
out.g = a.g;
out.b = a.b;
return out;
};

/**
 * Set the components of a color to the given values.
 *
 * @param {color3} out - The color to modified.
 * @param {number} r - Value assigned to r component.
 * @param {number} g - Value assigned to g component.
 * @param {number} b - Value assigned to b component.
 * @returns {color3} out.
 */
color3.set = function set (out, r, g, b)
{
  out.r = r;
  out.g = g;
  out.b = b;
  return out;
};

/**
 * Converts the hexadecimal formal color into rgb formal.
 *
 * @param {color3} out - Color to store result.
 * @param {Number} hex - The color's hexadecimal formal.
 * @returns {color3} out.
 * @function
 */
color3.fromHex = function fromHex (out, hex)
{
var r = ((hex >> 16)) / 255.0;
var g = ((hex >> 8) & 0xff) / 255.0;
var b = ((hex) & 0xff) / 255.0;

out.r = r;
out.g = g;
out.b = b;
return out;
};

/**
 * Add components of two colors, respectively.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - The first operand.
 * @param {color3} b - The second operand.
 * @returns {color3} out.
 */
color3.add = function add (out, a, b)
{
out.r = a.r + b.r;
out.g = a.g + b.g;
out.b = a.b + b.b;
return out;
};

/**
 * Subtract components of color b from components of color a, respectively.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - The a.
 * @param {color3} b - The b.
 * @returns {color3} out.
 */
color3.subtract = function subtract (out, a, b)
{
out.r = a.r - b.r;
out.g = a.g - b.g;
out.b = a.b - b.b;
return out;
};

/**
 * Alias of {@link color3.subtract}.
 */
color3.sub = function sub (out, a, b)
{
return color3.subtract(out, a, b);
};

/**
 * Multiply components of two colors, respectively.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - The first operand.
 * @param {color3} b - The second operand.
 * @returns {color3} out.
 */
color3.multiply = function multiply (out, a, b)
{
out.r = a.r * b.r;
out.g = a.g * b.g;
out.b = a.b * b.b;
return out;
};

/**
 * Alias of {@link color3.multiply}.
 */
color3.mul = function mul (out, a, b)
{
return color3.multiply(out, a, b);
};

/**
 * Divide components of color a by components of color b, respectively.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - The first operand.
 * @param {color3} b - The second operand.
 * @returns {color3} out.
 */
color3.divide = function divide (out, a, b)
{
out.r = a.r / b.r;
out.g = a.g / b.g;
out.b = a.b / b.b;
return out;
};

/**
 * Alias of {@link color3.divide}.
 */
color3.div = function div (out, a, b)
{
return color3.divide(out, a, b);
};

/**
 * Scales a color by a number.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - Color to scale.
 * @param {number} b - The scale number.
 * @returns {color3} out.
 */
color3.scale = function scale (out, a, b)
{
out.r = a.r * b;
out.g = a.g * b;
out.b = a.b * b;
return out;
};

/**
 * Performs a linear interpolation between two colors.
 *
 * @param {color3} out - Color to store result.
 * @param {color3} a - The first operand.
 * @param {color3} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {color3} out.
 */
color3.lerp = function lerp$$1 (out, a, b, t)
{
var ar = a.r,
    ag = a.g,
    ab = a.b;
out.r = ar + t * (b.r - ar);
out.g = ag + t * (b.g - ag);
out.b = ab + t * (b.b - ab);
return out;
};

/**
 * Returns string representation of a color.
 *
 * @param {color3} a - The color.
 * @returns {String} - String representation of this color.
 */
color3.str = function str (a)
{
  return ("color3(" + (a.r) + ", " + (a.g) + ", " + (a.b) + ")");
};

/**
 * Store components of a color into array.
 *
 * @param {Array} out - Array to store result.
 * @param {color3} a - The color.
 * @returns {Array} out.
 */
color3.array = function array (out, a)
{
out[0] = a.r;
out[1] = a.g;
out[2] = a.b;

return out;
};

/**
 * Returns whether the specified colors are equal. (Compared using ===)
 *
 * @param {color3} a - The first color.
 * @param {color3} b - The second color.
 * @returns {Boolean} True if the colors are equal, false otherwise.
 */
color3.exactEquals = function exactEquals (a, b)
{
return a.r === b.r && a.g === b.g && a.b === b.b;
};

/**
 * Returns whether the specified colors are approximately equal.
 *
 * @param {color3} a - The first color.
 * @param {color3} b - The second color.
 * @returns {Boolean} True if the colors are approximately equal, false otherwise.
 */
color3.equals = function equals$$1 (a, b)
{
var a0 = a.r, a1 = a.g, a2 = a.b;
var b0 = b.r, b1 = b.g, b2 = b.b;
return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
  Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
  Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)));
};

/**
 * Converts a color's rgb formal into the hexadecimal one.
 *
 * @param {color3} a - The color.
 * @returns {Number} - The color's hexadecimal formal.
 */
color3.hex = function hex (a)
{
return (a.r * 255) << 16 | (a.g * 255) << 8 | (a.b * 255);
};

/**
 * Represents a color with red(r), green(g), blue(b) component of that color and
 * and an extra alpha(a) component indicating how opaque this color is.
 */
var color4 = function color4(r, g, b, a)
{
/**
 * The r component.
 * @type {number}
 * */
this.r = r;
/**
 * The g component.
 * @type {number}
 * */
this.g = g;
/**
 * The b component.
 * @type {number}
 * */
this.b = b;
/**
 * The a component.
 * @type {number}
 * */
this.a = a;
};

/**
 * Creates a color, with components specified separately.
 *
 * @param {number} r - Value assigned to r component.
 * @param {number} g - Value assigned to g component.
 * @param {number} b - Value assigned to b component.
 * @param {number} a - Value assigned to a component.
 * @return {color4} The newly created color.
 */
color4.new = function new$1 (r, g, b, a)
{
return new color4(r, g, b, a);
};

/**
 * Creates a color representing white color.
 *
 * @return {color4} The newly created color.
 */
color4.create = function create ()
{
return new color4(1, 1, 1, 1);
};

/**
 * Clone a color.
 *
 * @param {color4} a - Color to clone.
 * @returns {color4} The newly created color.
 */
color4.clone = function clone (a)
{
  return new color4(a.r, a.g, a.b, a.a);
};

/**
 * Copy content of a color into another.
 *
 * @param {color4} out - The color to modified.
 * @param {color4} a - The specified color.
 * @returns {color4} out.
 */
color4.copy = function copy (out, a)
{
out.r = a.r;
out.g = a.g;
out.b = a.b;
out.a = a.a;
return out;
};

/**
 * Set the components of a color to the given values.
 *
 * @param {color4} out - The color to modified.
 * @param {number} r - Value assigned to r component.
 * @param {number} g - Value assigned to g component.
 * @param {number} b - Value assigned to b component.
 * @param {number} a - Value assigned to a component.
 * @returns {color4} out.
 */
color4.set = function set (out, r, g, b, a)
{
  out.r = r;
  out.g = g;
  out.b = b;
  out.a = a;
  return out;
};

/**
 * Converts the hexadecimal formal color into rgb formal.
 *
 * @param {color4} out - Color to store result.
 * @param {Number} hex - The color's hexadecimal formal.
 * @returns {color4} out.
 * @function
 */
color4.fromHex = function fromHex (out, hex)
{
var r = ((hex >> 24)) / 255.0;
var g = ((hex >> 16) & 0xff) / 255.0;
var b = ((hex >> 8) & 0xff) / 255.0;
var a = ((hex) & 0xff) / 255.0;

out.r = r;
out.g = g;
out.b = b;
out.a = a;
return out;
};

/**
 * Add components of two colors, respectively.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - The first operand.
 * @param {color4} b - The second operand.
 * @returns {color4} out.
 */
color4.add = function add (out, a, b)
{
out.r = a.r + b.r;
out.g = a.g + b.g;
out.b = a.b + b.b;
out.a = a.a + b.a;
return out;
};

/**
 * Subtract components of color b from components of color a, respectively.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - The a.
 * @param {color4} b - The b.
 * @returns {color4} out.
 */
color4.subtract = function subtract (out, a, b)
{
out.r = a.r - b.r;
out.g = a.g - b.g;
out.b = a.b - b.b;
out.a = a.a - b.a;
return out;
};

/**
 * Alias of {@link color4.subtract}.
 */
color4.sub = function sub (out, a, b)
{
return color4.subtract(out, a, b);
};

/**
 * Multiply components of two colors, respectively.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - The first operand.
 * @param {color4} b - The second operand.
 * @returns {color4} out.
 */
color4.multiply = function multiply (out, a, b)
{
out.r = a.r * b.r;
out.g = a.g * b.g;
out.b = a.b * b.b;
out.a = a.a * b.a;
return out;
};

/**
 * Alias of {@link color4.multiply}.
 */
color4.mul = function mul (out, a, b)
{
return color4.multiply(out, a, b);
};

/**
 * Divide components of color a by components of color b, respectively.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - The first operand.
 * @param {color4} b - The second operand.
 * @returns {color4} out.
 */
color4.divide = function divide (out, a, b)
{
out.r = a.r / b.r;
out.g = a.g / b.g;
out.b = a.b / b.b;
out.a = a.a / b.a;
return out;
};

/**
 * Alias of {@link color4.divide}.
 */
color4.div = function div (out, a, b)
{
return color4.divide(out, a, b);
};

/**
 * Scales a color by a number.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - Color to scale.
 * @param {number} b - The scale number.
 * @returns {color4} out.
 */
color4.scale = function scale (out, a, b)
{
out.r = a.r * b;
out.g = a.g * b;
out.b = a.b * b;
out.a = a.a * b;
return out;
};

/**
 * Performs a linear interpolation between two colors.
 *
 * @param {color4} out - Color to store result.
 * @param {color4} a - The first operand.
 * @param {color4} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {color4} out.
 */
color4.lerp = function lerp$$1 (out, a, b, t)
{
var ar = a.r,
    ag = a.g,
    ab = a.b,
    aa = a.a;
out.r = ar + t * (b.r - ar);
out.g = ag + t * (b.g - ag);
out.b = ab + t * (b.b - ab);
out.a = aa + t * (b.a - aa);
return out;
};

/**
 * Returns string representation of a color.
 *
 * @param {color4} a - The color.
 * @returns {String} - String representation of this color.
 */
color4.str = function str (a)
{
  return ("color4(" + (a.r) + ", " + (a.g) + ", " + (a.b) + ", " + (a.a) + ")");
};

/**
 * Store components of a color into array.
 *
 * @param {Array} out - Array to store result.
 * @param {color4} a - The color.
 * @returns {Array} out.
 */
color4.array = function array (out, a)
{
out[0] = a.r;
out[1] = a.g;
out[2] = a.b;
out[3] = a.a;

return out;
};

/**
 * Returns whether the specified colors are equal. (Compared using ===)
 *
 * @param {color4} a - The first color.
 * @param {color4} b - The second color.
 * @returns {Boolean} True if the colors are equal, false otherwise.
 */
color4.exactEquals = function exactEquals (a, b)
{
return a.r === b.r && a.g === b.g && a.b === b.b && a.a === b.a;
};

/**
 * Returns whether the specified colors are approximately equal.
 *
 * @param {color4} a - The first color.
 * @param {color4} b - The second color.
 * @returns {Boolean} True if the colors are approximately equal, false otherwise.
 */
color4.equals = function equals$$1 (a, b)
{
var a0 = a.r, a1 = a.g, a2 = a.b, a3 = a.a;
var b0 = b.r, b1 = b.g, b2 = b.b, b3 = b.a;
return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
  Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
  Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
  Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)));
};

/**
 * Converts a color's rgb formal into the hexadecimal one.
 *
 * @param {color4} a - The color.
 * @returns {Number} - The color's hexadecimal formal.
 */
color4.hex = function hex (a)
{
return ((a.r * 255) << 24 | (a.g * 255) << 16 | (a.b * 255) << 8 | a.a * 255) >>> 0;
};

// NOTE: there is no syntax for: export {* as bits} from './lib/bits';
/**
 * Export module bits.
 */

var easing = {
  linear: {
    none: function (k) {
      return k;
    }
  },
  quadratic: {
    in: function (k) {
      return k * k;
    },
    out: function (k) {
      return k * (2 - k);
    },
    inOut: function (k) {
      if ((k *= 2) < 1) {
        return 0.5 * k * k;
      }
      return - 0.5 * (--k * (k - 2) - 1);
    }
  },
  cubic: {
    in: function (k) {
      return k * k * k;
    },
    out: function (k) {
      return --k * k * k + 1;
    },
    inOut: function (k) {
      if ((k *= 2) < 1) {
        return 0.5 * k * k * k;
      }
      return 0.5 * ((k -= 2) * k * k + 2);
    }
  },
  quartic: {
    in: function (k) {
      return k * k * k * k;
    },
    out: function (k) {
      return 1 - (--k * k * k * k);
    },
    inOut: function (k) {
      if ((k *= 2) < 1) {
        return 0.5 * k * k * k * k;
      }
      return - 0.5 * ((k -= 2) * k * k * k - 2);
    }
  },
  quintic: {
    in: function (k) {
      return k * k * k * k * k;
    },
    out: function (k) {
      return --k * k * k * k * k + 1;
    },
    inOut: function (k) {
      if ((k *= 2) < 1) {
        return 0.5 * k * k * k * k * k;
      }
      return 0.5 * ((k -= 2) * k * k * k * k + 2);
    }
  },
  sinusoidal: {
    in: function (k) {
      return 1 - Math.cos(k * Math.PI / 2);
    },
    out: function (k) {
      return Math.sin(k * Math.PI / 2);
    },
    inOut: function (k) {
      return 0.5 * (1 - Math.cos(Math.PI * k));
    }
  },
  exponential: {
    in: function (k) {
      return k === 0 ? 0 : Math.pow(1024, k - 1);
    },
    out: function (k) {
      return k === 1 ? 1 : 1 - Math.pow(2, - 10 * k);
    },
    inOut: function (k) {
      if (k === 0) {
        return 0;
      }
      if (k === 1) {
        return 1;
      }
      if ((k *= 2) < 1) {
        return 0.5 * Math.pow(1024, k - 1);
      }
      return 0.5 * (- Math.pow(2, - 10 * (k - 1)) + 2);
    }
  },
  circular: {
    in: function (k) {
      return 1 - Math.sqrt(1 - k * k);
    },
    out: function (k) {
      return Math.sqrt(1 - (--k * k));
    },
    inOut: function (k) {
      if ((k *= 2) < 1) {
        return - 0.5 * (Math.sqrt(1 - k * k) - 1);
      }
      return 0.5 * (Math.sqrt(1 - (k -= 2) * k) + 1);
    }
  },
  elastic: {
    in: function (k) {
      if (k === 0) {
        return 0;
      }
      if (k === 1) {
        return 1;
      }
      return -Math.pow(2, 10 * (k - 1)) * Math.sin((k - 1.1) * 5 * Math.PI);
    },
    out: function (k) {
      if (k === 0) {
        return 0;
      }
      if (k === 1) {
        return 1;
      }
      return Math.pow(2, -10 * k) * Math.sin((k - 0.1) * 5 * Math.PI) + 1;
    },
    inOut: function (k) {
      if (k === 0) {
        return 0;
      }
      if (k === 1) {
        return 1;
      }
      k *= 2;
      if (k < 1) {
        return -0.5 * Math.pow(2, 10 * (k - 1)) * Math.sin((k - 1.1) * 5 * Math.PI);
      }
      return 0.5 * Math.pow(2, -10 * (k - 1)) * Math.sin((k - 1.1) * 5 * Math.PI) + 1;
    }
  },
  back: {
    in: function (k) {
      var s = 1.70158;
      return k * k * ((s + 1) * k - s);
    },
    out: function (k) {
      var s = 1.70158;
      return --k * k * ((s + 1) * k + s) + 1;
    },
    inOut: function (k) {
      var s = 1.70158 * 1.525;
      if ((k *= 2) < 1) {
        return 0.5 * (k * k * ((s + 1) * k - s));
      }
      return 0.5 * ((k -= 2) * k * ((s + 1) * k + s) + 2);
    }
  },
  bounce: {
    in: function (k) {
      return 1 - easing.bounce.out(1 - k);
    },
    out: function (k) {
      if (k < (1 / 2.75)) {
        return 7.5625 * k * k;
      } else if (k < (2 / 2.75)) {
        return 7.5625 * (k -= (1.5 / 2.75)) * k + 0.75;
      } else if (k < (2.5 / 2.75)) {
        return 7.5625 * (k -= (2.25 / 2.75)) * k + 0.9375;
      } else {
        return 7.5625 * (k -= (2.625 / 2.75)) * k + 0.984375;
      }
    },
    inOut: function (k) {
      if (k < 0.5) {
        return easing.bounce.in(k * 2) * 0.5;
      }
      return easing.bounce.out(k * 2 - 1) * 0.5 + 0.5;
    }
  }
};

var _vec2 = vec2.zero();
var _vec3 = vec3.zero();
var _vec4 = vec4.zero();
var _color3 = color3.create();
var _color4 = color4.create();
var _quat = quat.create();

function getCloneFromType(type, value) {
  if (type === 'number') {
    return value;
  } else if (type === 'vec2') {
    return vec2.clone(value);
  } else if (type === 'vec3') {
    return vec3.clone(value);
  } else if (type === 'vec4') {
    return vec4.clone(value);
  } else if (type === 'color3') {
    return color3.clone(value);
  } else if (type === 'color4') {
    return color4.clone(value);
  } else if (type === 'quat') {
    return quat.clone(value);
  }

  return value;
}

function parseValue(type, value) {
  var ret;

  if (type === 'number') {
    return value;
  } else if (type === 'vec2') {
    ret = vec2.zero();
    vec2.set(ret, value[0], value[1]);
    return ret;
  } else if (type === 'vec3') {
    ret = vec3.zero();
    vec3.set(ret, value[0], value[1], value[2]);
    return ret;
  } else if (type === 'vec4') {
    ret = vec4.zero();
    vec4.set(ret, value[0], value[1], value[2], value[3]);
    return ret;
  } else if (type === 'color3') {
    ret = color3.create();
    color3.set(ret, value[0], value[1], value[2]);
    return ret;
  } else if (type === 'color4') {
    ret = color4.create();
    color4.set(ret, value[0], value[1], value[2], value[3]);
    return ret;
  } else if (type === 'quat') {
    ret = quat.create();
    quat.set(ret, value[0], value[1], value[2], value[3]);
    return ret;
  }

  return null;
}

function getLerpFunc(type) {
  if (type === 'number') {
    return function (from, to, t) { var out = ((to - from) * t) + from; return out; };
  } else if (type === 'vec2') {
    return function (from, to, t) { return vec2.lerp(_vec2, from, to, t); };
  } else if (type === 'vec3') {
    return function (from, to, t) { return vec3.lerp(_vec3, from, to, t); };
  } else if (type === 'vec4') {
    return function (from, to, t) { return vec4.lerp(_vec4, from, to, t); };
  } else if (type === 'color3') {
    return function (from, to, t) { return color3.lerp(_color3, from, to, t); };
  } else if (type === 'color4') {
    return function (from, to, t) { return color4.lerp(_color4, from, to, t); };
  } else if (type === 'quat') {
    return function (from, to, t) { return quat.lerp(_quat, from, to, t); };
  }
}

function findMaxDuration(tracks) {
  var maxDuration = 0;
  for (var i = 0; i < tracks.length; i++) {
    if (tracks[i].duration >= maxDuration) {
      maxDuration = tracks[i].duration;
    }
  }
  return maxDuration;
}

function _createKeys(targetProperty, keysProp, keyType, opts) {
  var keys = [];
  var prevKey;
  var type = keyType;

  if (Array.isArray(keysProp) === false) {
    keysProp = [keysProp];
  }

  for (var i = 0; i < keysProp.length; i++) {
    var keyProp = keysProp[i];

    var duration = keysProp[i].duration;
    if (duration === undefined) {
      duration = opts.duration;
    }

    var delay = keyProp.delay;
    if (delay === undefined) {
      delay = opts.delay;
    }

    var easing$$1 = keyProp.easing;
    if (easing$$1 === undefined) {
      easing$$1 = opts.easing;
    }

    var elasticity = keyProp.elasticity;
    if (elasticity === undefined) {
      elasticity = opts.elasticity;
    }

    var to = (void 0);
    var from = (void 0);

    if (keyProp.value[0] instanceof Array === true) {
      from = parseValue(type, keyProp.value[0]);
      to = parseValue(type, keyProp.value[1]);
    } else {
      from = getCloneFromType(type, targetProperty);
      to = parseValue(type, keyProp.value);
    }

    var startTime = prevKey ? prevKey.endTime : 0;

    var key = {
      duration: duration,
      delay: delay,
      easing: easing$$1,
      elasticity: elasticity,
      startTime: startTime,
      endTime: startTime + duration + delay,
      to: to,
      from: prevKey ? prevKey.to : from,
    };

    prevKey = key;
    keys.push(key);
  }

  return keys;
}

function _createTracks(targets, props, opts) {
  var tracks = [];

  if (targets instanceof Array) {
    for (var i = 0; i < targets.length; ++i) {
      var target = targets[i];

      for (var name in props) {
        var propInfo = props[name];

        var keys = _createKeys(target[name], propInfo.keys, propInfo.type, opts);

        var type = propInfo.type;

        tracks.push({
          target: target,
          property: name,
          type: type,
          keys: keys,
          duration: keys[keys.length - 1].endTime,
          delay: keys[0].delay,
          lerp: getLerpFunc(type),
        });
      }
    }
  }

  return tracks;
}

var Task = function Task(engine, targets, props, opts) {
  this._next = null;
  this._prev = null;
  this._engine = engine;

  // animation options
  this.loop = 1;
  if (opts.loop !== undefined) {
    this.loop = opts.loop;
  }

  this.direction = 'normal';
  if (opts.direction !== undefined) {
    this.direction = opts.direction;
  }

  this.autoplay = true;
  if (opts.autoplay !== undefined) {
    this.autoplay = opts.autoplay;
  }

  this.duration = 1000;
  if (opts.duration !== undefined) {
    this.duration = opts.duration;
  }

  // callbacks
  this.update = opts.update;
  this.start = opts.start;
  this.run = opts.run;
  this.end = opts.end;

  // create animations
  this.tracks = _createTracks(targets, props, opts);
  this.duration = findMaxDuration(this.tracks);

  this.paused = true;
  this.began = false;
  this.completed = false;
  this.reversed = false;

  this.currentTime = 0;
  this.remaining = 0;
  this.now = 0;
  this.startTime = 0;
  this.lastTime = 0;

  // children
  this.children = [];

  this.reset();

  if (this.autoplay) {
    this.play();
  }
};

Task.prototype.adjustTime = function adjustTime (time) {
  return this.reversed ? this.duration - time : time;
};

Task.prototype.toggle = function toggle () {
  this.reversed = !this.reversed;
};

Task.prototype.countIteration = function countIteration () {
  if (this.remaining && this.remaining !== true) {
    this.remaining--;
  }
};

Task.prototype.invokeCallback = function invokeCallback (cb) {
  if (this[cb]) {
    this[cb](this);
  }
};

Task.prototype.syncChildren = function syncChildren (time) {
  var children = this.children;
  var childrenLength = children.length;

  if (time >= this.currentTime) {
    for (var i = 0; i < childrenLength; i++) {
      children[i].seek(time);
    }
  } else {
    for (var i$1 = childrenLength; i$1--;) {
      children[i$1].seek(time);
    }
  }
};

Task.prototype._setTracksProgress = function _setTracksProgress (time) {
    var this$1 = this;

  for (var i = 0; i < this.tracks.length; i++) {
    var track = this$1.tracks[i];
    for (var j = 0; j < track.keys.length; j++) {
      var key = track.keys[j];
      if (key.startTime <= time && key.endTime >= time) {
        var elapsed = clamp(time - key.startTime - key.delay, 0, key.duration) / key.duration;
        var eased = isNaN(elapsed) ? 1 : key.easing(elapsed);

        track.target[track.property] = track.lerp(key.from, key.to, eased);

        break;
      } else if (key.startTime > time) {
        track.target[track.property] = track.lerp(key.from, key.to, 0);
      } else if (key.endTime < time) {
        track.target[track.property] = track.lerp(key.from, key.to, 1);
      }
    }
  }
  this.currentTime = time;
};

Task.prototype._setProgress = function _setProgress (t) {
  var duration = this.duration;
  var time = this.adjustTime(t);
  if (this.children.length) {
    this.syncChildren(time);
  }

  if (time >= 0 || !duration) {
    if (!this.began) {
      this.began = true;
      this.invokeCallback('start');
    }
    this.invokeCallback('run');
  }

  if (time > 0 && time < duration) {
    this._setTracksProgress(time);
  } else {
    if (time <= 0 && this.currentTime !== 0) {
      this._setTracksProgress(0);
      if (this.reversed) {
        this.countIteration();
      }
    }
    if ((time >= duration && this.currentTime !== duration) || !duration) {
      this._setTracksProgress(duration);
      if (!this.reversed) {
        this.countIteration();
      }
    }
  }

  this.invokeCallback('update');

  if (t >= duration) {
    if (this.remaining) {
      this.startTime = this.now;
      if (this.direction === 'alternate') {
        this.toggle();
      }
    } else {
      this.pause();
      if (!this.completed) {
        this.completed = true;
        this.invokeCallback('end');
      }
    }
    this.lastTime = 0;
  }
};

Task.prototype.tick = function tick (t) {
  this.now = t;
  if (!this.startTime) {
    this.startTime = this.now;
  }
  var _t = this.lastTime + this.now - this.startTime;
  this._setProgress(_t);
};

Task.prototype.seek = function seek (time) {
  this._setProgress(this.adjustTime(time));
};

Task.prototype.pause = function pause () {
  this._engine._remove(this);
  this.paused = true;
};

Task.prototype.play = function play () {
  if (!this.paused) {
    return;
  }
  this.paused = false;
  this.startTime = 0;
  this.lastTime = this.adjustTime(this.currentTime);

  this._engine._add(this);
};

Task.prototype.reverse = function reverse () {
  this.toggle();
  this.startTime = 0;
  this.lastTime = this.adjustTime(this.currentTime);
};

Task.prototype.restart = function restart () {
  this.pause();
  this.reset();
  this.play();
};

Task.prototype.reset = function reset () {
    var this$1 = this;

  var direction = this.direction;
  var loops = this.loop;
  this.currentTime = 0;
  this.progress = 0;
  this.paused = true;
  this.began = false;
  this.completed = false;
  this.reversed = direction === 'reverse';
  this.remaining = direction === 'alternate' && loops === 1 ? 2 : loops;
  this._setTracksProgress(0);
  for (var i = this.children.length; i--;) {
    this$1.children[i].reset();
  }
};

//core
var VTween = function VTween() {
  this._activeTasks = new LinkedArray();
};

VTween.prototype.newTask = function newTask (targets, props, opts) {
  if (opts.duration === undefined) {
    opts.duration = 1000;
  }
  if (opts.delay === undefined) {
    opts.delay = 0;
  }
  if (opts.easing === undefined) {
    opts.easing = easing.linear.none;
  }
  if (opts.elasticity === undefined) {
    opts.elasticity = 500;
  }

  if (Array.isArray(targets) === false) {
    targets = [targets];
  }

  return new Task(this, targets, props, opts);
};

VTween.prototype.newTimeLine = function newTimeLine (opts) {
  opts.autoplay = false;
  var timeLine = new Task(this, {}, {}, opts);
  timeLine.pause();
    
  timeLine.add = function (task) {
    if (timeLine.duration <= task.duration) {
      timeLine.duration = task.duration;
    }
    timeLine.pause();
    task.pause();
    task.began = true;
    task.completed = true;
    task.direction = timeLine.direction;
    task.loops = timeLine.loop;
      
    timeLine.children.push(task);
  };

  return timeLine;
};

VTween.prototype._add = function _add (task) {
  this._activeTasks.add(task);
};

VTween.prototype._remove = function _remove (task) {
  this._activeTasks.remove(task);
};

VTween.prototype.tick = function tick (dt) {
  var cursor = this._activeTasks.head;
  var next = cursor;

  while (cursor) {
    next = cursor._next;
    cursor.tick(dt);
    cursor = next;
  }
};

exports.VTween = VTween;
exports.easing = easing;
//# sourceMappingURL=vtween.js.map
