
/*
 * engine-3d v0.13.0
 * (c) 2018 @cocos
 * Released under the MIT License.
 */

var sceneGraph = (function (exports) {
'use strict';

/**
 * @ignore
 */
var _d2r = Math.PI / 180.0;
/**
 * @ignore
 */
var _r2d = 180.0 / Math.PI;

/**
 * @property {number} EPSILON
 */
var EPSILON = 0.000001;

/**
 * Tests whether or not the arguments have approximately the same value, within an absolute
 * or relative tolerance of glMatrix.EPSILON (an absolute tolerance is used for values less
 * than or equal to 1.0, and a relative tolerance is used for larger values)
 *
 * @param {Number} a The first number to test.
 * @param {Number} b The second number to test.
 * @returns {Boolean} True if the numbers are approximately equal, false otherwise.
 */


/**
 * Tests whether or not the arguments have approximately the same value by given maxDiff
 *
 * @param {Number} a The first number to test.
 * @param {Number} b The second number to test.
 * @param {Number} maxDiff Maximum difference.
 * @returns {Boolean} True if the numbers are approximately equal, false otherwise.
 */


/**
 * Clamps a value between a minimum float and maximum float value.
 *
 * @method clamp
 * @param {number} val
 * @param {number} min
 * @param {number} max
 * @return {number}
 */


/**
 * Clamps a value between 0 and 1.
 *
 * @method clamp01
 * @param {number} val
 * @return {number}
 */


/**
 * @method lerp
 * @param {number} from
 * @param {number} to
 * @param {number} ratio - the interpolation coefficient
 * @return {number}
 */


/**
* Convert Degree To Radian
*
* @param {Number} a Angle in Degrees
*/


/**
* Convert Radian To Degree
*
* @param {Number} a Angle in Radian
*/


/**
* @method random
*/
var random = Math.random;

/**
 * Returns a floating-point random number between min (inclusive) and max (exclusive).
 *
 * @method randomRange
 * @param {number} min
 * @param {number} max
 * @return {number} the random number
 */


/**
 * Returns a random integer between min (inclusive) and max (exclusive).
 *
 * @method randomRangeInt
 * @param {number} min
 * @param {number} max
 * @return {number} the random integer
 */


/**
 * Linear congruential generator using Hull-Dobell Theorem.
 *
 * @method pseudoRandom
 * @param {number} seed the random seed
 * @return {number} the pseudo random
 */


/**
 * Returns a floating-point pseudo-random number between min (inclusive) and max (exclusive).
 *
 * @method pseudoRandomRange
 * @param {number} seed
 * @param {number} min
 * @param {number} max
 * @return {number} the random number
 */


/**
 * Returns a pseudo-random integer between min (inclusive) and max (exclusive).
 *
 * @method pseudoRandomRangeInt
 * @param {number} seed
 * @param {number} min
 * @param {number} max
 * @return {number} the random integer
 */


/**
 * Returns the next power of two for the value
 *
 * @method nextPow2
 * @param {number} val
 * @return {number} the the next power of two
 */


/**
 * Returns float remainder for t / length
 *
 * @method repeat
 * @param {number} t time start at 0
 * @param {number} length time of one cycle
 * @return {number} the time wrapped in the first cycle
 */


/**
 * Returns time wrapped in ping-pong mode
 *
 * @method repeat
 * @param {number} t time start at 0
 * @param {number} length time of one cycle
 * @return {number} the time wrapped in the first cycle
 */


/**
 * Returns ratio of a value within a given range
 *
 * @method repeat
 * @param {number} from start value
 * @param {number} to end value
 * @param {number} value given value
 * @return {number} the ratio between [from,to]
 */

/**
 * Bit twiddling hacks for JavaScript.
 *
 * Author: Mikola Lysenko
 *
 * Ported from Stanford bit twiddling hack library:
 *    http://graphics.stanford.edu/~seander/bithacks.html
 */

// Number of bits in an integer
/**
 * @ignore
 */

/**
 * @ignore
 */

/**
 * @ignore
 */


/**
 * Returns -1, 0, +1 depending on sign of x
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes absolute value of integer
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes minimum of integers x and y
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Computes maximum of integers x and y
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Checks if a number is a power of two
 *
 * @param {number} v
 * @returns {boolean}
 */


/**
 * Computes log base 2 of v
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes log base 10 of v
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Counts number of bits
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Counts number of trailing zeros
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Rounds to next power of 2
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Rounds down to previous power of 2
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Computes parity of word
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * @ignore
 */
var REVERSE_TABLE = new Array(256);

(function(tab) {
  for(var i=0; i<256; ++i) {
    var v = i, r = i, s = 7;
    for (v >>>= 1; v; v >>>= 1) {
      r <<= 1;
      r |= v & 1;
      --s;
    }
    tab[i] = (r << s) & 0xff;
  }
})(REVERSE_TABLE);

/**
 * Reverse bits in a 32 bit word
 *
 * @param {number} v
 * @returns {number}
 */


/**
 * Interleave bits of 2 coordinates with 16 bits. Useful for fast quadtree codes
 *
 * @param {number} x
 * @param {number} y
 * @returns {number}
 */


/**
 * Extracts the nth interleaved component
 *
 * @param {number} v
 * @param {number} n
 * @returns {number}
 */


/**
 * Interleave bits of 3 coordinates, each with 10 bits.  Useful for fast octree codes
 *
 * @param {number} x
 * @param {number} y
 * @param {number} z
 * @returns {number}
 */


/**
 * Extracts nth interleaved component of a 3-tuple
 *
 * @param {number} v
 * @param {number} n
 * @returns {number}
 */


/**
 * Computes next combination in colexicographic order (this is mistakenly called nextPermutation on the bit twiddling hacks page)
 *
 * @param {number} v
 * @returns {number}
 */

/**
 * Mathematical 2-dimensional vector.
 *
 * x, y is alias of the first, second component of vector, respectively.
 */
var vec2 = function vec2(x, y)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
};

/**
 * Creates a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @return {vec2} The newly created vector.
 */
vec2.new = function new$1 (x, y)
{
  return new vec2(x, y);
};

/**
 * Creates a zero vector.
 *
 * @return {vec2} The newly created vector.
 */
vec2.zero = function zero ()
{
  return new vec2(0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec2} a - Vector to clone.
 * @returns {vec2} The newly created vector.
 */
vec2.clone = function clone (a)
{
  return new vec2(a.x, a.y);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec2} out - The vector to modified.
 * @param {vec2} a - The specified vector.
 * @returns {vec2} out.
 */
vec2.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec2} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @returns {vec2} out.
 */
vec2.set = function set (out, x, y)
{
  out.x = x;
  out.y = y;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  return out;
};

/**
 *Alias of {@link vec2.subtract}.
 */
vec2.sub = function sub (out, a, b)
{
  return vec2.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  return out;
};

/**
 *Alias of {@link vec2.multiply}.
 */
vec2.mul = function mul (out, a, b)
{
  return vec2.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  return out;
};

/**
 *Alias of {@link vec2.divide}.
 */
vec2.div = function div (out, a, b)
{
  return vec2.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec2} out.
 */
vec2.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to perform operation.
 * @returns {vec2} out.
 */
vec2.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec2} out.
 * */
vec2.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec2} out.
 */
vec2.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec2.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y;
  return Math.sqrt(x * x + y * y);
};

/**
 *Alias of {@link vec2.distance}.
 */
vec2.dist = function dist (a, b)
{
  return vec2.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec2.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y;
  return x * x + y * y;
};

/**
 *Alias of {@link vec2.squaredDistance}.
 */
vec2.sqrDist = function sqrDist (a, b)
{
  return vec2.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec2.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y;
  return Math.sqrt(x * x + y * y);
};

/**
 *Alias of {@link vec2.magnitude}.
 */
vec2.mag = function mag (a)
{
  return vec2.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec2.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y;
  return x * x + y * y;
};

/**
 *Alias of {@link vec2.squaredMagnitude}
 */
vec2.sqrMag = function sqrMag (a)
{
  return vec2.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to negate.
 * @returns {vec2} out.
 */
vec2.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  return out;
};

/**
 * Invert the components of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to invert.
 * @returns {vec2} out.
 */
vec2.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  return out;
};

/**
 * Safely invert the components of a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to invert.
 * @returns {vec2} out.
 */
vec2.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / a.y;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to normalize.
 * @returns {vec2} out.
 */
vec2.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y;
  var len = x * x + y * y;
  if (len > 0)
  {
    //TODO: evaluate use of glm_invsqrt here?
    len = 1 / Math.sqrt(len);
    out.x = a.x * len;
    out.y = a.y * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec2.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y;
};

/**
 * Calculate the cross product of two vectors.
 * Note that the cross product must by definition produce a 3D vector.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @returns {vec3} out.
 */
vec2.cross = function cross (out, a, b)
{
  var z = a.x * b.y - a.y * b.x;
  out.x = out.y = 0;
  out.z = z;
  return out;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - The first operand.
 * @param {vec2} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec2} out.
 */
vec2.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec2} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec2} out.
 */
vec2.random = function random$1 (out, scale)
{
  scale = scale || 1.0;
  var r = random() * 2.0 * Math.PI;
  out.x = Math.cos(r) * scale;
  out.y = Math.sin(r) * scale;
  return out;
};

/**
 * Transforms a vector with a 2x2 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat2} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat2 = function transformMat2 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m02 * y;
  out.y = m.m01 * x + m.m03 * y;
  return out;
};

/**
 * Transforms a vector with a 2x3 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat23} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat23 = function transformMat23 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m02 * y + m.m04;
  out.y = m.m01 * x + m.m03 * y + m.m05;
  return out;
};

/**
 * Transforms a vector with a 3x3 matrix.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat3} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat3 = function transformMat3 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m03 * y + m.m06;
  out.y = m.m01 * x + m.m04 * y + m.m07;
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix.
 * 3rd vector component is implicitly '0'.
 * 4th vector component is implicitly '1'.
 *
 * @param {vec2} out - Vector to store result.
 * @param {vec2} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec2} out.
 */
vec2.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x,
    y = a.y;
  out.x = m.m00 * x + m.m04 * y + m.m12;
  out.y = m.m01 * x + m.m05 * y + m.m13;
  return out;
};

/**
 * Perform some operation over an array of vec2s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec2.forEach = function forEach (a, stride, offset, count, fn, arg)
{
  return vec2._forEach(a, stride, offset, count, fn, arg);
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec2} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec2.str = function str (a)
{
  return ("vec2(" + (a.x) + ", " + (a.y) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec2} v - The vector.
 * @returns {Array} out.
 */
vec2.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec2} a - The first vector.
 * @param {vec2} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec2.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec2} a The first vector.
 * @param {vec2} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec2.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y;
  var b0 = b.x, b1 = b.y;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)));
};

/**
 * Perform some operation over an array of vec2s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec2. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec2s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 * @ignore
 */
vec2._forEach = (function ()
{
  var vec = vec2.zero();

  return function (a, stride, offset, count, fn, arg)
  {
    var i, l;
    if (!stride)
    {
      stride = 2;
    }

    if (!offset)
    {
      offset = 0;
    }

    if (count)
    {
      l = Math.min((count * stride) + offset, a.length);
    } else
    {
      l = a.length;
    }

    for (i = offset; i < l; i += stride)
    {
      vec.x = a[i];
      vec.y = a[i + 1];
      fn(vec, vec, arg);
      a[i] = vec.x;
      a[i + 1] = vec.y;
    }

    return a;
  };
})();

/**
 * Mathematical 3-dimensional vector.
 *
 * x, y, z is alias of the first, second, third component of vector, respectively.
 */
var vec3 = function vec3(x, y, z)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
};

/**
 * Creates a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @return {vec3} The newly created vector.
 */
vec3.new = function new$1 (x, y, z)
{
  return new vec3(x, y, z);
};

/**
 * Creates a zero vector.
 *
 * @return {vec3} The newly created vector.
 */
vec3.zero = function zero ()
{
  return vec3.new(0, 0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec3} a - Vector to clone.
 * @returns {vec3} The newly created vector.
 */
vec3.clone = function clone (a)
{
  return new vec3(a.x, a.y, a.z);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec3} out - The vector to modified.
 * @param {vec3} a - The specified vector.
 * @returns {vec3} out.
 */
vec3.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  out.z = a.z;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec3} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @returns {vec3} out.
 */
vec3.set = function set (out, x, y, z)
{
  out.x = x;
  out.y = y;
  out.z = z;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  out.z = a.z + b.z;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  out.z = a.z - b.z;
  return out;
};

/**
 * Alias of {@link vec3.subtract}.
 */
vec3.sub = function sub (out, a, b)
{
	return vec3.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  out.z = a.z * b.z;
  return out;
};

/**
 * Alias of {@link vec3.multiply}.
 */
vec3.mul = function mul (out, a, b)
{
	return vec3.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  out.z = a.z / b.z;
  return out;
};

/**
 * Alias of {@link vec3.divide}.
 */
vec3.div = function div (out, a, b)
{
	return vec3.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  out.z = Math.ceil(a.z);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  out.z = Math.floor(a.z);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  out.z = Math.min(a.z, b.z);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  out.z = Math.max(a.z, b.z);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to perform operation.
 * @returns {vec3} out.
 */
vec3.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  out.z = Math.round(a.z);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec3} out.
 * */
vec3.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec3} out.
 */
vec3.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  out.z = a.z + (b.z * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec3.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z;
  return Math.sqrt(x * x + y * y + z * z);
};

/**
 * Alias of {@link vec3.distance}.
 */
vec3.dist = function dist (a, b)
{
	return vec3.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec3.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z;
  return x * x + y * y + z * z;
};

/**
 * Alias of {@link vec3.squaredDistance}.
 */
vec3.sqrDist = function sqrDist (a, b)
{
	return vec3.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec3.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z;
  return Math.sqrt(x * x + y * y + z * z);
};

/**
 *Alias of {@link vec3.magnitude}.
 */
vec3.mag = function mag (a)
{
  return vec3.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec3.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z;
  return x * x + y * y + z * z;
};

/**
 *Alias of {@link vec3.squaredMagnitude}
 */
vec3.sqrMag = function sqrMag (a)
{
  return vec3.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to negate.
 * @returns {vec3} out.
 */
vec3.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  return out;
};

/**
 * Inverts the components of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to invert.
 * @returns {vec3} out.
 */
vec3.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  out.z = 1.0 / a.z;
  return out;
};

/**
 * Safely inverts the components of a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to invert.
 * @returns {vec3} out.
 */
vec3.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / y;
  }

  if (Math.abs(z) < EPSILON)
  {
    out.z = 0;
  } else
  {
    out.z = 1.0 / z;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to normalize.
 * @returns {vec3} out.
 */
vec3.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z;

  var len = x * x + y * y + z * z;
  if (len > 0)
  {
    //TODO: evaluate use of glm_invsqrt here?
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec3.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z;
};

/**
 * Calculates the cross product of two vectors.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {vec3} out.
 */
vec3.cross = function cross (out, a, b)
{
  var ax = a.x, ay = a.y, az = a.z,
    bx = b.x, by = b.y, bz = b.z;

  out.x = ay * bz - az * by;
  out.y = az * bx - ax * bz;
  out.z = ax * by - ay * bx;
  return out;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  return out;
};

/**
 * Performs a hermite interpolation with two control points.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {vec3} c - The third operand.
 * @param {vec3} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.hermite = function hermite (out, a, b, c, d, t)
{
  var factorTimes2 = t * t,
    factor1 = factorTimes2 * (2 * t - 3) + 1,
    factor2 = factorTimes2 * (t - 2) + t,
    factor3 = factorTimes2 * (t - 1),
    factor4 = factorTimes2 * (3 - 2 * t);

  out.x = a.x * factor1 + b.x * factor2 + c.x * factor3 + d.x * factor4;
  out.y = a.y * factor1 + b.y * factor2 + c.y * factor3 + d.y * factor4;
  out.z = a.z * factor1 + b.z * factor2 + c.z * factor3 + d.z * factor4;

  return out;
};

/**
 * Performs a bezier interpolation with two control points.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @param {vec3} c - The third operand.
 * @param {vec3} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec3} out.
 */
vec3.bezier = function bezier (out, a, b, c, d, t)
{
  var inverseFactor = 1 - t,
    inverseFactorTimesTwo = inverseFactor * inverseFactor,
    factorTimes2 = t * t,
    factor1 = inverseFactorTimesTwo * inverseFactor,
    factor2 = 3 * t * inverseFactorTimesTwo,
    factor3 = 3 * factorTimes2 * inverseFactor,
    factor4 = factorTimes2 * t;

  out.x = a.x * factor1 + b.x * factor2 + c.x * factor3 + d.x * factor4;
  out.y = a.y * factor1 + b.y * factor2 + c.y * factor3 + d.y * factor4;
  out.z = a.z * factor1 + b.z * factor2 + c.z * factor3 + d.z * factor4;

  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec3} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec3} out.
 */
vec3.random = function random$1 (out, scale)
{
  scale = scale || 1.0;

  var r = random() * 2.0 * Math.PI;
  var z = (random() * 2.0) - 1.0;
  var zScale = Math.sqrt(1.0 - z * z) * scale;

  out.x = Math.cos(r) * zScale;
  out.y = Math.sin(r) * zScale;
  out.z = z * scale;
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix. 4th vector component is implicitly '1'.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec3} out.
 */
vec3.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z,
    w = m.m03 * x + m.m07 * y + m.m11 * z + m.m15;
  w = w || 1.0;
  out.x = (m.m00 * x + m.m04 * y + m.m08 * z + m.m12) / w;
  out.y = (m.m01 * x + m.m05 * y + m.m09 * z + m.m13) / w;
  out.z = (m.m02 * x + m.m06 * y + m.m10 * z + m.m14) / w;
  return out;
};

/**
 * Transforms a vector with a 3x3 matrix.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {mat3} m - The matrix.
 * @returns {vec3} out.
 */
vec3.transformMat3 = function transformMat3 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z;
  out.x = x * m.m00 + y * m.m03 + z * m.m06;
  out.y = x * m.m01 + y * m.m04 + z * m.m07;
  out.z = x * m.m02 + y * m.m05 + z * m.m08;
  return out;
};

/**
 * Transforms a vector with a quaternion.
 *
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - Vector to transform.
 * @param {quat} q - The quaternion.
 * @returns {vec3} out.
 */
vec3.transformQuat = function transformQuat (out, a, q)
{
  // benchmarks: http://jsperf.com/quaternion-transform-vec3-implementations

  var x = a.x, y = a.y, z = a.z;
  var qx = q.x, qy = q.y, qz = q.z, qw = q.w;

  // calculate quat * vec
  var ix = qw * x + qy * z - qz * y;
  var iy = qw * y + qz * x - qx * z;
  var iz = qw * z + qx * y - qy * x;
  var iw = -qx * x - qy * y - qz * z;

  // calculate result * inverse quat
  out.x = ix * qw + iw * -qx + iy * -qz - iz * -qy;
  out.y = iy * qw + iw * -qy + iz * -qx - ix * -qz;
  out.z = iz * qw + iw * -qz + ix * -qy - iy * -qx;
  return out;
};

/**
 * Rotates a 3D vector around the x-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateX = function rotateX (out, a, b, c)
{
  var p = [], r = [];
  // Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.x;
  r.y = p.y * Math.cos(c) - p.z * Math.sin(c);
  r.z = p.y * Math.sin(c) + p.z * Math.cos(c);

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Rotates a 3D vector around the y-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateY = function rotateY (out, a, b, c)
{
  var p = [], r = [];
  //Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.z * Math.sin(c) + p.x * Math.cos(c);
  r.y = p.y;
  r.z = p.z * Math.cos(c) - p.x * Math.sin(c);

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Rotates a 3D vector around the z-axis.
 * @param {vec3} out - Vector to store result.
 * @param {vec3} a - The point to rotate.
 * @param {vec3} b - The origin of the rotation.
 * @param {Number} c - The angle of rotation.
 * @returns {vec3} out.
 */
vec3.rotateZ = function rotateZ (out, a, b, c)
{
  var p = [], r = [];
  //Translate point to the origin
  p.x = a.x - b.x;
  p.y = a.y - b.y;
  p.z = a.z - b.z;

  //perform rotation
  r.x = p.x * Math.cos(c) - p.y * Math.sin(c);
  r.y = p.x * Math.sin(c) + p.y * Math.cos(c);
  r.z = p.z;

  //translate to correct position
  out.x = r.x + b.x;
  out.y = r.y + b.y;
  out.z = r.z + b.z;

  return out;
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec3} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec3.str = function str (a)
{
  return ("vec3(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec3} v - The vector.
 * @returns {Array} out.
 */
vec3.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;
  out[2] = v.z;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec3} a - The first vector.
 * @param {vec3} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec3.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y && a.z === b.z;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec3} a The first vector.
 * @param {vec3} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec3.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y, a2 = a.z;
  var b0 = b.x, b1 = b.y, b2 = b.z;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
    Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)));
};
  
/**
 * Perform some operation over an array of vec3s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec3.forEach = function forEach (a, stride, offset, count, fn, arg)
{
	return vec3._forEach(a, stride, offset, count, fn, arg);
};
  
/**
 * Gets the angle between two 3D vectors.
 * @param {vec3} a - The first operand.
 * @param {vec3} b - The second operand.
 * @returns {Number} - The angle in radians.
 */
vec3.angle = function angle (a, b)
{
	return vec3._angle(a, b);
};

/**
* Perform some operation over an array of vec3s.
*
* @param {Array} a the array of vectors to iterate over.
* @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed.
* @param {Number} offset Number of elements to skip at the beginning of the array.
* @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array.
* @param {Function} fn Function to call for each vector in the array.
* @param {Object} [arg] additional argument to pass to fn.
* @returns {Array} a.
* @ignore.
*/
vec3._forEach = (function ()
{
var vec = vec3.zero();

return function (a, stride, offset, count, fn, arg)
{
  var i, l;
  if (!stride)
  {
	stride = 3;
  }

  if (!offset)
  {
	offset = 0;
  }

  if (count)
  {
	l = Math.min((count * stride) + offset, a.length);
  } else
  {
	l = a.length;
  }

  for (i = offset; i < l; i += stride)
  {
	vec.x = a[i];
	vec.y = a[i + 1];
	vec.z = a[i + 2];
	fn(vec, vec, arg);
	a[i] = vec.x;
	a[i + 1] = vec.y;
	a[i + 2] = vec.z;
  }

  return a;
};
})();

  /**
   * Get the angle between two 3D vectors.
   * @param {vec3} a The first operand.
   * @param {vec3} b The second operand.
   * @returns {Number} The angle in radians.
   * @ignore
   */
vec3._angle = (function ()
{
var tempA = vec3.zero();
var tempB = vec3.zero();

return function (a, b)
{
  vec3.copy(tempA, a);
  vec3.copy(tempB, b);

  vec3.normalize(tempA, tempA);
  vec3.normalize(tempB, tempB);

  var cosine = vec3.dot(tempA, tempB);

  if (cosine > 1.0)
  {
	return 0;
  }

  if (cosine < -1.0)
  {
	return Math.PI;
  }

  return Math.acos(cosine);
};
})();

/**
 * Mathematical 4-dimensional vector.
 *
 * x, y, z, w is alias of the first, second, third, fourth component of vector, respectively.
 */
var vec4 = function vec4(x, y, z, w)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
  /**
   * The w component.
   * @type {number}
   * */
  this.w = w;
};

/**
 * Create a vector, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @param {number} w - Value assigned to w component.
 * @return {vec4} The newly created vector.
 */
vec4.new = function new$1 (x, y, z, w)
{
  return new vec4(x, y, z, w);
};

/**
 * Creates a zero vector.
 *
 * @return {vec4} The newly created vector.
 */
vec4.zero = function zero ()
{
  return vec4.new(0, 0, 0, 0);
};

/**
 * Clone a vector.
 *
 * @param {vec4} a - Vector to clone.
 * @returns {vec4} The newly created vector.
 */
vec4.clone = function clone (a)
{
  return new vec4(a.x, a.y, a.z, a.w);
};

/**
 * Copy content of a vector into another.
 *
 * @param {vec4} out - The vector to modified.
 * @param {vec4} a - The specified vector.
 * @returns {vec4} out.
 */
vec4.copy = function copy (out, a)
{
  out.x = a.x;
  out.y = a.y;
  out.z = a.z;
  out.w = a.w;
  return out;
};

/**
 * Sets the components of a vector to the given values.
 *
 * @param {vec4} out - The vector to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @param {Number} w - Value set to w component.
 * @returns {vec4} out.
 */
vec4.set = function set (out, x, y, z, w)
{
  out.x = x;
  out.y = y;
  out.z = z;
  out.w = w;
  return out;
};

/**
 * Add two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.add = function add (out, a, b)
{
  out.x = a.x + b.x;
  out.y = a.y + b.y;
  out.z = a.z + b.z;
  out.w = a.w + b.w;
  return out;
};

/**
 * Subtract two vectors.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.subtract = function subtract (out, a, b)
{
  out.x = a.x - b.x;
  out.y = a.y - b.y;
  out.z = a.z - b.z;
  out.w = a.w - b.w;
  return out;
};

/**
 * Alias of {@link vec4.subtract}.
 */
vec4.sub = function sub (out, a, b)
{
  return vec4.subtract(out, a, b);
};

/**
 * Performs multiply on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.multiply = function multiply (out, a, b)
{
  out.x = a.x * b.x;
  out.y = a.y * b.y;
  out.z = a.z * b.z;
  out.w = a.w * b.w;
  return out;
};

/**
 * Alias of {@link vec4.multiply}.
 */
vec4.mul = function mul (out, a, b)
{
  return vec4.multiply(out, a, b);
};

/**
 * Performs division on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.divide = function divide (out, a, b)
{
  out.x = a.x / b.x;
  out.y = a.y / b.y;
  out.z = a.z / b.z;
  out.w = a.w / b.w;
  return out;
};

/**
 * Alias of {@link vec4.divide}.
 */
vec4.div = function div (out, a, b)
{
  return vec4.divide(out, a, b);
};

/**
 * Performs Math.ceil on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.ceil = function ceil (out, a)
{
  out.x = Math.ceil(a.x);
  out.y = Math.ceil(a.y);
  out.z = Math.ceil(a.z);
  out.w = Math.ceil(a.w);
  return out;
};

/**
 * Performs Math.floor on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.floor = function floor (out, a)
{
  out.x = Math.floor(a.x);
  out.y = Math.floor(a.y);
  out.z = Math.floor(a.z);
  out.w = Math.floor(a.w);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.min = function min (out, a, b)
{
  out.x = Math.min(a.x, b.x);
  out.y = Math.min(a.y, b.y);
  out.z = Math.min(a.z, b.z);
  out.w = Math.min(a.w, b.w);
  return out;
};

/**
 * Performs Math.min on each component of two vectors respectively.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {vec4} out.
 */
vec4.max = function max (out, a, b)
{
  out.x = Math.max(a.x, b.x);
  out.y = Math.max(a.y, b.y);
  out.z = Math.max(a.z, b.z);
  out.w = Math.max(a.w, b.w);
  return out;
};

/**
 * Performs Math.round on each component of a vector.
 *
 * It doesn't matter that any amount of these parameters refer to same vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to perform operation.
 * @returns {vec4} out.
 */
vec4.round = function round (out, a)
{
  out.x = Math.round(a.x);
  out.y = Math.round(a.y);
  out.z = Math.round(a.z);
  out.w = Math.round(a.w);
  return out;
};

/**
 * Scales a vector with a number.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to scale.
 * @param {number} b - The scale number.
 * @returns {vec4} out.
 * */
vec4.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  out.w = a.w * b;
  return out;
};

/**
 * Add two vectors after scaling the second operand by a number.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @param {number} scale - The scale number before adding.
 * @returns {vec4} out.
 */
vec4.scaleAndAdd = function scaleAndAdd (out, a, b, scale)
{
  out.x = a.x + (b.x * scale);
  out.y = a.y + (b.y * scale);
  out.z = a.z + (b.z * scale);
  out.w = a.w + (b.w * scale);
  return out;
};

/**
 * Calculates the euclidian distance between two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {number} Distance between a and b.
 */
vec4.distance = function distance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z,
    w = b.w - a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 * Alias of {@link vec4.distance}.
 */
vec4.dist = function dist (a, b)
{
  return vec4.distance(a, b);
};

/**
 * Calculates the squared euclidian distance between two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {number} Squared distance between a and b.
 */
vec4.squaredDistance = function squaredDistance (a, b)
{
  var x = b.x - a.x,
    y = b.y - a.y,
    z = b.z - a.z,
    w = b.w - a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 * Alias of {@link vec4.squaredDistance}.
 */
vec4.sqrDist = function sqrDist (a, b)
{
  return vec4.squaredDistance(a, b);
};

/**
 * Calculates the length of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {Number} Length of the vector.
 */
vec4.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 *Alias of {@link vec4.magnitude}.
 */
vec4.mag = function mag (a)
{
  return vec4.magnitude(a);
};

/**
 * Calculates the squared length of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {Number} Squared length of the vector.
 */
vec4.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 *Alias of {@link vec4.squaredMagnitude}
 */
vec4.sqrMag = function sqrMag (a)
{
  return vec4.squaredMagnitude(a);
};

/**
 * Negates each component of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to negate.
 * @returns {vec4} out.
 */
vec4.negate = function negate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  out.w = -a.w;
  return out;
};

/**
 * Inverts the components of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to invert.
 * @returns {vec4} out.
 */
vec4.inverse = function inverse (out, a)
{
  out.x = 1.0 / a.x;
  out.y = 1.0 / a.y;
  out.z = 1.0 / a.z;
  out.w = 1.0 / a.w;
  return out;
};

/**
 * Safely inverts the components of a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to invert.
 * @returns {vec4} out.
 */
vec4.inverseSafe = function inverseSafe (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;

  if (Math.abs(x) < EPSILON)
  {
    out.x = 0;
  } else
  {
    out.x = 1.0 / x;
  }

  if (Math.abs(y) < EPSILON)
  {
    out.y = 0;
  } else
  {
    out.y = 1.0 / y;
  }

  if (Math.abs(z) < EPSILON)
  {
    out.z = 0;
  } else
  {
    out.z = 1.0 / z;
  }

  if (Math.abs(w) < EPSILON)
  {
    out.w = 0;
  } else
  {
    out.w = 1.0 / w;
  }

  return out;
};

/**
 * Normalizes a vector.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to normalize.
 * @returns {vec4} out.
 */
vec4.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  var len = x * x + y * y + z * z + w * w;
  if (len > 0)
  {
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
    out.w = w * len;
  }
  return out;
};

/**
 * Calculates the dot product of two vectors.
 *
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @returns {Number} Dot product of a and b.
 */
vec4.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
};

/**
 * Performs a linear interpolation between two vectors.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - The first operand.
 * @param {vec4} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {vec4} out.
 */
vec4.lerp = function lerp$$1 (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z,
    aw = a.w;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  out.w = aw + t * (b.w - aw);
  return out;
};

/**
 * Generates a random vector with the given scale.
 *
 * @param {vec4} out - Vector to store result.
 * @param {Number} [scale] Length of the resulting vector. If ommitted, a unit vector will be returned.
 * @returns {vec4} out.
 */
vec4.random = function random$1 (out, scale)
{
  scale = scale || 1.0;

  //TODO: This is a pretty awful way of doing this. Find something better.
  out.x = random();
  out.y = random();
  out.z = random();
  out.w = random();
  vec4.normalize(out, out);
  vec4.scale(out, out, scale);
  return out;
};

/**
 * Transforms a vector with a 4x4 matrix.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to transform.
 * @param {mat4} m - The matrix.
 * @returns {vec4} out.
 */
vec4.transformMat4 = function transformMat4 (out, a, m)
{
  var x = a.x, y = a.y, z = a.z, w = a.w;
  out.x = m.m00 * x + m.m04 * y + m.m08 * z + m.m12 * w;
  out.y = m.m01 * x + m.m05 * y + m.m09 * z + m.m13 * w;
  out.z = m.m02 * x + m.m06 * y + m.m10 * z + m.m14 * w;
  out.w = m.m03 * x + m.m07 * y + m.m11 * z + m.m15 * w;
  return out;
};

/**
 * Transforms a vector with a quaternion.
 *
 * @param {vec4} out - Vector to store result.
 * @param {vec4} a - Vector to transform.
 * @param {quat} q - The quaternion.
 * @returns {vec4} out.
 */
vec4.transformQuat = function transformQuat (out, a, q)
{
  var x = a.x, y = a.y, z = a.z;
  var qx = q.x, qy = q.y, qz = q.z, qw = q.w;

  // calculate quat * vec
  var ix = qw * x + qy * z - qz * y;
  var iy = qw * y + qz * x - qx * z;
  var iz = qw * z + qx * y - qy * x;
  var iw = -qx * x - qy * y - qz * z;

  // calculate result * inverse quat
  out.x = ix * qw + iw * -qx + iy * -qz - iz * -qy;
  out.y = iy * qw + iw * -qy + iz * -qx - ix * -qz;
  out.z = iz * qw + iw * -qz + ix * -qy - iy * -qx;
  out.w = a.w;
  return out;
};

/**
 * Returns string representation of a vector.
 *
 * @param {vec4} a - The vector.
 * @returns {String} - String representation of this vector.
 */
vec4.str = function str (a)
{
  return ("vec4(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ", " + (a.w) + ")");
};

/**
 * Store components of a vector into array.
 *
 * @param {Array} out - Array to store result.
 * @param {vec4} v - The vector.
 * @returns {Array} out.
 */
vec4.array = function array (out, v)
{
  out[0] = v.x;
  out[1] = v.y;
  out[2] = v.z;
  out[3] = v.w;

  return out;
};

/**
 * Returns whether the specified vectors are equal. (Compared using ===)
 *
 * @param {vec4} a - The first vector.
 * @param {vec4} b - The second vector.
 * @returns {Boolean} True if the vectors are equal, false otherwise.
 */
vec4.exactEquals = function exactEquals (a, b)
{
  return a.x === b.x && a.y === b.y && a.z === b.z && a.w === b.w;
};

/**
 * Returns whether the specified vectors are approximately equal.
 *
 * @param {vec4} a The first vector.
 * @param {vec4} b The second vector.
 * @returns {Boolean} True if the vectors are approximately equal, false otherwise.
 */
vec4.equals = function equals$$1 (a, b)
{
  var a0 = a.x, a1 = a.y, a2 = a.z, a3 = a.w;
  var b0 = b.x, b1 = b.y, b2 = b.z, b3 = b.w;
  return (Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
    Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
    Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
    Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)));
};

/**
 * Performs some operation over an array of vec4s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 */
vec4.forEach = function forEach (a, stride, offset, count, fn, arg)
{
  return vec4._forEach(a, stride, offset, count, fn, arg);
};

/**
 * Perform some operation over an array of vec4s.
 *
 * @param {Array} a the array of vectors to iterate over.
 * @param {Number} stride Number of elements between the start of each vec4. If 0 assumes tightly packed.
 * @param {Number} offset Number of elements to skip at the beginning of the array.
 * @param {Number} count Number of vec4s to iterate over. If 0 iterates over entire array.
 * @param {Function} fn Function to call for each vector in the array.
 * @param {Object} [arg] additional argument to pass to fn.
 * @returns {Array} a.
 * @ignore
 */
vec4._forEach = (function()
{
  var vec = vec4.zero();

  return function (a, stride, offset, count, fn, arg)
  {
    var i, l;
    if (!stride)
    {
      stride = 4;
    }

    if (!offset)
    {
      offset = 0;
    }

    if (count)
    {
      l = Math.min((count * stride) + offset, a.length);
    } else
    {
      l = a.length;
    }

    for (i = offset; i < l; i += stride)
    {
      vec.x = a[i];
      vec.y = a[i + 1];
      vec.z = a[i + 2];
      vec.w = a[i + 3];
      fn(vec, vec, arg);
      a[i] = vec.x;
      a[i + 1] = vec.y;
      a[i + 2] = vec.z;
      a[i + 3] = vec.w;
    }

    return a;
  };
})();

/**
 * Mathematical 3x3 matrix.
 *
 * NOTE: we use column-major matrix for all matrix calculation.
 *
 * This may lead to some confusion when referencing OpenGL documentation,
 * however, which represents out all matricies in column-major format.
 * This means that while in code a matrix may be typed out as:
 *
 * [1, 0, 0, 0,
 *  0, 1, 0, 0,
 *  0, 0, 1, 0,
 *  x, y, z, 0]
 *
 * The same matrix in the [OpenGL documentation](https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glTranslate.xml)
 * is written as:
 *
 *  1 0 0 x
 *  0 1 0 y
 *  0 0 1 z
 *  0 0 0 0
 *
 * Please rest assured, however, that they are the same thing!
 * This is not unique to glMatrix, either, as OpenGL developers have long been confused by the
 * apparent lack of consistency between the memory layout and the documentation.
 */
var mat3 = function mat3(
m00, m01, m02, m03,
m04, m05, m06, m07,
m08
)
{
/**
 * The element at column 0 row 0.
 * @type {number}
 * */
this.m00 = m00;
/**
 * The element at column 0 row 1.
 * @type {number}
 * */
this.m01 = m01;
/**
 * The element at column 0 row 2.
 * @type {number}
 * */
this.m02 = m02;

/**
 * The element at column 1 row 0.
 * @type {number}
 * */
this.m03 = m03;
/**
 * The element at column 1 row 1.
 * @type {number}
 * */
this.m04 = m04;
/**
 * The element at column 1 row 2.
 * @type {number}
 * */
this.m05 = m05;

/**
 * The element at column 2 row 0.
 * @type {number}
 * */
this.m06 = m06;
/**
 * The element at column 2 row 1.
 * @type {number}
 * */
this.m07 = m07;
/**
 * The element at column 2 row 2.
 * @type {number}
 * */
this.m08 = m08;
};

/**
 * Creates a new identity matrix.
 *
 * @returns {mat3} The newly created matrix.
 */
mat3.create = function create ()
{
return new mat3(
  1, 0, 0,
  0, 1, 0,
  0, 0, 1
);
};

/**
 * Creates a matrix, with elements specified separately.
 *
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m03 - Value assigned to element at column 1 row 0.
 * @param {Number} m04 - Value assigned to element at column 1 row 1.
 * @param {Number} m05 - Value assigned to element at column 1 row 2.
 * @param {Number} m06 - Value assigned to element at column 2 row 0.
 * @param {Number} m07 - Value assigned to element at column 2 row 1.
 * @param {Number} m08 - Value assigned to element at column 2 row 2.
 * @returns {mat3} The newly created matrix.
 */
mat3.new = function new$1 (m00, m01, m02, m03, m04, m05, m06, m07, m08)
{
return new mat3(m00, m01, m02, m03, m04, m05, m06, m07, m08);
};

/**
 * Clone a matrix.
 *
 * @param {mat3} a - Matrix to clone.
 * @returns {mat3} The newly created matrix.
 */
mat3.clone = function clone (a)
{
  return new mat3(
    a.m00, a.m01, a.m02,
    a.m03, a.m04, a.m05,
    a.m06, a.m07, a.m08
  );
};

/**
 * Copy content of a matrix into another.
 *
 * @param {mat3} out - Matrix to modified.
 * @param {mat3} a - The specified matrix.
 * @returns {mat3} out.
 */
mat3.copy = function copy (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = a.m02;
out.m03 = a.m03;
out.m04 = a.m04;
out.m05 = a.m05;
out.m06 = a.m06;
out.m07 = a.m07;
out.m08 = a.m08;
return out;
};

/**
 * Sets the elements of a matrix to the given values.
 *
 * @param {mat3} out - The matrix to modified.
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m10 - Value assigned to element at column 1 row 0.
 * @param {Number} m11 - Value assigned to element at column 1 row 1.
 * @param {Number} m12 - Value assigned to element at column 1 row 2.
 * @param {Number} m20 - Value assigned to element at column 2 row 0.
 * @param {Number} m21 - Value assigned to element at column 2 row 1.
 * @param {Number} m22 - Value assigned to element at column 2 row 2.
 * @returns {mat3} out.
 */
mat3.set = function set (out, m00, m01, m02, m10, m11, m12, m20, m21, m22)
{
  out.m00 = m00;
  out.m01 = m01;
  out.m02 = m02;
  out.m03 = m10;
  out.m04 = m11;
  out.m05 = m12;
  out.m06 = m20;
  out.m07 = m21;
  out.m08 = m22;
  return out;
};

/**
 * Sets a matrix as identity matrix.
 *
 * @param {mat3} out - Matrix to modified.
 * @returns {mat3} out.
 */
mat3.identity = function identity (out)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 1;
out.m05 = 0;
out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Transposes a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to transpose.
 * @returns {mat3} out.
 */
mat3.transpose = function transpose (out, a)
{
// If we are transposing ourselves we can skip a few steps but have to cache some values
if (out === a) {
  var a01 = a.m01, a02 = a.m02, a12 = a.m05;
  out.m01 = a.m03;
  out.m02 = a.m06;
  out.m03 = a01;
  out.m05 = a.m07;
  out.m06 = a02;
  out.m07 = a12;
} else {
  out.m00 = a.m00;
  out.m01 = a.m03;
  out.m02 = a.m06;
  out.m03 = a.m01;
  out.m04 = a.m04;
  out.m05 = a.m07;
  out.m06 = a.m02;
  out.m07 = a.m05;
  out.m08 = a.m08;
}

return out;
};

/**
 * Inverts a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to invert.
 * @returns {mat3} out.
 */
mat3.invert = function invert (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var b01 = a22 * a11 - a12 * a21;
var b11 = -a22 * a10 + a12 * a20;
var b21 = a21 * a10 - a11 * a20;

// Calculate the determinant
var det = a00 * b01 + a01 * b11 + a02 * b21;

if (!det) {
  return null;
}
det = 1.0 / det;

out.m00 = b01 * det;
out.m01 = (-a22 * a01 + a02 * a21) * det;
out.m02 = (a12 * a01 - a02 * a11) * det;
out.m03 = b11 * det;
out.m04 = (a22 * a00 - a02 * a20) * det;
out.m05 = (-a12 * a00 + a02 * a10) * det;
out.m06 = b21 * det;
out.m07 = (-a21 * a00 + a01 * a20) * det;
out.m08 = (a11 * a00 - a01 * a10) * det;
return out;
};

/**
 * Calculates the adjugate of a matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to calculate.
 * @returns {mat3} out.
 */
mat3.adjoint = function adjoint (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

out.m00 = (a11 * a22 - a12 * a21);
out.m01 = (a02 * a21 - a01 * a22);
out.m02 = (a01 * a12 - a02 * a11);
out.m03 = (a12 * a20 - a10 * a22);
out.m04 = (a00 * a22 - a02 * a20);
out.m05 = (a02 * a10 - a00 * a12);
out.m06 = (a10 * a21 - a11 * a20);
out.m07 = (a01 * a20 - a00 * a21);
out.m08 = (a00 * a11 - a01 * a10);
return out;
};

/**
 * Calculates the determinant of a matrix.
 *
 * @param {mat3} a - Matrix to calculate.
 * @returns {Number} Determinant of a.
 */
mat3.determinant = function determinant (a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

return a00 * (a22 * a11 - a12 * a21) + a01 * (-a22 * a10 + a12 * a20) + a02 * (a21 * a10 - a11 * a20);
};

/**
 * Multiply two matrices explicitly.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.multiply = function multiply (out, a, b)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var b00 = b.m00, b01 = b.m01, b02 = b.m02;
var b10 = b.m03, b11 = b.m04, b12 = b.m05;
var b20 = b.m06, b21 = b.m07, b22 = b.m08;

out.m00 = b00 * a00 + b01 * a10 + b02 * a20;
out.m01 = b00 * a01 + b01 * a11 + b02 * a21;
out.m02 = b00 * a02 + b01 * a12 + b02 * a22;

out.m03 = b10 * a00 + b11 * a10 + b12 * a20;
out.m04 = b10 * a01 + b11 * a11 + b12 * a21;
out.m05 = b10 * a02 + b11 * a12 + b12 * a22;

out.m06 = b20 * a00 + b21 * a10 + b22 * a20;
out.m07 = b20 * a01 + b21 * a11 + b22 * a21;
out.m08 = b20 * a02 + b21 * a12 + b22 * a22;
return out;
};

/**
 * Alias of {@link mat3.multiply}.
 */
mat3.mul = function mul (out, a, b)
{
return mat3.multiply(out, a, b);
};

/**
 * Multiply a matrix with a translation matrix given by a translation offset.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to multiply.
 * @param {vec2} v - The translation offset.
 * @returns {mat3} out.
 */
mat3.translate = function translate (out, a, v)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;
var x = v.x, y = v.y;

out.m00 = a00;
out.m01 = a01;
out.m02 = a02;

out.m03 = a10;
out.m04 = a11;
out.m05 = a12;

out.m06 = x * a00 + y * a10 + a20;
out.m07 = x * a01 + y * a11 + a21;
out.m08 = x * a02 + y * a12 + a22;
return out;
};

/**
 * Rotates a matrix by the given angle.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to rotate.
 * @param {Number} rad - The rotation angle.
 * @returns {mat3} out
 */
mat3.rotate = function rotate (out, a, rad)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02,
    a10 = a.m03, a11 = a.m04, a12 = a.m05,
    a20 = a.m06, a21 = a.m07, a22 = a.m08;

var s = Math.sin(rad);
var c = Math.cos(rad);

out.m00 = c * a00 + s * a10;
out.m01 = c * a01 + s * a11;
out.m02 = c * a02 + s * a12;

out.m03 = c * a10 - s * a00;
out.m04 = c * a11 - s * a01;
out.m05 = c * a12 - s * a02;

out.m06 = a20;
out.m07 = a21;
out.m08 = a22;
return out;
};

/**
 * Multiply a matrix with a scale matrix given by a scale vector.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to multiply.
 * @param {vec2} v - The scale vector.
 * @returns {mat3} out
 **/
mat3.scale = function scale (out, a, v)
{
var x = v.x, y = v.y;

out.m00 = x * a.m00;
out.m01 = x * a.m01;
out.m02 = x * a.m02;

out.m03 = y * a.m03;
out.m04 = y * a.m04;
out.m05 = y * a.m05;

out.m06 = a.m06;
out.m07 = a.m07;
out.m08 = a.m08;
return out;
};

/**
 * Copies the upper-left 3x3 values of a 4x4 matrix into a 3x3 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat4} a - The 4x4 matrix.
 * @returns {mat3} out.
 */
mat3.fromMat4 = function fromMat4 (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = a.m02;
out.m03 = a.m04;
out.m04 = a.m05;
out.m05 = a.m06;
out.m06 = a.m08;
out.m07 = a.m09;
out.m08 = a.m10;
return out;
};

/**
 * Creates a matrix from a translation offset.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.translate(dest, dest, vec);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec2} v - The translation offset.
 * @returns {mat3} out.
 */
mat3.fromTranslation = function fromTranslation (out, v)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 1;
out.m05 = 0;
out.m06 = v.x;
out.m07 = v.y;
out.m08 = 1;
return out;
};

/**
 * Creates a matrix from a given angle.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.rotate(dest, dest, rad);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @returns {mat3} out.
 */
mat3.fromRotation = function fromRotation (out, rad)
{
var s = Math.sin(rad), c = Math.cos(rad);

out.m00 = c;
out.m01 = s;
out.m02 = 0;

out.m03 = -s;
out.m04 = c;
out.m05 = 0;

out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Creates a matrix from a scale vector.
 * This is equivalent to (but much faster than):
 *
 *   mat3.identity(dest);
 *   mat3.scale(dest, dest, vec);
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec2} v - Scale vector.
 * @returns {mat3} out.
 */
mat3.fromScaling = function fromScaling (out, v)
{
out.m00 = v.x;
out.m01 = 0;
out.m02 = 0;

out.m03 = 0;
out.m04 = v.y;
out.m05 = 0;

out.m06 = 0;
out.m07 = 0;
out.m08 = 1;
return out;
};

/**
 * Copies the values from a 2x3 matrix into a 3x3 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat23} a - The 2x3 matrix.
 * @returns {mat3} out.
 **/
mat3.fromMat2d = function fromMat2d (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = 0;

out.m03 = a.m02;
out.m04 = a.m03;
out.m05 = 0;

out.m06 = a.m04;
out.m07 = a.m05;
out.m08 = 1;
return out;
};

/**
 * Calculates a 3x3 matrix from the given quaternion.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {quat} q - The quaternion.
 *
 * @returns {mat3} out.
 */
mat3.fromQuat = function fromQuat (out, q)
{
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var yx = y * x2;
var yy = y * y2;
var zx = z * x2;
var zy = z * y2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;

out.m00 = 1 - yy - zz;
out.m03 = yx - wz;
out.m06 = zx + wy;

out.m01 = yx + wz;
out.m04 = 1 - xx - zz;
out.m07 = zy - wx;

out.m02 = zx - wy;
out.m05 = zy + wx;
out.m08 = 1 - xx - yy;

return out;
};

/**
 * Calculates a 3x3 matrix from view direction and up direction.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {vec3} view - View direction (must be normalized).
 * @param {vec3} [up] - Up direction, default is (0,1,0) (must be normalized).
 *
 * @returns {mat3} out
 */
mat3.fromViewUp = function fromViewUp (out, view, up)
{
return mat3._fromViewUp(out, view, up);
};

/**
 * Calculates a 3x3 normal matrix (transpose inverse) from the 4x4 matrix.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat4} a - A 4x4 matrix to derive the normal matrix from.
 *
 * @returns {mat3} out.
 */
mat3.normalFromMat4 = function normalFromMat4 (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

var b00 = a00 * a11 - a01 * a10;
var b01 = a00 * a12 - a02 * a10;
var b02 = a00 * a13 - a03 * a10;
var b03 = a01 * a12 - a02 * a11;
var b04 = a01 * a13 - a03 * a11;
var b05 = a02 * a13 - a03 * a12;
var b06 = a20 * a31 - a21 * a30;
var b07 = a20 * a32 - a22 * a30;
var b08 = a20 * a33 - a23 * a30;
var b09 = a21 * a32 - a22 * a31;
var b10 = a21 * a33 - a23 * a31;
var b11 = a22 * a33 - a23 * a32;

// Calculate the determinant
var det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

if (!det) {
  return null;
}
det = 1.0 / det;

out.m00 = (a11 * b11 - a12 * b10 + a13 * b09) * det;
out.m01 = (a12 * b08 - a10 * b11 - a13 * b07) * det;
out.m02 = (a10 * b10 - a11 * b08 + a13 * b06) * det;

out.m03 = (a02 * b10 - a01 * b11 - a03 * b09) * det;
out.m04 = (a00 * b11 - a02 * b08 + a03 * b07) * det;
out.m05 = (a01 * b08 - a00 * b10 - a03 * b06) * det;

out.m06 = (a31 * b05 - a32 * b04 + a33 * b03) * det;
out.m07 = (a32 * b02 - a30 * b05 - a33 * b01) * det;
out.m08 = (a30 * b04 - a31 * b02 + a33 * b00) * det;

return out;
};

/**
 * Returns a string representation of a matrix.
 *
 * @param {mat3} a - The matrix.
 * @returns {String} String representation of this matrix.
 */
mat3.str = function str (a)
{
  ("mat3(" + (a.m00) + ", " + (a.m01) + ", " + (a.m02) + ", " + (a.m03) + ", " + (a.m04) + ", " + (a.m05) + ", " + (a.m06) + ", " + (a.m07) + ", " + (a.m08) + ")");
};

/**
 * Store elements of a matrix into array.
 *
 * @param {array} out - Array to store result.
 * @param {mat3} m - The matrix.
 * @returns {Array} out.
 */
mat3.array = function array (out, m)
{
out[0] = m.m00;
out[1] = m.m01;
out[2] = m.m02;
out[3] = m.m03;
out[4] = m.m04;
out[5] = m.m05;
out[6] = m.m06;
out[7] = m.m07;
out[8] = m.m08;

return out;
};

/**
 * Returns Frobenius norm of a matrix.
 *
 * @param {mat3} a - Matrix to calculate Frobenius norm of.
 * @returns {Number} - The frobenius norm.
 */
mat3.frob = function frob (a)
{
return (Math.sqrt(Math.pow(a.m00, 2) + Math.pow(a.m01, 2) + Math.pow(a.m02, 2) + Math.pow(a.m03, 2) + Math.pow(a.m04, 2) + Math.pow(a.m05, 2) + Math.pow(a.m06, 2) + Math.pow(a.m07, 2) + Math.pow(a.m08, 2)));
};

/**
 * Adds two matrices.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.add = function add (out, a, b)
{
out.m00 = a.m00 + b.m00;
out.m01 = a.m01 + b.m01;
out.m02 = a.m02 + b.m02;
out.m03 = a.m03 + b.m03;
out.m04 = a.m04 + b.m04;
out.m05 = a.m05 + b.m05;
out.m06 = a.m06 + b.m06;
out.m07 = a.m07 + b.m07;
out.m08 = a.m08 + b.m08;
return out;
};

/**
 * Subtracts matrix b from matrix a.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @returns {mat3} out.
 */
mat3.subtract = function subtract (out, a, b)
{
out.m00 = a.m00 - b.m00;
out.m01 = a.m01 - b.m01;
out.m02 = a.m02 - b.m02;
out.m03 = a.m03 - b.m03;
out.m04 = a.m04 - b.m04;
out.m05 = a.m05 - b.m05;
out.m06 = a.m06 - b.m06;
out.m07 = a.m07 - b.m07;
out.m08 = a.m08 - b.m08;
return out;
};

/**
 * Alias of {@link mat3.subtract}.
 */
mat3.sub = function sub (out, a, b)
{
return mat3.subtract(out, a, b);
};

/**
 * Multiply each element of a matrix by a scalar number.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - Matrix to scale
 * @param {Number} b - The scale number.
 * @returns {mat3} out.
 */
mat3.multiplyScalar = function multiplyScalar (out, a, b)
{
out.m00 = a.m00 * b;
out.m01 = a.m01 * b;
out.m02 = a.m02 * b;
out.m03 = a.m03 * b;
out.m04 = a.m04 * b;
out.m05 = a.m05 * b;
out.m06 = a.m06 * b;
out.m07 = a.m07 * b;
out.m08 = a.m08 * b;
return out;
};

/**
 * Adds two matrices after multiplying each element of the second operand by a scalar number.
 *
 * @param {mat3} out - Matrix to store result.
 * @param {mat3} a - The first operand.
 * @param {mat3} b - The second operand.
 * @param {Number} scale - The scale number.
 * @returns {mat3} out.
 */
mat3.multiplyScalarAndAdd = function multiplyScalarAndAdd (out, a, b, scale)
{
out.m00 = a.m00 + (b.m00 * scale);
out.m01 = a.m01 + (b.m01 * scale);
out.m02 = a.m02 + (b.m02 * scale);
out.m03 = a.m03 + (b.m03 * scale);
out.m04 = a.m04 + (b.m04 * scale);
out.m05 = a.m05 + (b.m05 * scale);
out.m06 = a.m06 + (b.m06 * scale);
out.m07 = a.m07 + (b.m07 * scale);
out.m08 = a.m08 + (b.m08 * scale);
return out;
};

/**
 * Returns whether the specified matrices are equal. (Compared using ===)
 *
 * @param {mat3} a - The first matrix.
 * @param {mat3} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat3.exactEquals = function exactEquals (a, b)
{
return a.m00 === b.m00 && a.m01 === b.m01 && a.m02 === b.m02 &&
  a.m03 === b.m03 && a.m04 === b.m04 && a.m05 === b.m05 &&
  a.m06 === b.m06 && a.m07 === b.m07 && a.m08 === b.m08;
};

/**
 * Returns whether the specified matrices are approximately equal.
 *
 * @param {mat3} a - The first matrix.
 * @param {mat3} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat3.equals = function equals$$1 (a, b)
{
var a0 = a.m00, a1 = a.m01, a2 = a.m02, a3 = a.m03, a4 = a.m04, a5 = a.m05, a6 = a.m06, a7 = a.m07, a8 = a.m08;
var b0 = b.m00, b1 = b.m01, b2 = b.m02, b3 = b.m03, b4 = b.m04, b5 = b.m05, b6 = b.m06, b7 = b.m07, b8 = b.m08;
return (
  Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
  Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
  Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
  Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)) &&
  Math.abs(a4 - b4) <= EPSILON * Math.max(1.0, Math.abs(a4), Math.abs(b4)) &&
  Math.abs(a5 - b5) <= EPSILON * Math.max(1.0, Math.abs(a5), Math.abs(b5)) &&
  Math.abs(a6 - b6) <= EPSILON * Math.max(1.0, Math.abs(a6), Math.abs(b6)) &&
  Math.abs(a7 - b7) <= EPSILON * Math.max(1.0, Math.abs(a7), Math.abs(b7)) &&
  Math.abs(a8 - b8) <= EPSILON * Math.max(1.0, Math.abs(a8), Math.abs(b8))
);
};

mat3._fromViewUp = (function ()
{
  var default_up = vec3.new(0, 1, 0);
  var x = vec3.zero();
  var y = vec3.zero();

  return function (out, view, up)
  {
    if (vec3.sqrMag(view) < EPSILON * EPSILON)
    {
      mat3.identity(out);
      return out;
    }

    up = up || default_up;
    vec3.cross(x, up, view);

    if (vec3.sqrMag(x) < EPSILON * EPSILON)
    {
      mat3.identity(out);
      return out;
    }

    vec3.cross(y, view, x);
    mat3.set(
	  out,
      x.x, x.y, x.z,
      y.x, y.y, y.z,
      view.x, view.y, view.z
    );

    return out;
  };
})();

/**
 * Mathematical quaternion.
 *
 * A quaternion is a hypercomplex number represented by w + xi + yj + zk, where
 * x, y, z and w are real numbers(called here its components), and i, j, and k are the fundamental quaternion units.
 */
var quat = function quat(x, y, z, w)
{
  /**
   * The x component.
   * @type {number}
   * */
  this.x = x;
  /**
   * The y component.
   * @type {number}
   * */
  this.y = y;
  /**
   * The z component.
   * @type {number}
   * */
  this.z = z;
  /**
   * The w component.
   * @type {number}
   * */
  this.w = w;
};

/**
 * Creates a quaternion, with components specified separately.
 *
 * @param {number} x - Value assigned to x component.
 * @param {number} y - Value assigned to y component.
 * @param {number} z - Value assigned to z component.
 * @param {number} w - Value assigned to w component.
 * @return {quat} The newly created quaternion.
 */
quat.new = function new$1 (x, y, z, w)
{
  return new quat(x, y, z, w);
};

/**
 * Creates an identity quaternion.
 *
 * @returns {quat} The newly created quaternion.
 */
quat.create = function create ()
{
  return new quat(0, 0, 0, 1);
};

/**
 * Clone a quaternion.
 *
 * @param {quat} a - Quaternion to clone.
 * @returns {quat} The newly created quaternion.
 */
quat.clone = function clone (a)
{
  return new quat(a.x, a.y, a.z, a.w);
};

/**
 * Copy content of a quaternion into another.
 *
 * @param {quat} out - Quaternion to modified.
 * @param {quat} a - The specified quaternion.
 * @returns {quat} out.
 */
quat.copy = function copy (out, a)
{
	return vec4.copy(out, a);
};

/**
 * Sets the components of a quaternion to the given values.
 *
 * @param {quat} out - The quaternion to modified.
 * @param {Number} x - Value set to x component.
 * @param {Number} y - Value set to y component.
 * @param {Number} z - Value set to z component.
 * @param {Number} w - Value set to w component.
 * @returns {quat} out.
 */
quat.set = function set (out, x, y, z, w)
{
  out.x = x;
  out.y = y;
  out.z = z;
  out.w = w;
  return out;
};
  
/**
 * Sets a quaternion as identity quaternion.
 *
 * @param {quat} out - Quaternion to set.
 * @returns {quat} out.
 */
quat.identity = function identity (out)
{
  out.x = 0;
  out.y = 0;
  out.z = 0;
  out.w = 1;
  return out;
};

/**
 * Sets a quaternion to represent the shortest rotation from one
 * vector to another.
 *
 * Both vectors are assumed to be unit length.
 *
 * @param {quat} out - Quaternion to set.
 * @param {vec3} a - The initial vector.
 * @param {vec3} b - The destination vector.
 * @returns {quat} out.
 */
quat.rotationTo = function rotationTo (out, a, b)
{
  return quat._rotationTo(out, a, b);
};

/**
 * Gets the rotation axis and angle for a given
 *quaternion. If a quaternion is created with
 *fromAxisAngle, this method will return the same
 *values as provided in the original parameter list
 *OR functionally equivalent values.
 * Example: The quaternion formed by axis [0, 0, 1] and
 *angle -90 is the same as the quaternion formed by
 *[0, 0, 1] and 270. This method favors the latter.
 * @param{vec3} out_axis - Vector to store the rotation axis.
 * @param{quat} q - Quaternion to be decomposed.
 * @return {Number} - Angle, in radians, of the rotation.
 */
quat.getAxisAngle = function getAxisAngle (out_axis, q)
{
  var rad = Math.acos(q.w) * 2.0;
  var s = Math.sin(rad / 2.0);
  if (s != 0.0)
  {
    out_axis.x = q.x / s;
    out_axis.y = q.y / s;
    out_axis.z = q.z / s;
  } else
  {
    // If s is zero, return any axis (no rotation - axis does not matter)
    out_axis.x = 1;
    out_axis.y = 0;
    out_axis.z = 0;
  }
  return rad;
};

/**
 * Multiply two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @returns {quat} out.
 */
quat.multiply = function multiply (out, a, b)
{
  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = b.x, by = b.y, bz = b.z, bw = b.w;

  out.x = ax * bw + aw * bx + ay * bz - az * by;
  out.y = ay * bw + aw * by + az * bx - ax * bz;
  out.z = az * bw + aw * bz + ax * by - ay * bx;
  out.w = aw * bw - ax * bx - ay * by - az * bz;
  return out;
};

/**
 * Alias of {@link quat.multiply}.
 */
quat.mul = function mul (out, a, b)
{
	return quat.multiply(out, a, b);
};

/**
 * Scales a quaternion with a number.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to scale.
 * @param {quat} b - The scale number.
 * @returns {quat} out.
 * */
quat.scale = function scale (out, a, b)
{
  out.x = a.x * b;
  out.y = a.y * b;
  out.z = a.z * b;
  out.w = a.w * b;
  return out;
};
  
/**
 * Rotates a quaternion by the given angle about the X axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateX = function rotateX (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw + aw * bx;
  out.y = ay * bw + az * bx;
  out.z = az * bw - ay * bx;
  out.w = aw * bw - ax * bx;
  return out;
};

/**
 * Rotates a quaternion by the given angle about the Y axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateY = function rotateY (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    by = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw - az * by;
  out.y = ay * bw + aw * by;
  out.z = az * bw + ax * by;
  out.w = aw * bw - ay * by;
  return out;
};

/**
 * Rotates a quaternion by the given angle about the Z axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to rotate.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateZ = function rotateZ (out, a, rad)
{
  rad *= 0.5;

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bz = Math.sin(rad), bw = Math.cos(rad);

  out.x = ax * bw + ay * bz;
  out.y = ay * bw - ax * bz;
  out.z = az * bw + aw * bz;
  out.w = aw * bw - az * bz;
  return out;
};

/**
 * Rotates a quaternion by the given angle about a world space axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} rot - Quaternion to rotate.
 * @param {vec3} axis - The axis around which to rotate in world space.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateAround = function rotateAround (out, rot, axis, rad)
{
  return quat._rotateAround(out, rot, axis, rad);
};

/**
 * Rotates a quaternion by the given angle about a local space axis.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} rot - Quaternion to rotate.
 * @param {vec3} axis - The axis around which to rotate in local space.
 * @param {number} rad - Angle (in radians) to rotate.
 * @returns {quat} out.
 */
quat.rotateAroundLocal = function rotateAroundLocal (out, rot, axis, rad)
{
  return quat._rotateAroundLocal(out, rot, axis, rad);
};

/**
 * Calculates the W component of a quaternion from the X, Y, and Z components.
 * Assumes that quaternion is 1 unit in length.
 * Any existing W component will be ignored.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate W.
 * @returns {quat} out.
 */
quat.calculateW = function calculateW (out, a)
{
  var x = a.x, y = a.y, z = a.z;

  out.x = x;
  out.y = y;
  out.z = z;
  out.w = Math.sqrt(Math.abs(1.0 - x * x - y * y - z * z));
  return out;
};

/**
 * Calculates the dot product of two quaternions.
 *
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @returns {Number} - The dot product of a and b.
 */
quat.dot = function dot (a, b)
{
  return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
};
  
/**
 * Performs a linear interpolation between two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out.
 */
quat.lerp = function lerp (out, a, b, t)
{
  var ax = a.x,
    ay = a.y,
    az = a.z,
    aw = a.w;
  out.x = ax + t * (b.x - ax);
  out.y = ay + t * (b.y - ay);
  out.z = az + t * (b.z - az);
  out.w = aw + t * (b.w - aw);
  return out;
};
  
/**
 * Performs a spherical linear interpolation between two quaternions.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out.
 */
quat.slerp = function slerp (out, a, b, t)
{
  // benchmarks:
  //  http://jsperf.com/quaternion-slerp-implementations

  var ax = a.x, ay = a.y, az = a.z, aw = a.w,
    bx = b.x, by = b.y, bz = b.z, bw = b.w;

  var omega, cosom, sinom, scale0, scale1;

  // calc cosine
  cosom = ax * bx + ay * by + az * bz + aw * bw;
  // adjust signs (if necessary)
  if (cosom < 0.0)
  {
    cosom = -cosom;
    bx = -bx;
    by = -by;
    bz = -bz;
    bw = -bw;
  }
  // calculate coefficients
  if ((1.0 - cosom) > 0.000001)
  {
    // standard case (slerp)
    omega = Math.acos(cosom);
    sinom = Math.sin(omega);
    scale0 = Math.sin((1.0 - t) * omega) / sinom;
    scale1 = Math.sin(t * omega) / sinom;
  } else
  {
    // "from" and "to" quaternions are very close
    //... so we can do a linear interpolation
    scale0 = 1.0 - t;
    scale1 = t;
  }
  // calculate final values
  out.x = scale0 * ax + scale1 * bx;
  out.y = scale0 * ay + scale1 * by;
  out.z = scale0 * az + scale1 * bz;
  out.w = scale0 * aw + scale1 * bw;

  return out;
};

/**
 * Performs a spherical linear interpolation with two control points.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - The first operand.
 * @param {quat} b - The second operand.
 * @param {quat} c - The third operand.
 * @param {quat} d - The fourth operand.
 * @param {Number} t - The interpolation coefficient.
 * @returns {quat} out
 */
quat.sqlerp = function sqlerp (out, a, b, c, d, t)
{
  return quat._sqlerp(out, a, b, c, d, t);
};

/**
 * Calculates the inverse of a quaternion.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate inverse of.
 * @returns {quat} out.
 */
quat.invert = function invert (out, a)
{
  var a0 = a.x, a1 = a.y, a2 = a.z, a3 = a.w;
  var dot = a0 * a0 + a1 * a1 + a2 * a2 + a3 * a3;
  var invDot = dot ? 1.0 / dot : 0;

  // TODO: Would be faster to return [0,0,0,0] immediately if dot == 0

  out.x = -a0 * invDot;
  out.y = -a1 * invDot;
  out.z = -a2 * invDot;
  out.w = a3 * invDot;
  return out;
};

/**
 * Calculates the conjugate of a quaternion.
 * If the quaternion is normalized, this function is faster than quat.inverse and produces the same result.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to calculate conjugate of.
 * @returns {quat} out.
 */
quat.conjugate = function conjugate (out, a)
{
  out.x = -a.x;
  out.y = -a.y;
  out.z = -a.z;
  out.w = a.w;
  return out;
};

/**
 * Calculates the length of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {Number} Length of the quaternion.
 */
quat.magnitude = function magnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return Math.sqrt(x * x + y * y + z * z + w * w);
};

/**
 *Alias of {@link quat.magnitude}.
 */
quat.mag = function mag (a)
{
  return quat.magnitude(a);
};

/**
 * Calculates the squared length of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {Number} Squared length of the quaternion.
 */
quat.squaredMagnitude = function squaredMagnitude (a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  return x * x + y * y + z * z + w * w;
};

/**
 *Alias of {@link quat.squaredMagnitude}
 */
quat.sqrMag = function sqrMag (a)
{
  return quat.squaredMagnitude(a);
};
  
/**
 * Normalizes a quaternion.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {quat} a - Quaternion to normalize.
 * @returns {quat} out.
 * @function
 */
quat.normalize = function normalize (out, a)
{
  var x = a.x,
    y = a.y,
    z = a.z,
    w = a.w;
  var len = x * x + y * y + z * z + w * w;
  if (len > 0)
  {
    len = 1 / Math.sqrt(len);
    out.x = x * len;
    out.y = y * len;
    out.z = z * len;
    out.w = w * len;
  }
  return out;
};
  
/**
 * Sets the specified quaternion with values corresponding to the given
 * axes. Each axis is a vec3 and is expected to be unit length and
 * perpendicular to all other specified axes.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} xAxis - Vector representing the local "right" direction.
 * @param {vec3} yAxis - Vector representing the local "up" direction.
 * @param {vec3} zAxis - Vector representing the viewing direction.
 * @returns {quat} out.
 */
quat.fromAxes = function fromAxes (out, xAxis, yAxis, zAxis)
{
  return quat._fromAxes(out, xAxis, yAxis, zAxis);
};

/**
 * Calculates a quaternion from view direction and up direction
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} view - View direction (must be normalized).
 * @param {vec3} [up] - Up direction, default is (0,1,0) (must be normalized).
 *
 * @returns {quat} out.
 */
quat.fromViewUp = function fromViewUp (out, view, up)
{
  return quat._fromViewUp(out, view, up);
};

/**
 * Sets a quaternion from the given angle and rotation axis,
 * then returns it.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {vec3} axis - The axis around which to rotate.
 * @param {Number} rad - The angle in radians.
 * @returns {quat} out.
 **/
quat.fromAxisAngle = function fromAxisAngle (out, axis, rad)
{
  rad = rad * 0.5;
  var s = Math.sin(rad);
  out.x = s * axis.x;
  out.y = s * axis.y;
  out.z = s * axis.z;
  out.w = Math.cos(rad);
  return out;
};

/**
 * Creates a quaternion from the given 3x3 rotation matrix.
 *
 * NOTE: The resultant quaternion is not normalized, so you should be sure
 * to re-normalize the quaternion yourself where necessary.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {mat3} m - The rotation matrix.
 * @returns {quat} out.
 * @function
 */
quat.fromMat3 = function fromMat3 (out, m)
{
  // http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm

  var m00 = m.m00, m01 = m.m03, m02 = m.m06,
    m10 = m.m01, m11 = m.m04, m12 = m.m07,
    m20 = m.m02, m21 = m.m05, m22 = m.m08;

  var trace = m00 + m11 + m22;

  if (trace > 0)
  {
    var s = 0.5 / Math.sqrt(trace + 1.0);

    out.w = 0.25 / s;
    out.x = (m21 - m12) * s;
    out.y = (m02 - m20) * s;
    out.z = (m10 - m01) * s;

  } else if ((m00 > m11) && (m00 > m22))
  {
    var s$1 = 2.0 * Math.sqrt(1.0 + m00 - m11 - m22);

    out.w = (m21 - m12) / s$1;
    out.x = 0.25 * s$1;
    out.y = (m01 + m10) / s$1;
    out.z = (m02 + m20) / s$1;

  } else if (m11 > m22)
  {
    var s$2 = 2.0 * Math.sqrt(1.0 + m11 - m00 - m22);

    out.w = (m02 - m20) / s$2;
    out.x = (m01 + m10) / s$2;
    out.y = 0.25 * s$2;
    out.z = (m12 + m21) / s$2;

  } else
  {
    var s$3 = 2.0 * Math.sqrt(1.0 + m22 - m00 - m11);

    out.w = (m10 - m01) / s$3;
    out.x = (m02 + m20) / s$3;
    out.y = (m12 + m21) / s$3;
    out.z = 0.25 * s$3;
  }

  return out;
};

/**
 * Creates a quaternion from the given euler angle x, y, z.
 *
 * @param {quat} out - Quaternion to store result.
 * @param {number} x - Angle to rotate around X axis in degrees.
 * @param {number} y - Angle to rotate around Y axis in degrees.
 * @param {number} z - Angle to rotate around Z axis in degrees.
 * @returns {quat} out.
 * @function
 */
quat.fromEuler = function fromEuler (out, x, y, z)
{
  var halfToRad = 0.5 * Math.PI / 180.0;
  x *= halfToRad;
  y *= halfToRad;
  z *= halfToRad;

  var sx = Math.sin(x);
  var cx = Math.cos(x);
  var sy = Math.sin(y);
  var cy = Math.cos(y);
  var sz = Math.sin(z);
  var cz = Math.cos(z);

  out.x = sx * cy * cz - cx * sy * sz;
  out.y = cx * sy * cz + sx * cy * sz;
  out.z = cx * cy * sz - sx * sy * cz;
  out.w = cx * cy * cz + sx * sy * sz;

  return out;
};

/**
 * Returns string representation of a quaternion.
 *
 * @param {quat} a - The quaternion.
 * @returns {String} - String representation of this quaternion.
 */
quat.str = function str (a)
{
  return ("quat(" + (a.x) + ", " + (a.y) + ", " + (a.z) + ", " + (a.w) + ")");
};

/**
 * Store components of a quaternion into array.
 *
 * @param {Array} out - Array to store result.
 * @param {quat} q - The quaternion.
 * @returns {Array} out.
 */
quat.array = function array (out, q)
{
  out[0] = q.x;
  out[1] = q.y;
  out[2] = q.z;
  out[3] = q.w;

  return out;
};

/**
 * Returns whether the specified quaternions are equal. (Compared using ===)
 *
 * @param {quat} a - The first quaternion.
 * @param {quat} b - The second quaternion.
 * @returns {Boolean} True if the quaternions are equal, false otherwise.
 */
quat.exactEquals = function exactEquals (a, b)
{
	return vec4.exactEquals(a, b);
};

/**
 * Returns whether the specified quaternions are approximately equal.
 *
 * @param {quat} a The first quaternion.
 * @param {quat} b The second quaternion.
 * @returns {Boolean} True if the quaternions are approximately equal, false otherwise.
 */
quat.equals = function equals (a, b)
{
	return vec4.equals(a, b);
};

/**
 * Sets a quaternion to represent the shortest rotation from one
 * vector to another.
 *
 * Both vectors are assumed to be unit length.
 *
 * @param {quat} out the receiving quaternion.
 * @param {vec3} a the initial vector
 * @param {vec3} b the destination vector
 * @returns {quat} out
 * @ignore
 */
quat._rotationTo = (function ()
{
  var tmpvec3 = vec3.zero();
  var xUnitVec3 = vec3.new(1, 0, 0);
  var yUnitVec3 = vec3.new(0, 1, 0);

  return function (out, a, b)
  {
    var dot = vec3.dot(a, b);
    if (dot < -0.999999)
    {
      vec3.cross(tmpvec3, xUnitVec3, a);
      if (vec3.magnitude(tmpvec3) < 0.000001)
      {
        vec3.cross(tmpvec3, yUnitVec3, a);
      }
      vec3.normalize(tmpvec3, tmpvec3);
      quat.fromAxisAngle(out, tmpvec3, Math.PI);
      return out;
    } else if (dot > 0.999999)
    {
      out.x = 0;
      out.y = 0;
      out.z = 0;
      out.w = 1;
      return out;
    } else
    {
      vec3.cross(tmpvec3, a, b);
      out.x = tmpvec3.x;
      out.y = tmpvec3.y;
      out.z = tmpvec3.z;
      out.w = 1 + dot;
      return quat.normalize(out, out);
    }
  };
})();

/**
 * Rotates a quaternion by the given angle about the axis in world space
 *
 * @param {quat} out quat receiving operation result
 * @param {quat} rot quat to rotate
 * @param {vec3} axis the axis around which to rotate in world space
 * @param {number} rad angle (in radians) to rotate
 * @returns {quat} out
 * @ignore
 */
quat._rotateAround = (function ()
{
  var v3_tmp = vec3.zero();
  var q_tmp = quat.create();

  return function (out, rot, axis, rad)
  {
    // get inv-axis (local to rot)
    quat.invert(q_tmp, rot);
    vec3.transformQuat(v3_tmp, axis, q_tmp);
    // rotate by inv-axis
    quat.fromAxisAngle(q_tmp, v3_tmp, rad);
    quat.mul(out, rot, q_tmp);

    return out;
  };
})();

/**
 * Rotates a quaternion by the given angle about the axis in local space
 *
 * @param {quat} out quat receiving operation result
 * @param {quat} rot quat to rotate
 * @param {vec3} axis the axis around which to rotate in local space
 * @param {number} rad angle (in radians) to rotate
 * @returns {quat} out
 * @ignore
 */
quat._rotateAroundLocal = (function ()
{
  var q_tmp = quat.create();

  return function (out, rot, axis, rad)
  {
    quat.fromAxisAngle(q_tmp, axis, rad);
    quat.mul(out, rot, q_tmp);

    return out;
  };
})();

/**
 * Performs a spherical linear interpolation with two control points
 *
 * @param {quat} out the receiving quaternion
 * @param {quat} a the first operand
 * @param {quat} b the second operand
 * @param {quat} c the third operand
 * @param {quat} d the fourth operand
 * @param {Number} t interpolation amount
 * @returns {quat} out
 * @ignore
 */
quat._sqlerp = (function ()
{
  var temp1 = quat.create();
  var temp2 = quat.create();

  return function (out, a, b, c, d, t)
  {
    quat.slerp(temp1, a, d, t);
    quat.slerp(temp2, b, c, t);
    quat.slerp(out, temp1, temp2, 2 * t * (1 - t));

    return out;
  };
}());

/**
 * Sets the specified quaternion with values corresponding to the given
 * axes. Each axis is a vec3 and is expected to be unit length and
 * perpendicular to all other specified axes.
 *
 * @param {vec3} xAxis the vector representing the local "right" direction
 * @param {vec3} yAxis the vector representing the local "up" direction
 * @param {vec3} zAxis the vector representing the viewing direction
 * @returns {quat} out
 * @ignore
 */
quat._fromAxes = (function ()
{
  var matr = mat3.create();

  return function (out, xAxis, yAxis, zAxis)
  {
    mat3.set(matr,
      xAxis.x, xAxis.y, xAxis.z,
      yAxis.x, yAxis.y, yAxis.z,
      zAxis.x, zAxis.y, zAxis.z
    );
    return quat.normalize(out, quat.fromMat3(out, matr));
  };
})();

/**
 * Calculates a quaternion from view direction and up direction
 *
 * @param {quat} out mat3 receiving operation result
 * @param {vec3} view view direction (must be normalized)
 * @param {vec3} [up] up direction, default is (0,1,0) (must be normalized)
 *
 * @returns {quat} out
 * @ignore
 */
quat._fromViewUp = (function ()
{
  var matr = mat3.create();

  return function (out, view, up)
  {
    mat3.fromViewUp(matr, view, up);
    if (!matr)
    {
      return null;
    }

    return quat.normalize(out, quat.fromMat3(out, matr));
  };
})();

/**
 * Mathematical 4x4 matrix.
 *
 * NOTE: we use column-major matrix for all matrix calculation.
 *
 * This may lead to some confusion when referencing OpenGL documentation,
 * however, which represents out all matricies in column-major format.
 * This means that while in code a matrix may be typed out as:
 *
 * [1, 0, 0, 0,
 *  0, 1, 0, 0,
 *  0, 0, 1, 0,
 *  x, y, z, 0]
 *
 * The same matrix in the [OpenGL documentation](https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glTranslate.xml)
 * is written as:
 *
 *  1 0 0 x
 *  0 1 0 y
 *  0 0 1 z
 *  0 0 0 0
 *
 * Please rest assured, however, that they are the same thing!
 * This is not unique to glMatrix, either, as OpenGL developers have long been confused by the
 * apparent lack of consistency between the memory layout and the documentation.
 */
var mat4 = function mat4(
m00, m01, m02, m03,
m04, m05, m06, m07,
m08, m09, m10, m11,
m12, m13, m14, m15
)
{
/**
 * The element at column 0 row 0.
 * @type {number}
 * */
this.m00 = m00;
/**
 * The element at column 0 row 1.
 * @type {number}
 * */
this.m01 = m01;
/**
 * The element at column 0 row 2.
 * @type {number}
 * */
this.m02 = m02;
/**
 * The element at column 0 row 3.
 * @type {number}
 * */
this.m03 = m03;

/**
 * The element at column 1 row 0.
 * @type {number}
 * */
this.m04 = m04;
/**
 * The element at column 1 row 1.
 * @type {number}
 * */
this.m05 = m05;
/**
 * The element at column 1 row 2.
 * @type {number}
 * */
this.m06 = m06;
/**
 * The element at column 1 row 3.
 * @type {number}
 * */
this.m07 = m07;

/**
 * The element at column 2 row 0.
 * @type {number}
 * */
this.m08 = m08;
/**
 * The element at column 2 row 1.
 * @type {number}
 * */
this.m09 = m09;
/**
 * The element at column 2 row 2.
 * @type {number}
 * */
this.m10 = m10;
/**
 * The element at column 2 row 3.
 * @type {number}
 * */
this.m11 = m11;

/**
 * The element at column 3 row 0.
 * @type {number}
 * */
this.m12 = m12;
/**
 * The element at column 3 row 1.
 * @type {number}
 * */
this.m13 = m13;
/**
 * The element at column 3 row 2.
 * @type {number}
 * */
this.m14 = m14;
/**
 * The element at column 3 row 3.
 * @type {number}
 * */
this.m15 = m15;
};

/**
 * Creates a matrix, with elements specified separately.
 *
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m03 - Value assigned to element at column 0 row 3.
 * @param {Number} m04 - Value assigned to element at column 1 row 0.
 * @param {Number} m05 - Value assigned to element at column 1 row 1.
 * @param {Number} m06 - Value assigned to element at column 1 row 2.
 * @param {Number} m07 - Value assigned to element at column 1 row 3.
 * @param {Number} m08 - Value assigned to element at column 2 row 0.
 * @param {Number} m09 - Value assigned to element at column 2 row 1.
 * @param {Number} m10 - Value assigned to element at column 2 row 2.
 * @param {Number} m11 - Value assigned to element at column 2 row 3.
 * @param {Number} m12 - Value assigned to element at column 3 row 0.
 * @param {Number} m13 - Value assigned to element at column 3 row 1.
 * @param {Number} m14 - Value assigned to element at column 3 row 2.
 * @param {Number} m15 - Value assigned to element at column 3 row 3.
 * @returns {mat4} The newly created matrix.
 */
mat4.new = function new$1 (
m00, m01, m02, m03,
m04, m05, m06, m07,
m08, m09, m10, m11,
m12, m13, m14, m15
)
{
return new mat4(
  m00, m01, m02, m03,
  m04, m05, m06, m07,
  m08, m09, m10, m11,
  m12, m13, m14, m15);
};

/**
 * Creates a new identity matrix.
 *
 * @returns {mat4} The newly created matrix.
 */
mat4.create = function create ()
{
return new mat4(
  1, 0, 0, 0,
  0, 1, 0, 0,
  0, 0, 1, 0,
  0, 0, 0, 1
);
};

/**
 * Clone a matrix.
 *
 * @param {mat4} a - Matrix to clone.
 * @returns {mat4} The newly created matrix.
 */
mat4.clone = function clone (a)
{
  return new mat4(
    a.m00, a.m01, a.m02, a.m03,
    a.m04, a.m05, a.m06, a.m07,
    a.m08, a.m09, a.m10, a.m11,
    a.m12, a.m13, a.m14, a.m15
  );
};

/**
 * Copy content of a matrix into another.
 *
 * @param {mat4} out - Matrix to modified.
 * @param {mat4} a - The specified matrix.
 * @returns {mat4} out.
 */
mat4.copy = function copy (out, a)
{
out.m00 = a.m00;
out.m01 = a.m01;
out.m02 = a.m02;
out.m03 = a.m03;
out.m04 = a.m04;
out.m05 = a.m05;
out.m06 = a.m06;
out.m07 = a.m07;
out.m08 = a.m08;
out.m09 = a.m09;
out.m10 = a.m10;
out.m11 = a.m11;
out.m12 = a.m12;
out.m13 = a.m13;
out.m14 = a.m14;
out.m15 = a.m15;
return out;
};

/**
 * Sets the elements of a matrix to the given values.
 *
 * @param {mat4} out - The matrix to modified.
 * @param {Number} m00 - Value assigned to element at column 0 row 0.
 * @param {Number} m01 - Value assigned to element at column 0 row 1.
 * @param {Number} m02 - Value assigned to element at column 0 row 2.
 * @param {Number} m03 - Value assigned to element at column 0 row 3.
 * @param {Number} m10 - Value assigned to element at column 1 row 0.
 * @param {Number} m11 - Value assigned to element at column 1 row 1.
 * @param {Number} m12 - Value assigned to element at column 1 row 2.
 * @param {Number} m13 - Value assigned to element at column 1 row 3.
 * @param {Number} m20 - Value assigned to element at column 2 row 0.
 * @param {Number} m21 - Value assigned to element at column 2 row 1.
 * @param {Number} m22 - Value assigned to element at column 2 row 2.
 * @param {Number} m23 - Value assigned to element at column 2 row 3.
 * @param {Number} m30 - Value assigned to element at column 3 row 0.
 * @param {Number} m31 - Value assigned to element at column 3 row 1.
 * @param {Number} m32 - Value assigned to element at column 3 row 2.
 * @param {Number} m33 - Value assigned to element at column 3 row 3.
 * @returns {mat4} out.
 */
mat4.set = function set (out, m00, m01, m02, m03, m10, m11, m12, m13, m20, m21, m22, m23, m30, m31, m32, m33)
{
  out.m00 = m00;
  out.m01 = m01;
  out.m02 = m02;
  out.m03 = m03;
  out.m04 = m10;
  out.m05 = m11;
  out.m06 = m12;
  out.m07 = m13;
  out.m08 = m20;
  out.m09 = m21;
  out.m10 = m22;
  out.m11 = m23;
  out.m12 = m30;
  out.m13 = m31;
  out.m14 = m32;
  out.m15 = m33;
  return out;
};


/**
 * Sets a matrix as identity matrix.
 *
 * @param {mat4} out - Matrix to modified.
 * @returns {mat4} out.
 */
mat4.identity = function identity (out)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = 1;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = 1;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Transposes a matrix.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to transpose.
 * @returns {mat4} out.
 */
mat4.transpose = function transpose (out, a)
{
// If we are transposing ourselves we can skip a few steps but have to cache some values
if (out === a) {
  var a01 = a.m01, a02 = a.m02, a03 = a.m03,
      a12 = a.m06, a13 = a.m07,
      a23 = a.m11;

  out.m01 = a.m04;
  out.m02 = a.m08;
  out.m03 = a.m12;
  out.m04 = a01;
  out.m06 = a.m09;
  out.m07 = a.m13;
  out.m08 = a02;
  out.m09 = a12;
  out.m11 = a.m14;
  out.m12 = a03;
  out.m13 = a13;
  out.m14 = a23;
} else {
  out.m00 = a.m00;
  out.m01 = a.m04;
  out.m02 = a.m08;
  out.m03 = a.m12;
  out.m04 = a.m01;
  out.m05 = a.m05;
  out.m06 = a.m09;
  out.m07 = a.m13;
  out.m08 = a.m02;
  out.m09 = a.m06;
  out.m10 = a.m10;
  out.m11 = a.m14;
  out.m12 = a.m03;
  out.m13 = a.m07;
  out.m14 = a.m11;
  out.m15 = a.m15;
}

return out;
};

/**
 * Inverts a matrix.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to invert.
 * @returns {mat4} out.
 */
mat4.invert = function invert (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

var b00 = a00 * a11 - a01 * a10;
var b01 = a00 * a12 - a02 * a10;
var b02 = a00 * a13 - a03 * a10;
var b03 = a01 * a12 - a02 * a11;
var b04 = a01 * a13 - a03 * a11;
var b05 = a02 * a13 - a03 * a12;
var b06 = a20 * a31 - a21 * a30;
var b07 = a20 * a32 - a22 * a30;
var b08 = a20 * a33 - a23 * a30;
var b09 = a21 * a32 - a22 * a31;
var b10 = a21 * a33 - a23 * a31;
var b11 = a22 * a33 - a23 * a32;

// Calculate the determinant
var det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

if (!det) {
  return null;
}
det = 1.0 / det;

out.m00 = (a11 * b11 - a12 * b10 + a13 * b09) * det;
out.m01 = (a02 * b10 - a01 * b11 - a03 * b09) * det;
out.m02 = (a31 * b05 - a32 * b04 + a33 * b03) * det;
out.m03 = (a22 * b04 - a21 * b05 - a23 * b03) * det;
out.m04 = (a12 * b08 - a10 * b11 - a13 * b07) * det;
out.m05 = (a00 * b11 - a02 * b08 + a03 * b07) * det;
out.m06 = (a32 * b02 - a30 * b05 - a33 * b01) * det;
out.m07 = (a20 * b05 - a22 * b02 + a23 * b01) * det;
out.m08 = (a10 * b10 - a11 * b08 + a13 * b06) * det;
out.m09 = (a01 * b08 - a00 * b10 - a03 * b06) * det;
out.m10 = (a30 * b04 - a31 * b02 + a33 * b00) * det;
out.m11 = (a21 * b02 - a20 * b04 - a23 * b00) * det;
out.m12 = (a11 * b07 - a10 * b09 - a12 * b06) * det;
out.m13 = (a00 * b09 - a01 * b07 + a02 * b06) * det;
out.m14 = (a31 * b01 - a30 * b03 - a32 * b00) * det;
out.m15 = (a20 * b03 - a21 * b01 + a22 * b00) * det;

return out;
};

/**
 * Calculates the adjugate of a matrix.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to calculate.
 * @returns {mat4} out.
 */
mat4.adjoint = function adjoint (out, a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

out.m00 = (a11 * (a22 * a33 - a23 * a32) - a21 * (a12 * a33 - a13 * a32) + a31 * (a12 * a23 - a13 * a22));
out.m01 = -(a01 * (a22 * a33 - a23 * a32) - a21 * (a02 * a33 - a03 * a32) + a31 * (a02 * a23 - a03 * a22));
out.m02 = (a01 * (a12 * a33 - a13 * a32) - a11 * (a02 * a33 - a03 * a32) + a31 * (a02 * a13 - a03 * a12));
out.m03 = -(a01 * (a12 * a23 - a13 * a22) - a11 * (a02 * a23 - a03 * a22) + a21 * (a02 * a13 - a03 * a12));
out.m04 = -(a10 * (a22 * a33 - a23 * a32) - a20 * (a12 * a33 - a13 * a32) + a30 * (a12 * a23 - a13 * a22));
out.m05 = (a00 * (a22 * a33 - a23 * a32) - a20 * (a02 * a33 - a03 * a32) + a30 * (a02 * a23 - a03 * a22));
out.m06 = -(a00 * (a12 * a33 - a13 * a32) - a10 * (a02 * a33 - a03 * a32) + a30 * (a02 * a13 - a03 * a12));
out.m07 = (a00 * (a12 * a23 - a13 * a22) - a10 * (a02 * a23 - a03 * a22) + a20 * (a02 * a13 - a03 * a12));
out.m08 = (a10 * (a21 * a33 - a23 * a31) - a20 * (a11 * a33 - a13 * a31) + a30 * (a11 * a23 - a13 * a21));
out.m09 = -(a00 * (a21 * a33 - a23 * a31) - a20 * (a01 * a33 - a03 * a31) + a30 * (a01 * a23 - a03 * a21));
out.m10 = (a00 * (a11 * a33 - a13 * a31) - a10 * (a01 * a33 - a03 * a31) + a30 * (a01 * a13 - a03 * a11));
out.m11 = -(a00 * (a11 * a23 - a13 * a21) - a10 * (a01 * a23 - a03 * a21) + a20 * (a01 * a13 - a03 * a11));
out.m12 = -(a10 * (a21 * a32 - a22 * a31) - a20 * (a11 * a32 - a12 * a31) + a30 * (a11 * a22 - a12 * a21));
out.m13 = (a00 * (a21 * a32 - a22 * a31) - a20 * (a01 * a32 - a02 * a31) + a30 * (a01 * a22 - a02 * a21));
out.m14 = -(a00 * (a11 * a32 - a12 * a31) - a10 * (a01 * a32 - a02 * a31) + a30 * (a01 * a12 - a02 * a11));
out.m15 = (a00 * (a11 * a22 - a12 * a21) - a10 * (a01 * a22 - a02 * a21) + a20 * (a01 * a12 - a02 * a11));
return out;
};

/**
 * Calculates the determinant of a matrix.
 *
 * @param {mat4} a - Matrix to calculate.
 * @returns {Number} Determinant of a.
 */
mat4.determinant = function determinant (a)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

var b00 = a00 * a11 - a01 * a10;
var b01 = a00 * a12 - a02 * a10;
var b02 = a00 * a13 - a03 * a10;
var b03 = a01 * a12 - a02 * a11;
var b04 = a01 * a13 - a03 * a11;
var b05 = a02 * a13 - a03 * a12;
var b06 = a20 * a31 - a21 * a30;
var b07 = a20 * a32 - a22 * a30;
var b08 = a20 * a33 - a23 * a30;
var b09 = a21 * a32 - a22 * a31;
var b10 = a21 * a33 - a23 * a31;
var b11 = a22 * a33 - a23 * a32;

// Calculate the determinant
return b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;
};

/**
 * Multiply two matrices explicitly.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - The first operand.
 * @param {mat4} b - The second operand.
 * @returns {mat4} out.
 */
mat4.multiply = function multiply (out, a, b)
{
var a00 = a.m00, a01 = a.m01, a02 = a.m02, a03 = a.m03,
    a10 = a.m04, a11 = a.m05, a12 = a.m06, a13 = a.m07,
    a20 = a.m08, a21 = a.m09, a22 = a.m10, a23 = a.m11,
    a30 = a.m12, a31 = a.m13, a32 = a.m14, a33 = a.m15;

// Cache only the current line of the second matrix
var b0 = b.m00, b1 = b.m01, b2 = b.m02, b3 = b.m03;
out.m00 = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
out.m01 = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
out.m02 = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
out.m03 = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;

b0 = b.m04; b1 = b.m05; b2 = b.m06; b3 = b.m07;
out.m04 = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
out.m05 = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
out.m06 = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
out.m07 = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;

b0 = b.m08; b1 = b.m09; b2 = b.m10; b3 = b.m11;
out.m08 = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
out.m09 = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
out.m10 = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
out.m11 = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;

b0 = b.m12; b1 = b.m13; b2 = b.m14; b3 = b.m15;
out.m12 = b0 * a00 + b1 * a10 + b2 * a20 + b3 * a30;
out.m13 = b0 * a01 + b1 * a11 + b2 * a21 + b3 * a31;
out.m14 = b0 * a02 + b1 * a12 + b2 * a22 + b3 * a32;
out.m15 = b0 * a03 + b1 * a13 + b2 * a23 + b3 * a33;
return out;
};

/**
 * Alias of {@link mat4.multiply}.
 */
mat4.mul = function mul (out, a, b)
{
return mat4.multiply(out, a, b);
};

/**
 * Multiply a matrix with a translation matrix given by a translation offset.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {vec3} v - The translation offset.
 * @returns {mat4} out.
 */
mat4.translate = function translate (out, a, v)
{
var x = v.x, y = v.y, z = v.z,
    a00, a01, a02, a03,
    a10, a11, a12, a13,
    a20, a21, a22, a23;

if (a === out) {
  out.m12 = a.m00 * x + a.m04 * y + a.m08 * z + a.m12;
  out.m13 = a.m01 * x + a.m05 * y + a.m09 * z + a.m13;
  out.m14 = a.m02 * x + a.m06 * y + a.m10 * z + a.m14;
  out.m15 = a.m03 * x + a.m07 * y + a.m11 * z + a.m15;
} else {
  a00 = a.m00; a01 = a.m01; a02 = a.m02; a03 = a.m03;
  a10 = a.m04; a11 = a.m05; a12 = a.m06; a13 = a.m07;
  a20 = a.m08; a21 = a.m09; a22 = a.m10; a23 = a.m11;

  out.m00 = a00; out.m01 = a01; out.m02 = a02; out.m03 = a03;
  out.m04 = a10; out.m05 = a11; out.m06 = a12; out.m07 = a13;
  out.m08 = a20; out.m09 = a21; out.m10 = a22; out.m11 = a23;

  out.m12 = a00 * x + a10 * y + a20 * z + a.m12;
  out.m13 = a01 * x + a11 * y + a21 * z + a.m13;
  out.m14 = a02 * x + a12 * y + a22 * z + a.m14;
  out.m15 = a03 * x + a13 * y + a23 * z + a.m15;
}

return out;
};

/**
 * Multiply a matrix with a scale matrix given by a scale vector.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {vec3} v - The scale vector.
 * @returns {mat4} out
 **/
mat4.scale = function scale (out, a, v)
{
var x = v.x, y = v.y, z = v.z;

out.m00 = a.m00 * x;
out.m01 = a.m01 * x;
out.m02 = a.m02 * x;
out.m03 = a.m03 * x;
out.m04 = a.m04 * y;
out.m05 = a.m05 * y;
out.m06 = a.m06 * y;
out.m07 = a.m07 * y;
out.m08 = a.m08 * z;
out.m09 = a.m09 * z;
out.m10 = a.m10 * z;
out.m11 = a.m11 * z;
out.m12 = a.m12;
out.m13 = a.m13;
out.m14 = a.m14;
out.m15 = a.m15;
return out;
};

/**
 * Multiply a matrix with a rotation matrix denotes by the rotation around arbitrary axis.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {Number} rad - The rotation angle.
 * @param {vec3} axis - The rotation axis.
 * @returns {mat4} out.
 */
mat4.rotate = function rotate (out, a, rad, axis)
{
var x = axis.x, y = axis.y, z = axis.z;
var s, c, t,
    a00, a01, a02, a03,
    a10, a11, a12, a13,
    a20, a21, a22, a23,
    b00, b01, b02,
    b10, b11, b12,
    b20, b21, b22;

var len = Math.sqrt(x * x + y * y + z * z);

if (Math.abs(len) < EPSILON) {
  return null;
}

len = 1 / len;
x *= len;
y *= len;
z *= len;

s = Math.sin(rad);
c = Math.cos(rad);
t = 1 - c;

a00 = a.m00; a01 = a.m01; a02 = a.m02; a03 = a.m03;
a10 = a.m04; a11 = a.m05; a12 = a.m06; a13 = a.m07;
a20 = a.m08; a21 = a.m09; a22 = a.m10; a23 = a.m11;

// Construct the elements of the rotation matrix
b00 = x * x * t + c; b01 = y * x * t + z * s; b02 = z * x * t - y * s;
b10 = x * y * t - z * s; b11 = y * y * t + c; b12 = z * y * t + x * s;
b20 = x * z * t + y * s; b21 = y * z * t - x * s; b22 = z * z * t + c;

// Perform rotation-specific matrix multiplication
out.m00 = a00 * b00 + a10 * b01 + a20 * b02;
out.m01 = a01 * b00 + a11 * b01 + a21 * b02;
out.m02 = a02 * b00 + a12 * b01 + a22 * b02;
out.m03 = a03 * b00 + a13 * b01 + a23 * b02;
out.m04 = a00 * b10 + a10 * b11 + a20 * b12;
out.m05 = a01 * b10 + a11 * b11 + a21 * b12;
out.m06 = a02 * b10 + a12 * b11 + a22 * b12;
out.m07 = a03 * b10 + a13 * b11 + a23 * b12;
out.m08 = a00 * b20 + a10 * b21 + a20 * b22;
out.m09 = a01 * b20 + a11 * b21 + a21 * b22;
out.m10 = a02 * b20 + a12 * b21 + a22 * b22;
out.m11 = a03 * b20 + a13 * b21 + a23 * b22;

// If the source and destination differ, copy the unchanged last row
if (a !== out) {
  out.m12 = a.m12;
  out.m13 = a.m13;
  out.m14 = a.m14;
  out.m15 = a.m15;
}

return out;
};

/**
 * Multiply a matrix with a rotation matrix denotes by the rotation around x-axis.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.rotateX = function rotateX (out, a, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad),
    a10 = a.m04,
    a11 = a.m05,
    a12 = a.m06,
    a13 = a.m07,
    a20 = a.m08,
    a21 = a.m09,
    a22 = a.m10,
    a23 = a.m11;

if (a !== out) { // If the source and destination differ, copy the unchanged rows
  out.m00 = a.m00;
  out.m01 = a.m01;
  out.m02 = a.m02;
  out.m03 = a.m03;
  out.m12 = a.m12;
  out.m13 = a.m13;
  out.m14 = a.m14;
  out.m15 = a.m15;
}

// Perform axis-specific matrix multiplication
out.m04 = a10 * c + a20 * s;
out.m05 = a11 * c + a21 * s;
out.m06 = a12 * c + a22 * s;
out.m07 = a13 * c + a23 * s;
out.m08 = a20 * c - a10 * s;
out.m09 = a21 * c - a11 * s;
out.m10 = a22 * c - a12 * s;
out.m11 = a23 * c - a13 * s;

return out;
};

/**
 * Multiply a matrix with a rotation matrix denotes by the rotation around y-axis.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.rotateY = function rotateY (out, a, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad),
    a00 = a.m00,
    a01 = a.m01,
    a02 = a.m02,
    a03 = a.m03,
    a20 = a.m08,
    a21 = a.m09,
    a22 = a.m10,
    a23 = a.m11;

if (a !== out) { // If the source and destination differ, copy the unchanged rows
  out.m04 = a.m04;
  out.m05 = a.m05;
  out.m06 = a.m06;
  out.m07 = a.m07;
  out.m12 = a.m12;
  out.m13 = a.m13;
  out.m14 = a.m14;
  out.m15 = a.m15;
}

// Perform axis-specific matrix multiplication
out.m00 = a00 * c - a20 * s;
out.m01 = a01 * c - a21 * s;
out.m02 = a02 * c - a22 * s;
out.m03 = a03 * c - a23 * s;
out.m08 = a00 * s + a20 * c;
out.m09 = a01 * s + a21 * c;
out.m10 = a02 * s + a22 * c;
out.m11 = a03 * s + a23 * c;

return out;
};

/**
 * Multiply a matrix with a rotation matrix denotes by the rotation around z-axis.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to multiply.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.rotateZ = function rotateZ (out, a, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad),
    a00 = a.m00,
    a01 = a.m01,
    a02 = a.m02,
    a03 = a.m03,
    a10 = a.m04,
    a11 = a.m05,
    a12 = a.m06,
    a13 = a.m07;

// If the source and destination differ, copy the unchanged last row
if (a !== out) {
  out.m08 = a.m08;
  out.m09 = a.m09;
  out.m10 = a.m10;
  out.m11 = a.m11;
  out.m12 = a.m12;
  out.m13 = a.m13;
  out.m14 = a.m14;
  out.m15 = a.m15;
}

// Perform axis-specific matrix multiplication
out.m00 = a00 * c + a10 * s;
out.m01 = a01 * c + a11 * s;
out.m02 = a02 * c + a12 * s;
out.m03 = a03 * c + a13 * s;
out.m04 = a10 * c - a00 * s;
out.m05 = a11 * c - a01 * s;
out.m06 = a12 * c - a02 * s;
out.m07 = a13 * c - a03 * s;

return out;
};

/**
 * Create a translation matrix from a translation offset.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.translate(dest, dest, vec);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {vec3} v - The translation offset.
 * @returns {mat4} out.
 */
mat4.fromTranslation = function fromTranslation (out, v)
{
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = 1;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = 1;
out.m11 = 0;
out.m12 = v.x;
out.m13 = v.y;
out.m14 = v.z;
out.m15 = 1;
return out;
};

/**
 * Creates a scale matrix from a scale vector.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.scale(dest, dest, vec);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {vec3} v - The scale vector.
 * @returns {mat4} out.
 */
mat4.fromScaling = function fromScaling (out, v)
{
out.m00 = v.x;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = v.y;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = v.z;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Creates a rotation matrix from the rotation around arbitrary axis.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.rotate(dest, dest, rad, axis);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @param {vec3} axis - The rotation axis.
 * @returns {mat4} out.
 */
mat4.fromRotation = function fromRotation (out, rad, axis)
{
var x = axis.x, y = axis.y, z = axis.z;
var len = Math.sqrt(x * x + y * y + z * z);
var s, c, t;

if (Math.abs(len) < EPSILON) {
  return null;
}

len = 1 / len;
x *= len;
y *= len;
z *= len;

s = Math.sin(rad);
c = Math.cos(rad);
t = 1 - c;

// Perform rotation-specific matrix multiplication
out.m00 = x * x * t + c;
out.m01 = y * x * t + z * s;
out.m02 = z * x * t - y * s;
out.m03 = 0;
out.m04 = x * y * t - z * s;
out.m05 = y * y * t + c;
out.m06 = z * y * t + x * s;
out.m07 = 0;
out.m08 = x * z * t + y * s;
out.m09 = y * z * t - x * s;
out.m10 = z * z * t + c;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Creates a rotation matrix from the rotation around x-axis.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.rotateX(dest, dest, rad);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.fromXRotation = function fromXRotation (out, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad);

// Perform axis-specific matrix multiplication
out.m00 = 1;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = c;
out.m06 = s;
out.m07 = 0;
out.m08 = 0;
out.m09 = -s;
out.m10 = c;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Creates a rotation matrix from the rotation around y-axis.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.rotateY(dest, dest, rad);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.fromYRotation = function fromYRotation (out, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad);

// Perform axis-specific matrix multiplication
out.m00 = c;
out.m01 = 0;
out.m02 = -s;
out.m03 = 0;
out.m04 = 0;
out.m05 = 1;
out.m06 = 0;
out.m07 = 0;
out.m08 = s;
out.m09 = 0;
out.m10 = c;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Creates a rotation matrix from the rotation around z-axis.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.rotateZ(dest, dest, rad);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {Number} rad - The rotation angle.
 * @returns {mat4} out.
 */
mat4.fromZRotation = function fromZRotation (out, rad)
{
var s = Math.sin(rad),
    c = Math.cos(rad);

// Perform axis-specific matrix multiplication
out.m00 = c;
out.m01 = s;
out.m02 = 0;
out.m03 = 0;
out.m04 = -s;
out.m05 = c;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = 1;
out.m11 = 0;
out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;
return out;
};

/**
 * Creates a matrix from a quaternion rotation and a translation offset.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.translate(dest, vec);
 *   let quatMat = mat4.create();
 *   quat.toMat4(quat, quatMat);
 *   mat4.multiply(dest, quatMat);
 *
 * @param {mat4} out - Matrix to store result.
 * @param {quat} q - Rotation quaternion.
 * @param {vec3} v - Translation vector.
 * @returns {mat4} out.
 */
mat4.fromRT = function fromRT (out, q, v)
{
// Quaternion math
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var xy = x * y2;
var xz = x * z2;
var yy = y * y2;
var yz = y * z2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;

out.m00 = 1 - (yy + zz);
out.m01 = xy + wz;
out.m02 = xz - wy;
out.m03 = 0;
out.m04 = xy - wz;
out.m05 = 1 - (xx + zz);
out.m06 = yz + wx;
out.m07 = 0;
out.m08 = xz + wy;
out.m09 = yz - wx;
out.m10 = 1 - (xx + yy);
out.m11 = 0;
out.m12 = v.x;
out.m13 = v.y;
out.m14 = v.z;
out.m15 = 1;

return out;
};

/**
 * Returns the translation vector component of a transformation
 *matrix. If a matrix is built with fromRT,
 *the returned vector will be the same as the translation offset
 *originally supplied.
 * @param{vec3} out - Vector to store result.
 * @param{mat4} mat - Matrix to be decomposed.
 * @return {vec3} out.
 */
mat4.getTranslation = function getTranslation (out, mat)
{
out.x = mat.m12;
out.y = mat.m13;
out.z = mat.m14;

return out;
};

/**
 * Returns the scale component of a transformation
 *matrix. If a matrix is built with fromRTS
 *with a normalized Quaternion parameter, the returned vector will be
 *the same as the scale vector
 *originally supplied.
 * @param{vec3} out - Vector to store result.
 * @param{mat4} mat - Matrix to be decomposed.
 * @return {vec3} out.
 */
mat4.getScaling = function getScaling (out, mat)
{
var m11 = mat.m00,
    m12 = mat.m01,
    m13 = mat.m02,
    m21 = mat.m04,
    m22 = mat.m05,
    m23 = mat.m06,
    m31 = mat.m08,
    m32 = mat.m09,
    m33 = mat.m10;

out.x = Math.sqrt(m11 * m11 + m12 * m12 + m13 * m13);
out.y = Math.sqrt(m21 * m21 + m22 * m22 + m23 * m23);
out.z = Math.sqrt(m31 * m31 + m32 * m32 + m33 * m33);

return out;
};

/**
 * Returns a quaternion representing the rotational component
 *of a transformation matrix. If a matrix is built with
 *fromRT, the returned quaternion will be the
 *same as the quaternion originally supplied.
 * @param {quat} out - Quaternion to store result.
 * @param {mat4} mat - Matrix to be decomposed.
 * @return {quat} out.
 */
mat4.getRotation = function getRotation (out, mat)
{
// Algorithm taken from http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
var trace = mat.m00 + mat.m05 + mat.m10;
var S = 0;

if (trace > 0) {
  S = Math.sqrt(trace + 1.0) * 2;
  out.w = 0.25 * S;
  out.x = (mat.m06 - mat.m09) / S;
  out.y = (mat.m08 - mat.m02) / S;
  out.z = (mat.m01 - mat.m04) / S;
} else if ((mat.m00 > mat.m05) & (mat.m00 > mat.m10)) {
  S = Math.sqrt(1.0 + mat.m00 - mat.m05 - mat.m10) * 2;
  out.w = (mat.m06 - mat.m09) / S;
  out.x = 0.25 * S;
  out.y = (mat.m01 + mat.m04) / S;
  out.z = (mat.m08 + mat.m02) / S;
} else if (mat.m05 > mat.m10) {
  S = Math.sqrt(1.0 + mat.m05 - mat.m00 - mat.m10) * 2;
  out.w = (mat.m08 - mat.m02) / S;
  out.x = (mat.m01 + mat.m04) / S;
  out.y = 0.25 * S;
  out.z = (mat.m06 + mat.m09) / S;
} else {
  S = Math.sqrt(1.0 + mat.m10 - mat.m00 - mat.m05) * 2;
  out.w = (mat.m01 - mat.m04) / S;
  out.x = (mat.m08 + mat.m02) / S;
  out.y = (mat.m06 + mat.m09) / S;
  out.z = 0.25 * S;
}

return out;
};

/**
 * Creates a matrix from a quaternion rotation, translation offset and scale vector.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.translate(dest, vec);
 *   let quatMat = mat4.create();
 *   quat.toMat4(quat, quatMat);
 *   mat4.multiply(dest, quatMat);
 *   mat4.scale(dest, scale)
 *
 * @param {mat4} out mat4 - Matrix to store result.
 * @param {quat} q - Rotation quaternion.
 * @param {vec3} v - Translation offset.
 * @param {vec3} s - Scale vector.
 * @returns {mat4} out.
 */
mat4.fromRTS = function fromRTS (out, q, v, s)
{
// Quaternion math
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var xy = x * y2;
var xz = x * z2;
var yy = y * y2;
var yz = y * z2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;
var sx = s.x;
var sy = s.y;
var sz = s.z;

out.m00 = (1 - (yy + zz)) * sx;
out.m01 = (xy + wz) * sx;
out.m02 = (xz - wy) * sx;
out.m03 = 0;
out.m04 = (xy - wz) * sy;
out.m05 = (1 - (xx + zz)) * sy;
out.m06 = (yz + wx) * sy;
out.m07 = 0;
out.m08 = (xz + wy) * sz;
out.m09 = (yz - wx) * sz;
out.m10 = (1 - (xx + yy)) * sz;
out.m11 = 0;
out.m12 = v.x;
out.m13 = v.y;
out.m14 = v.z;
out.m15 = 1;

return out;
};

/**
 * Creates a matrix from a quaternion rotation, translation offset and scale vector, rotating and scaling around the given origin.
 * This is equivalent to (but much faster than):
 *
 *   mat4.identity(dest);
 *   mat4.translate(dest, vec);
 *   mat4.translate(dest, origin);
 *   let quatMat = mat4.create();
 *   quat.toMat4(quat, quatMat);
 *   mat4.multiply(dest, quatMat);
 *   mat4.scale(dest, scale)
 *   mat4.translate(dest, negativeOrigin);
 *
 * @param {mat4} out mat4 - Matrix to store result.
 * @param {quat} q - Rotation quaternion.
 * @param {vec3} v - Translation offset.
 * @param {vec3} s - Scale vector.
 * @param {vec3} o The origin vector around which to scale and rotate.
 * @returns {mat4} out.
 */
mat4.fromRTSOrigin = function fromRTSOrigin (out, q, v, s, o)
{
// Quaternion math
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var xy = x * y2;
var xz = x * z2;
var yy = y * y2;
var yz = y * z2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;

var sx = s.x;
var sy = s.y;
var sz = s.z;

var ox = o.x;
var oy = o.y;
var oz = o.z;

out.m00 = (1 - (yy + zz)) * sx;
out.m01 = (xy + wz) * sx;
out.m02 = (xz - wy) * sx;
out.m03 = 0;
out.m04 = (xy - wz) * sy;
out.m05 = (1 - (xx + zz)) * sy;
out.m06 = (yz + wx) * sy;
out.m07 = 0;
out.m08 = (xz + wy) * sz;
out.m09 = (yz - wx) * sz;
out.m10 = (1 - (xx + yy)) * sz;
out.m11 = 0;
out.m12 = v.x + ox - (out.m00 * ox + out.m04 * oy + out.m08 * oz);
out.m13 = v.y + oy - (out.m01 * ox + out.m05 * oy + out.m09 * oz);
out.m14 = v.z + oz - (out.m02 * ox + out.m06 * oy + out.m10 * oz);
out.m15 = 1;

return out;
};

/**
 * Calculates a 4x4 matrix from the given quaternion.
 *
 * @param {mat4} out mat4 - Matrix to store result.
 * @param {quat} q - Quaternion to create matrix from.
 *
 * @returns {mat4} out.
 */
mat4.fromQuat = function fromQuat (out, q)
{
var x = q.x, y = q.y, z = q.z, w = q.w;
var x2 = x + x;
var y2 = y + y;
var z2 = z + z;

var xx = x * x2;
var yx = y * x2;
var yy = y * y2;
var zx = z * x2;
var zy = z * y2;
var zz = z * z2;
var wx = w * x2;
var wy = w * y2;
var wz = w * z2;

out.m00 = 1 - yy - zz;
out.m01 = yx + wz;
out.m02 = zx - wy;
out.m03 = 0;

out.m04 = yx - wz;
out.m05 = 1 - xx - zz;
out.m06 = zy + wx;
out.m07 = 0;

out.m08 = zx + wy;
out.m09 = zy - wx;
out.m10 = 1 - xx - yy;
out.m11 = 0;

out.m12 = 0;
out.m13 = 0;
out.m14 = 0;
out.m15 = 1;

return out;
};

/**
 * Generates a frustum matrix with the given bounds.
 *
 * @param {mat4} out mat4 - Matrix to store result.
 * @param {Number} left - Left bound of the frustum.
 * @param {Number} right - Right bound of the frustum.
 * @param {Number} bottom - Bottom bound of the frustum.
 * @param {Number} top - Top bound of the frustum.
 * @param {Number} near - Near bound of the frustum.
 * @param {Number} far - Far bound of the frustum.
 * @returns {mat4} out.
 */
mat4.frustum = function frustum (out, left, right, bottom, top, near, far)
{
var rl = 1 / (right - left);
var tb = 1 / (top - bottom);
var nf = 1 / (near - far);

out.m00 = (near * 2) * rl;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = (near * 2) * tb;
out.m06 = 0;
out.m07 = 0;
out.m08 = (right + left) * rl;
out.m09 = (top + bottom) * tb;
out.m10 = (far + near) * nf;
out.m11 = -1;
out.m12 = 0;
out.m13 = 0;
out.m14 = (far * near * 2) * nf;
out.m15 = 0;
return out;
};

/**
 * Generates a perspective projection matrix with the given bounds.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {number} fovy - Vertical field of view in radians.
 * @param {number} aspect - Aspect ratio. typically viewport width/height.
 * @param {number} near - Near bound of the frustum.
 * @param {number} far - Far bound of the frustum.
 * @returns {mat4} out.
 */
mat4.perspective = function perspective (out, fovy, aspect, near, far)
{
var f = 1.0 / Math.tan(fovy / 2);
var nf = 1 / (near - far);

out.m00 = f / aspect;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = f;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = (far + near) * nf;
out.m11 = -1;
out.m12 = 0;
out.m13 = 0;
out.m14 = (2 * far * near) * nf;
out.m15 = 0;
return out;
};

/**
 * Generates a perspective projection matrix with the given field of view.
 * This is primarily useful for generating projection matrices to be used
 * with the still experiemental WebVR API.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {Object} fov - Object containing the following values: upDegrees, downDegrees, leftDegrees, rightDegrees.
 * @param {number} near - Near bound of the frustum.
 * @param {number} far - Far bound of the frustum.
 * @returns {mat4} out.
 */
mat4.perspectiveFromFieldOfView = function perspectiveFromFieldOfView (out, fov, near, far)
{
var upTan = Math.tan(fov.upDegrees * Math.PI / 180.0);
var downTan = Math.tan(fov.downDegrees * Math.PI / 180.0);
var leftTan = Math.tan(fov.leftDegrees * Math.PI / 180.0);
var rightTan = Math.tan(fov.rightDegrees * Math.PI / 180.0);
var xScale = 2.0 / (leftTan + rightTan);
var yScale = 2.0 / (upTan + downTan);

out.m00 = xScale;
out.m01 = 0.0;
out.m02 = 0.0;
out.m03 = 0.0;
out.m04 = 0.0;
out.m05 = yScale;
out.m06 = 0.0;
out.m07 = 0.0;
out.m08 = -((leftTan - rightTan) * xScale * 0.5);
out.m09 = ((upTan - downTan) * yScale * 0.5);
out.m10 = far / (near - far);
out.m11 = -1.0;
out.m12 = 0.0;
out.m13 = 0.0;
out.m14 = (far * near) / (near - far);
out.m15 = 0.0;
return out;
};

/**
 * Generates a orthogonal projection matrix with the given bounds.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {number} left - Left bound of the frustum.
 * @param {number} right - Right bound of the frustum.
 * @param {number} bottom - Bottom bound of the frustum.
 * @param {number} top - Top bound of the frustum.
 * @param {number} near - Near bound of the frustum.
 * @param {number} far - Far bound of the frustum.
 * @returns {mat4} out.
 */
mat4.ortho = function ortho (out, left, right, bottom, top, near, far)
{
var lr = 1 / (left - right);
var bt = 1 / (bottom - top);
var nf = 1 / (near - far);
out.m00 = -2 * lr;
out.m01 = 0;
out.m02 = 0;
out.m03 = 0;
out.m04 = 0;
out.m05 = -2 * bt;
out.m06 = 0;
out.m07 = 0;
out.m08 = 0;
out.m09 = 0;
out.m10 = 2 * nf;
out.m11 = 0;
out.m12 = (left + right) * lr;
out.m13 = (top + bottom) * bt;
out.m14 = (far + near) * nf;
out.m15 = 1;
return out;
};

/**
 * Generates a look-at matrix with the given eye position, focal point, and up axis.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {vec3} eye - Position of the viewer.
 * @param {vec3} center - Point the viewer is looking at.
 * @param {vec3} up - Vector pointing up.
 * @returns {mat4} out
 */
mat4.lookAt = function lookAt (out, eye, center, up)
{
var x0, x1, x2, y0, y1, y2, z0, z1, z2, len;
var eyex = eye.x;
var eyey = eye.y;
var eyez = eye.z;
var upx = up.x;
var upy = up.y;
var upz = up.z;
var centerx = center.x;
var centery = center.y;
var centerz = center.z;

if (
  Math.abs(eyex - centerx) < EPSILON &&
  Math.abs(eyey - centery) < EPSILON &&
  Math.abs(eyez - centerz) < EPSILON
) {
  return mat4.identity(out);
}

z0 = eyex - centerx;
z1 = eyey - centery;
z2 = eyez - centerz;

len = 1 / Math.sqrt(z0 * z0 + z1 * z1 + z2 * z2);
z0 *= len;
z1 *= len;
z2 *= len;

x0 = upy * z2 - upz * z1;
x1 = upz * z0 - upx * z2;
x2 = upx * z1 - upy * z0;
len = Math.sqrt(x0 * x0 + x1 * x1 + x2 * x2);
if (!len) {
  x0 = 0;
  x1 = 0;
  x2 = 0;
} else {
  len = 1 / len;
  x0 *= len;
  x1 *= len;
  x2 *= len;
}

y0 = z1 * x2 - z2 * x1;
y1 = z2 * x0 - z0 * x2;
y2 = z0 * x1 - z1 * x0;

len = Math.sqrt(y0 * y0 + y1 * y1 + y2 * y2);
if (!len) {
  y0 = 0;
  y1 = 0;
  y2 = 0;
} else {
  len = 1 / len;
  y0 *= len;
  y1 *= len;
  y2 *= len;
}

out.m00 = x0;
out.m01 = y0;
out.m02 = z0;
out.m03 = 0;
out.m04 = x1;
out.m05 = y1;
out.m06 = z1;
out.m07 = 0;
out.m08 = x2;
out.m09 = y2;
out.m10 = z2;
out.m11 = 0;
out.m12 = -(x0 * eyex + x1 * eyey + x2 * eyez);
out.m13 = -(y0 * eyex + y1 * eyey + y2 * eyez);
out.m14 = -(z0 * eyex + z1 * eyey + z2 * eyez);
out.m15 = 1;

return out;
};

/**
 * Returns a string representation of a matrix.
 *
 * @param {mat4} a - The matrix.
 * @returns {String} String representation of this matrix.
 */
mat4.str = function str (a)
{
  return ("mat4(" + (a.m00) + ", " + (a.m01) + ", " + (a.m02) + ", " + (a.m03) + ", " + (a.m04) + ", " + (a.m05) + ", " + (a.m06) + ", " + (a.m07) + ", " + (a.m08) + ", " + (a.m09) + ", " + (a.m10) + ", " + (a.m11) + ", " + (a.m12) + ", " + (a.m13) + ", " + (a.m14) + ", " + (a.m15) + ")");
};

/**
 * Store elements of a matrix into array.
 *
 * @param {array} out - Array to store result.
 * @param {mat4} m - The matrix.
 * @returns {Array} out.
 */
mat4.array = function array (out, m)
{
out[0] = m.m00;
out[1] = m.m01;
out[2] = m.m02;
out[3] = m.m03;
out[4] = m.m04;
out[5] = m.m05;
out[6] = m.m06;
out[7] = m.m07;
out[8] = m.m08;
out[9] = m.m09;
out[10] = m.m10;
out[11] = m.m11;
out[12] = m.m12;
out[13] = m.m13;
out[14] = m.m14;
out[15] = m.m15;

return out;
};

/**
 * Returns Frobenius norm of a matrix.
 *
 * @param {mat4} a - Matrix to calculate Frobenius norm of.
 * @returns {Number} - The frobenius norm.
 */
mat4.frob = function frob (a)
{
return (Math.sqrt(Math.pow(a.m00, 2) + Math.pow(a.m01, 2) + Math.pow(a.m02, 2) + Math.pow(a.m03, 2) + Math.pow(a.m04, 2) + Math.pow(a.m05, 2) + Math.pow(a.m06, 2) + Math.pow(a.m07, 2) + Math.pow(a.m08, 2) + Math.pow(a.m09, 2) + Math.pow(a.m10, 2) + Math.pow(a.m11, 2) + Math.pow(a.m12, 2) + Math.pow(a.m13, 2) + Math.pow(a.m14, 2) + Math.pow(a.m15, 2)))
};

/**
 * Adds two matrices.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - The first operand.
 * @param {mat4} b - The second operand.
 * @returns {mat4} out.
 */
mat4.add = function add (out, a, b)
{
out.m00 = a.m00 + b.m00;
out.m01 = a.m01 + b.m01;
out.m02 = a.m02 + b.m02;
out.m03 = a.m03 + b.m03;
out.m04 = a.m04 + b.m04;
out.m05 = a.m05 + b.m05;
out.m06 = a.m06 + b.m06;
out.m07 = a.m07 + b.m07;
out.m08 = a.m08 + b.m08;
out.m09 = a.m09 + b.m09;
out.m10 = a.m10 + b.m10;
out.m11 = a.m11 + b.m11;
out.m12 = a.m12 + b.m12;
out.m13 = a.m13 + b.m13;
out.m14 = a.m14 + b.m14;
out.m15 = a.m15 + b.m15;
return out;
};

/**
 * Subtracts matrix b from matrix a.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - The first operand.
 * @param {mat4} b - The second operand.
 * @returns {mat4} out.
 */
mat4.subtract = function subtract (out, a, b)
{
out.m00 = a.m00 - b.m00;
out.m01 = a.m01 - b.m01;
out.m02 = a.m02 - b.m02;
out.m03 = a.m03 - b.m03;
out.m04 = a.m04 - b.m04;
out.m05 = a.m05 - b.m05;
out.m06 = a.m06 - b.m06;
out.m07 = a.m07 - b.m07;
out.m08 = a.m08 - b.m08;
out.m09 = a.m09 - b.m09;
out.m10 = a.m10 - b.m10;
out.m11 = a.m11 - b.m11;
out.m12 = a.m12 - b.m12;
out.m13 = a.m13 - b.m13;
out.m14 = a.m14 - b.m14;
out.m15 = a.m15 - b.m15;
return out;
};

/**
 * Alias of {@link mat4.subtract}.
 */
mat4.sub = function sub (out, a, b)
{
return mat4.subtract(out, a, b);
};

/**
 * Multiply each element of a matrix by a scalar number.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - Matrix to scale
 * @param {Number} b - The scale number.
 * @returns {mat4} out.
 */
mat4.multiplyScalar = function multiplyScalar (out, a, b)
{
out.m00 = a.m00 * b;
out.m01 = a.m01 * b;
out.m02 = a.m02 * b;
out.m03 = a.m03 * b;
out.m04 = a.m04 * b;
out.m05 = a.m05 * b;
out.m06 = a.m06 * b;
out.m07 = a.m07 * b;
out.m08 = a.m08 * b;
out.m09 = a.m09 * b;
out.m10 = a.m10 * b;
out.m11 = a.m11 * b;
out.m12 = a.m12 * b;
out.m13 = a.m13 * b;
out.m14 = a.m14 * b;
out.m15 = a.m15 * b;
return out;
};

/**
 * Adds two matrices after multiplying each element of the second operand by a scalar number.
 *
 * @param {mat4} out - Matrix to store result.
 * @param {mat4} a - The first operand.
 * @param {mat4} b - The second operand.
 * @param {Number} scale - The scale number.
 * @returns {mat4} out.
 */
mat4.multiplyScalarAndAdd = function multiplyScalarAndAdd (out, a, b, scale)
{
out.m00 = a.m00 + (b.m00 * scale);
out.m01 = a.m01 + (b.m01 * scale);
out.m02 = a.m02 + (b.m02 * scale);
out.m03 = a.m03 + (b.m03 * scale);
out.m04 = a.m04 + (b.m04 * scale);
out.m05 = a.m05 + (b.m05 * scale);
out.m06 = a.m06 + (b.m06 * scale);
out.m07 = a.m07 + (b.m07 * scale);
out.m08 = a.m08 + (b.m08 * scale);
out.m09 = a.m09 + (b.m09 * scale);
out.m10 = a.m10 + (b.m10 * scale);
out.m11 = a.m11 + (b.m11 * scale);
out.m12 = a.m12 + (b.m12 * scale);
out.m13 = a.m13 + (b.m13 * scale);
out.m14 = a.m14 + (b.m14 * scale);
out.m15 = a.m15 + (b.m15 * scale);
return out;
};

/**
 * Returns whether the specified matrices are equal. (Compared using ===)
 *
 * @param {mat4} a - The first matrix.
 * @param {mat4} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat4.exactEquals = function exactEquals (a, b)
{
return a.m00 === b.m00 && a.m01 === b.m01 && a.m02 === b.m02 && a.m03 === b.m03 &&
  a.m04 === b.m04 && a.m05 === b.m05 && a.m06 === b.m06 && a.m07 === b.m07 &&
  a.m08 === b.m08 && a.m09 === b.m09 && a.m10 === b.m10 && a.m11 === b.m11 &&
  a.m12 === b.m12 && a.m13 === b.m13 && a.m14 === b.m14 && a.m15 === b.m15;
};

/**
 * Returns whether the specified matrices are approximately equal.
 *
 * @param {mat4} a - The first matrix.
 * @param {mat4} b - The second matrix.
 * @returns {Boolean} True if the matrices are equal, false otherwise.
 */
mat4.equals = function equals$$1 (a, b)
{
var a0 = a.m00, a1 = a.m01, a2 = a.m02, a3 = a.m03,
    a4 = a.m04, a5 = a.m05, a6 = a.m06, a7 = a.m07,
    a8 = a.m08, a9 = a.m09, a10 = a.m10, a11 = a.m11,
    a12 = a.m12, a13 = a.m13, a14 = a.m14, a15 = a.m15;

var b0 = b.m00, b1 = b.m01, b2 = b.m02, b3 = b.m03,
    b4 = b.m04, b5 = b.m05, b6 = b.m06, b7 = b.m07,
    b8 = b.m08, b9 = b.m09, b10 = b.m10, b11 = b.m11,
    b12 = b.m12, b13 = b.m13, b14 = b.m14, b15 = b.m15;

return (
  Math.abs(a0 - b0) <= EPSILON * Math.max(1.0, Math.abs(a0), Math.abs(b0)) &&
  Math.abs(a1 - b1) <= EPSILON * Math.max(1.0, Math.abs(a1), Math.abs(b1)) &&
  Math.abs(a2 - b2) <= EPSILON * Math.max(1.0, Math.abs(a2), Math.abs(b2)) &&
  Math.abs(a3 - b3) <= EPSILON * Math.max(1.0, Math.abs(a3), Math.abs(b3)) &&
  Math.abs(a4 - b4) <= EPSILON * Math.max(1.0, Math.abs(a4), Math.abs(b4)) &&
  Math.abs(a5 - b5) <= EPSILON * Math.max(1.0, Math.abs(a5), Math.abs(b5)) &&
  Math.abs(a6 - b6) <= EPSILON * Math.max(1.0, Math.abs(a6), Math.abs(b6)) &&
  Math.abs(a7 - b7) <= EPSILON * Math.max(1.0, Math.abs(a7), Math.abs(b7)) &&
  Math.abs(a8 - b8) <= EPSILON * Math.max(1.0, Math.abs(a8), Math.abs(b8)) &&
  Math.abs(a9 - b9) <= EPSILON * Math.max(1.0, Math.abs(a9), Math.abs(b9)) &&
  Math.abs(a10 - b10) <= EPSILON * Math.max(1.0, Math.abs(a10), Math.abs(b10)) &&
  Math.abs(a11 - b11) <= EPSILON * Math.max(1.0, Math.abs(a11), Math.abs(b11)) &&
  Math.abs(a12 - b12) <= EPSILON * Math.max(1.0, Math.abs(a12), Math.abs(b12)) &&
  Math.abs(a13 - b13) <= EPSILON * Math.max(1.0, Math.abs(a13), Math.abs(b13)) &&
  Math.abs(a14 - b14) <= EPSILON * Math.max(1.0, Math.abs(a14), Math.abs(b14)) &&
  Math.abs(a15 - b15) <= EPSILON * Math.max(1.0, Math.abs(a15), Math.abs(b15))
);
};

// NOTE: there is no syntax for: export {* as bits} from './lib/bits';
/**
 * Export module bits.
 */

/**
 * Returns a random v4 UUID of the form xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx, 
 * where each x is replaced with a random hexadecimal digit from 0 to f, 
 * and y is replaced with a random hexadecimal digit from 8 to b.
 * @param {?number} a the placeholder
 * @returns {number} resulting UUID
 * @see https://gist.github.com/jed/982883
 */
function uuid(a) {
  return a           // if the placeholder was passed, return
    ? (              // a random number from 0 to 15
      a ^            // unless b is 8,
      Math.random()  // in which case
      * 16           // a random number from
      >> a / 4       // 8 to 11
    ).toString(16)   // in hexadecimal
    : (              // or otherwise a concatenated string:
      [1e7] +        // 10000000 +
      -1e3 +         // -1000 +
      -4e3 +         // -4000 +
      -8e3 +         // -80000000 +
      -1e11          // -100000000000,
    ).replace(       // replacing
      /[018]/g,      // zeroes, ones, and eights with
      uuid           // random hex digits
    );
}

/**
 * Entity layer system
 */
var Layers = function Layers () {};

Layers.addLayer = function addLayer (name) {
  if (Layers._nextAvailable > 31) 
    { return new Error('maximum layers reached.'); }
  Layers[name] = (1 << Layers._nextAvailable++);
  return Layers[name];
};

/**
 * Make a layer mask accepting nothing but the listed layers
 * @param {number[]} includes layers accepted by the mask
 * @return {number} the specified layer mask
 */
Layers.makeInclusiveMask = function makeInclusiveMask (includes) {
  var mask = 0;
  for (var i = 0; i < includes.length; i++)
    { mask |= includes[i]; }
  return mask;
};

/**
 * Make a layer mask accepting everything but the listed layers
 * @param {number[]} excludes layers rejected by the mask
 * @return {number} the specified layer mask
 */
Layers.makeExclusiveMask = function makeExclusiveMask (excludes) {
  return ~Layers.makeInclusiveMask(excludes);
};

/**
 * Check a layer is accepted by the mask or not
 * @param {number} layer the layer number to be tested
 * @param {number} mask the testing layer mask
 * @return {boolean} true if accepted
 */
Layers.check = function check (layer, mask) {
  return (layer & mask) == layer;
};

Layers._nextAvailable = 8;

// built-in layers, reserved up to (1 << 7)
Layers.Default =  (1 << 0);
Layers.IgnoreRaycast =  (1 << 1);

// masks
Layers.RaycastMask = Layers.makeExclusiveMask([Layers.IgnoreRaycast]);

var v3_a = vec3.zero();
var q_a = quat.create();
var m3_a = mat3.create();
var m3_b = mat3.create();
var m4_a = mat4.create();

var Node = function Node(name) {
  this._id = uuid();
  this._parent = null;
  this._children = [];

  this.name = name || '';
  this.lpos = vec3.new(0, 0, 0);
  this.lscale = vec3.new(1, 1, 1);
  this.lrot = quat.new(0, 0, 0, 1);
  this.layer = Layers.Default;
};

var prototypeAccessors = { id: { configurable: true },parent: { configurable: true },children: { configurable: true } };

/**
 * @property {number} id
 * @readonly
 */
Node.mixin = function mixin (cls) {
  Object.getOwnPropertyNames(Node.prototype).forEach(function (name) {
    if (cls.prototype.hasOwnProperty(name) === false) {
      Object.defineProperty(
        cls.prototype,
        name,
        Object.getOwnPropertyDescriptor(Node.prototype, name)
      );
    }
  });
  cls.prototype.__initNode = function () {
    this._id = uuid();
    this._parent = null;
    this._children = [];

    this.name = '';
    this.lpos = vec3.new(0, 0, 0);
    this.lscale = vec3.new(1, 1, 1);
    this.lrot = quat.new(0, 0, 0, 1);
    this.layer = Layers.Default;
  };
};

prototypeAccessors.id.get = function () {
  return this._id;
};

/**
 * @property parent
 * @readonly
 *
 * get parent
 */
prototypeAccessors.parent.get = function () {
  return this._parent;
};

/**
 * @property children
 *
 * get children
 */
prototypeAccessors.children.get = function () {
  return this._children;
};

// ===============================
// hierarchy
// ===============================

/**
 * @method setParent
 * @param {Node} newParent
 * @return {boolean}
 *
 * NOTE: This function will invoke `_onParentChanged` if it exists.
 */
Node.prototype.setParent = function setParent (newParent) {
    var this$1 = this;

  var oldParent = this._parent;

  // newParent is the current parent of this
  if (oldParent === newParent) {
    return false;
  }

  // make sure the newParent is not a child of this
  var cur = newParent;
  while (cur) {
    if (cur === this$1) {
      // console.warn(`Failed to set parent for ${this.name}: the new parent ${newParent.name} is its child.`);
      return false;
    }

    cur = cur._parent;
  }

  // remove this from its old parent (if it has)
  if (oldParent) {
    var len = oldParent._children.length;
    for (var i = 0; i < len; ++i) {
      if (oldParent._children[i] === this$1) {
        oldParent._children.splice(i, 1);
        break;
      }
    }
  }

  // append it to the new parent
  this._parent = newParent;
  if (newParent) {
    newParent._children.push(this);
  }

  // invoke _onParentChanged
  if (this._onParentChanged) {
    this._onParentChanged(oldParent, newParent);
  }

  return true;
};

/**
 * @method insertAt
 * @param {number} idx
 * @param {Node} node
 *
 * Insert `node` before the `idx` of children.
 * NOTE: This function will invoke `_onParentChanged` if it exists.
 */
Node.prototype.insertAt = function insertAt (idx, node) {
    var this$1 = this;

  if (!node) {
    return false;
  }

  // make sure the node is not the parent of this
  var cur = this;
  while (cur) {
    if (cur === node) {
      // console.warn(`Failed to append ${node.name}: it is the ancient of current node.`);
      return false;
    }

    cur = cur._parent;
  }

  var oldParent = node._parent;

  // 0 <= idx <= len
  var len = this._children.length;
  idx = idx < 0 ? 0 : idx > len ? len : idx;

  // remove node from its current parent
  if (oldParent) {
    len = oldParent._children.length;
    for (var i = 0; i < len; ++i) {
      if (oldParent._children[i] === node) {
        // if we already have the child
        if (oldParent === this$1) {
          // if its insert position didn't changed, don't do anything.
          if (i === idx || i === idx - 1) {
            return false;
          }

          if (i < idx - 1) {
            idx = idx - 1;
          }
        }

        oldParent._children.splice(i, 1);
        break;
      }
    }
  }

  // append the new node
  node._parent = this;
  this._children.splice(idx, 0, node);

  // invoke _onParentChanged
  if (node._onParentChanged && node._parent !== this) {
    node._onParentChanged(oldParent, this);
  }

  return true;
};

/**
 * @method append
 * @param {Node} node
 * @return {boolean}
 *
 * Append `node` at the end of children.
 * NOTE: This function will invoke `_onParentChanged` if it exists.
 */
Node.prototype.append = function append (node) {
    var this$1 = this;

  if (!node) {
    return false;
  }

  // make sure the node is not the parent of this
  var cur = this;
  while (cur) {
    if (cur === node) {
      // console.warn(`Failed to append ${node.name}: it is the ancient of current node.`);
      return false;
    }

    cur = cur._parent;
  }

  var oldParent = node._parent;

  // remove node from its current parent
  if (oldParent) {
    var len = oldParent._children.length;
    for (var i = 0; i < len; ++i) {
      if (oldParent._children[i] === node) {
        // if we already have the child and its insert position didn't changed, don't do anything.
        if (oldParent === this$1 && i === len - 1) {
          return false;
        }

        oldParent._children.splice(i, 1);
        break;
      }
    }
  }

  // append the new node
  node._parent = this;
  this._children.push(node);

  // invoke _onParentChanged
  if (node._onParentChanged && node._parent !== this) {
    node._onParentChanged(oldParent, this);
  }

  return true;
};

/**
 * @method remove
 *
 * Remove self from parent
 * NOTE: This function will invoke `_onParentChanged` if it exists.
 */
Node.prototype.remove = function remove () {
  if (this._parent) {
    this._parent.removeChild(this);
  }
};

/**
 * @method removeChild
 * @param {Node} node
 *
 * Remove child
 * NOTE: This function will invoke `_onParentChanged` if it exists.
 */
Node.prototype.removeChild = function removeChild (node) {
    var this$1 = this;

  var len = this._children.length;

  for (var i = 0; i < len; ++i) {
    if (this$1._children[i] === node) {
      this$1._children.splice(i, 1);
      node._parent = null;

      // invoke _onParentChanged
      if (node._onParentChanged) {
        node._onParentChanged(this$1, null);
      }

      return true;
    }
  }

  // console.warn(`Failed to remove node ${node.name}, can not find it.`);
  return false;
};

// ===============================
// transform helper
// ===============================

/**
 * @method getWorldPos
 * @param {vec3} pos
 * @param {vec3} [up] - default is (0,1,0)
 * @return {vec3}
 *
 * Set rotation by lookAt target point
 */
Node.prototype.lookAt = function lookAt (pos, up) {
  this.getWorldPos(v3_a);
  vec3.sub(v3_a, v3_a, pos); // NOTE: we use -z for view-dir
  vec3.normalize(v3_a, v3_a);
  quat.fromViewUp(q_a, v3_a, up);

  this.setWorldRot(q_a);
};

// ===============================
// transform
// ===============================

/**
 * @method getWorldPos
 * @param {vec3} out
 * @return {vec3}
 *
 * Calculate and return world position
 */
Node.prototype.getWorldPos = function getWorldPos (out) {
  vec3.copy(out, this.lpos);

  var cur = this._parent;
  while (cur) {
    // out = parent_lscale * lpos
    vec3.mul(out, out, cur.lscale);

    // out = parent_lrot * out
    vec3.transformQuat(out, out, cur.lrot);

    // out = out + lpos
    vec3.add(out, out, cur.lpos);

    cur = cur._parent;
  }

  return out;
};

/**
 * @method setWorldPos
 * @param {vec3} pos
 *
 * Set world position
 */
Node.prototype.setWorldPos = function setWorldPos (pos) {
  // NOTE: this is faster than invert world matrix and transform the point

  if (this._parent) {
    this._parent.invTransformPoint(this.lpos, pos);
    return;
  }

  vec3.copy(this.lpos, pos);
};

/**
 * @method getWorldRot
 * @param {quat} out
 * @return {quat}
 *
 * Calculate and return world rotation
 */
Node.prototype.getWorldRot = function getWorldRot (out) {
  quat.copy(out, this.lrot);

  // result = ... * parent.parent.lrot * parent.lrot * lrot
  var cur = this._parent;
  while (cur) {
    quat.mul(out, cur.lrot, out);

    cur = cur._parent;
  }

  return out;
};

/**
 * @method setWorldRot
 * @param {quat} rot
 *
 * Set world position
 */
Node.prototype.setWorldRot = function setWorldRot (rot) {
  // lrot = rot * inv(prarent.wrot);
  if (this._parent) {
    this._parent.getWorldRot(this.lrot);
    quat.conjugate(this.lrot, this.lrot);
    quat.mul(this.lrot, this.lrot, rot);

    return;
  }

  quat.copy(this.lrot, rot);
};

/**
 * @method getWorldScale
 * @param {quat} out
 * @return {quat}
 *
 * Calculate and return world rotation
 */
Node.prototype.getWorldScale = function getWorldScale (out) {
  // invRot = inv(world_rot)
  this.getWorldRot(q_a);
  quat.conjugate(q_a, q_a);
  mat3.fromQuat(out, q_a);

  // worldRotScale
  this.getWorldRS(m3_a);

  // invRot * worldRotScale
  mat3.mul(out, out, m3_a);

  return out;
};

/**
 * @method invTransformPoint
 * @param {vec3} out
 * @param {vec3} pos
 *
 * Transforms position from world space to local space.
 */
Node.prototype.invTransformPoint = function invTransformPoint (out, pos) {
  if (this._parent) {
    this._parent.invTransformPoint(out, pos);
  } else {
    vec3.copy(out, pos);
  }

  // out = parent_inv_pos - lpos
  vec3.sub(out, out, this.lpos);

  // out = inv(lrot) * out
  quat.conjugate(q_a, this.lrot);
  vec3.transformQuat(out, out, q_a);

  // out = (1/scale) * out
  vec3.inverseSafe(v3_a, this.lscale);
  vec3.mul(out, out, v3_a);

  return out;
};

/**
 * @method getLocalMatrix
 * @param {mat4} out
 * @return {mat4}
 *
 * Calculate and return local transform
 */
Node.prototype.getLocalMatrix = function getLocalMatrix (out) {
  mat4.fromRTS(out, this.lrot, this.lpos, this.lscale);
  return out;
};

/**
 * @method getWorldMatrix
 * @param {mat4} out
 * @return {mat4}
 *
 * Calculate and return world transform
 */
Node.prototype.getWorldMatrix = function getWorldMatrix (out) {
  // out = ... * parent.parent.local * parent.local * local;
  this.getLocalMatrix(out);

  var cur = this._parent;
  while (cur) {
    cur.getLocalMatrix(m4_a);
    mat4.mul(out, m4_a, out);

    cur = cur._parent;
  }

  return out;
};

/**
 * @method getWorldRT
 * @param {mat4} out
 * @return {mat4}
 *
 * Calculate and return world transform without scale
 */
Node.prototype.getWorldRT = function getWorldRT (out) {
  this._getWorldPosAndRot(v3_a, q_a);
  mat4.fromRT(out, q_a, v3_a);

  return out;
};

/**
 * @method getWorldRS
 * @param {mat3} out
 *
 * Calculate and return world rotation and scale matrix
 */
Node.prototype.getWorldRS = function getWorldRS (out) {
  mat3.set(m3_a,
    this.lscale.x, 0, 0,
    0, this.lscale.y, 0,
    0, 0, this.lscale.z
  );
  mat3.fromQuat(m3_b, this.lrot);

  if (this._parent) {
    // parent_RS * rot * scale
    this._parent.getWorldRS(out);
    mat3.mul(out, out, m3_b);
    mat3.mul(out, out, m3_a);
  } else {
    // rot * scale
    mat3.mul(out, m3_b, m3_a);
  }

  return out;
};

/**
 * @method _getWorldPosAndRot
 * @param {vec3} opos
 * @param {vec3} orot
 *
 * Calculate and return world position
 */
Node.prototype._getWorldPosAndRot = function _getWorldPosAndRot (opos, orot) {
  vec3.copy(opos, this.lpos);
  quat.copy(orot, this.lrot);

  var cur = this._parent;
  while (cur) {
    // opos = parent_lscale * lpos
    vec3.mul(opos, opos, cur.lscale);

    // opos = parent_lrot * opos
    vec3.transformQuat(opos, opos, cur.lrot);

    // opos = opos + lpos
    vec3.add(opos, opos, cur.lpos);

    // orot = lrot * orot
    quat.mul(orot, cur.lrot, orot);

    cur = cur._parent;
  }
};

Object.defineProperties( Node.prototype, prototypeAccessors );

/**
 * @method walk
 * @param {Node} node
 * @param {function(node: Node, parent: Node, level: number): boolean} fn
 * @param {number} level
 */
function walk(node, fn, level) {
  if ( level === void 0 ) level = 0;

  level += 1;
  var len = node.children.length;

  for (var i = 0; i < len; ++i) {
    var child = node.children[i];
    var continueWalk = fn(child, node, level);

    if (continueWalk === false) {
      break;
    }

    walk(child, fn, level);
  }
}

/**
 * fn1 is used when entering the node, fn2 is used when leaving the node
 * @param {Node} node
 * @param {function(node: Node, parent: Node, level: number): boolean} fn1
 * @param {function(node: Node, parent: Node, level: number)} fn2
 * @param {number} level
 */
function walk2(node, fn1, fn2, level) {
  if ( level === void 0 ) level = 0;

  level += 1;
  var len = node.children.length;

  for (var i = 0; i < len; ++i) {
    var child = node.children[i];
    var continueWalk = fn1(child, node, level);

    if (continueWalk === false) {
      fn2(child, node, level);
      break;
    }

    walk2(child, fn1, fn2, level);
    fn2(child, node, level);
  }
}

/**
 * walkSibling is almost same as walk except when the fn return false,
 * it just break walk in children but not break walk in sibiling
 *
 * @method walkSibling
 * @param {Node} node
 * @param {function(node: Node, parent: Node, level: number): boolean} fn
 * @param {number} level
 */
function walkSibling(node, fn, level) {
  if ( level === void 0 ) level = 0;

  level += 1;
  var len = node.children.length;

  for (var i = 0; i < len; ++i) {
    var child = node.children[i];
    var continueWalk = fn(child, node, level);

    if (continueWalk !== false) {
      walkSibling(child, fn, level);
    }
  }
}

/**
 * @method flat
 * @param {Node} node
 */
function flat(node) {
  var out = [];

  out.push(node);
  walk(node, function (iter) {
    out.push(iter);
  });

  return out;
}

/**
 * @method replace
 * @param {Node} oldNode
 * @param {Node} newNode
 */
function replace(oldNode, newNode) {
  newNode.remove();

  var parent = oldNode._parent;
  if (!parent) {
    return;
  }

  oldNode._parent = null;
  newNode._parent = parent;

  var len = parent._children.length;
  for (var i = 0; i < len; ++i) {
    if (parent._children[i] === oldNode) {
      parent._children[i] = newNode;
      return;
    }
  }
}

/**
 * @method clone
 * @param {Node} node
 * @param {function} ctor
 * @param {function} fn
 * @return {Node}
 */
function clone(node, ctor, fn) {
  if ( ctor === void 0 ) ctor = Node;
  if ( fn === void 0 ) fn = null;

  var newNode = new ctor();
  newNode.name = node.name;
  vec3.copy(newNode.lpos, node.lpos);
  vec3.copy(newNode.lscale, node.lscale);
  quat.copy(newNode.lrot, node.lrot);

  // do user custom clone function
  if (fn) {
    fn(newNode, node);
  }

  return newNode;
}

/**
 * @method deepClone
 * @param {Node} node
 * @param {function} ctor
 * @param {function} fn
 * @return {Node}
 */
function deepClone(node, ctor, fn) {
  if ( ctor === void 0 ) ctor = Node;
  if ( fn === void 0 ) fn = null;

  var newNode = clone(node, ctor, fn);

  newNode._children = new Array(node._children.length);
  for (var i = 0; i < node._children.length; ++i) {
    var child = node._children[i];
    var newChild = deepClone(child, ctor, fn);
    newNode._children[i] = newChild;
    newChild._parent = newNode;
  }

  return newNode;
}

/**
 * @method find
 * @param {Node} root
 * @param {string} path
 */
function find(root, path) {
  var names = path.split('/');

  function _recurse(node, level) {
    var len = node.children.length;
    var name = names[level];

    for (var i = 0; i < len; ++i) {
      var child = node.children[i];

      if (child.name !== name) {
        continue;
      }

      if (level === names.length - 1) {
        return child;
      } else {
        return _recurse(child, level + 1);
      }
    }

    return null;
  }

  return _recurse(root, 0);
}

var utils = {
  walk: walk,
  walk2: walk2,
  walkSibling: walkSibling,
  flat: flat,
  replace: replace,
  clone: clone,
  deepClone: deepClone,
  find: find,
};

exports.Node = Node;
exports.utils = utils;
exports.Layers = Layers;

return exports;

}({}));
//# sourceMappingURL=scene-graph.dev.js.map
